@@ -3,7 +3,7 @@
 
 #include <antlr/config.hpp>
 #include "GDLInterpreterTokenTypes.hpp"
-/* $ANTLR 2.7.7 (20110618): "gdlc.i.g" -> "GDLInterpreter.hpp"$ */
+/* $ANTLR 2.7.7 (20120518): "gdlc.i.g" -> "GDLInterpreter.hpp"$ */
 #include <antlr/TreeParser.hpp>
 
 
@@ -116,8 +116,11 @@ public:
     static int GetFunIx( const std::string& subName);
     static int GetProIx( ProgNodeP);//const std::string& subName);
     static int GetProIx( const std::string& subName);
-    DStructGDL* ObjectStruct( BaseGDL* self, ProgNodeP mp);
-    DStructGDL* ObjectStructCheckAccess( BaseGDL* self, ProgNodeP mp);
+    DStructGDL* ObjectStruct( DObjGDL* self, ProgNodeP mp);
+    void SetRootR( ProgNodeP tt, DotAccessDescT* aD, BaseGDL* r, ArrayIndexListT* aL);
+    void SetRootL( ProgNodeP tt, DotAccessDescT* aD, BaseGDL* r, ArrayIndexListT* aL);
+    // DStructGDL* ObjectStructCheckAccess( DObjGDL* self, ProgNodeP mp);
+    // DStructDesc* GDLObjectDesc( DObjGDL* self, ProgNodeP mp);
 
     // code in: dinterpreter.cpp
     static void SetFunIx( ProgNodeP f); // triggers read/compile
@@ -145,8 +148,10 @@ protected:
     {
         public:
         enum ExCode {
-            NONE=0, // normal RETALL
-            RUN     // RETALL from .RUN command
+            NONE=0 // normal RETALL
+            ,RUN   // RETALL from .RUN command
+            ,RESET // RETALL from .RESET command
+            ,FULL_RESET // RETALL from .FULL_RESET command
         };  
 
         private:
@@ -234,6 +239,14 @@ public:
             std::pair<SizeT, RefBaseGDL>( heapIx++, var));
         return tmpIx;
     }
+    static void FreeObjHeapDirect( DObj id, ObjHeapT::iterator it)
+    {
+        BaseGDL* del = (*it).second.get();
+        objHeap.erase( id); 
+        delete del;
+        // delete (*it).second.get();
+        // objHeap.erase( id);
+    }
     static void FreeObjHeap( DObj id)
     {
         if( id != 0)
@@ -241,15 +254,20 @@ public:
             ObjHeapT::iterator it=objHeap.find( id);
             if  ( it != objHeap.end()) 
             { 
-                delete (*it).second.get();
-                objHeap.erase( id);
+                FreeObjHeapDirect( id, it);
+                // delete (*it).second.get();
+                // objHeap.erase( id);
             }
         }
     }
-    static void FreeObjHeapDirect( DObj id, ObjHeapT::iterator it)
+    static void FreeHeapDirect( DPtr id, HeapT::iterator it)
     {
-        delete (*it).second.get();
-        objHeap.erase( id);
+        BaseGDL* del = (*it).second.get();
+        heap.erase( id); 
+        delete del;
+        // delete (*it).second.get();
+        // // useless because of next: (*it).second.get() = NULL;
+        // heap.erase( id); 
     }
     static void FreeHeap( DPtr id)
     {
@@ -258,16 +276,18 @@ public:
                 HeapT::iterator it=heap.find( id);
                 if( it != heap.end()) 
                     { 
-                        delete (*it).second.get();
-                        heap.erase( id); 
+                        FreeHeapDirect( id, it);
+                        // delete (*it).second.get();
+                        // heap.erase( id); 
                     }
             }
     }
-    static void FreeHeapDirect( DPtr id, HeapT::iterator it)
+    static void HeapErase( DPtr id) // for LIST
     {
-        delete (*it).second.get();
-        // useless because of next: (*it).second.get() = NULL;
-        heap.erase( id); 
+        if( id != 0)
+            {
+              heap.erase( id); 
+            }
     }
 
    static void FreeHeap( DPtrGDL* p)
@@ -349,7 +369,7 @@ std::cout << "<ObjHeapVar" << id << "> = " << (*it).second.Count() << std::endl;
     }
    static void DecRefObj( DObjGDL* p)
     {
-        SizeT nEl=p->N_Elements();
+        SizeT nEl=p->Size();//N_Elements();
         for( SizeT ix=0; ix < nEl; ix++)
         {
             DObj id= (*p)[ix];
@@ -426,7 +446,7 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
     }
    static void IncRefObj( DObjGDL* p)
     {
-        SizeT nEl=p->N_Elements();
+        SizeT nEl=p->Size();//N_Elements();
         for( SizeT ix=0; ix < nEl; ix++)
         {
             DObj id= (*p)[ix];
@@ -439,13 +459,15 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
     static BaseGDL*& GetHeap( DPtr ID)
     {
         HeapT::iterator it=heap.find( ID);
-        if( it == heap.end()) throw HeapException();
+        if( it == heap.end()) 
+            throw HeapException();
         return it->second.get();
     }
     static DStructGDL*& GetObjHeap( DObj ID)
     {
         ObjHeapT::iterator it=objHeap.find( ID);
-        if( it == objHeap.end()) throw HeapException();
+        if( it == objHeap.end()) 
+            throw HeapException();
         return it->second.get();
     }
 
@@ -457,12 +479,12 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
         if( it == objHeap.end()) return NULL;
         return it->second.get()->Desc()->GetOperator( opIx);
     }
-    // static DStructGDL* GetObjHeapNoThrow( DObj ID)
-    // {
-    //     ObjHeapT::iterator it=objHeap.find( ID);
-    //     if( it == objHeap.end()) return NULL;
-    //     return it->second.get();
-    // }
+    static DStructGDL* GetObjHeapNoThrow( DObj ID)
+    {
+        ObjHeapT::iterator it=objHeap.find( ID);
+        if( it == objHeap.end()) return NULL;
+        return it->second.get();
+    }
 //     static DStructGDL*& GetObjHeap( DObj ID, ObjHeapT::iterator& it)
 //     {
 // //         ObjHeapT::iterator it=objHeap.find( ID);
@@ -491,15 +513,15 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
         }
         return 0;
     }
-    static BaseGDL** GetPtrToHeap( BaseGDL* p)
-    {
-        for( HeapT::iterator it=heap.begin(); it != heap.end(); ++it)
-        {
-            if( it->second.get() == p)
-                return &it->second.get();
-        }
-        return NULL;
-    }
+    // static BaseGDL** GetPtrToHeap( BaseGDL* p)
+    // {
+    //     for( HeapT::iterator it=heap.begin(); it != heap.end(); ++it)
+    //     {
+    //         if( it->second.get() == p)
+    //             return &it->second.get();
+    //     }
+    //     return NULL;
+    // }
     static DPtrGDL* GetAllHeap()
     {
         SizeT nEl = heap.size();
@@ -576,6 +598,21 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
         return ret;
     }
 
+
+    static void ResetHeap() // purges both heaps
+    {
+        for( HeapT::iterator it=heap.begin(); it != heap.end(); ++it)
+        {
+           delete (*it).second.get();
+           heap.erase( it->first); 
+        }
+        for( ObjHeapT::iterator it=objHeap.begin(); it != objHeap.end(); ++it)
+        {
+            delete (*it).second.get();
+            objHeap.erase( it->first); 
+        }
+    }
+
     // name of data
     static const std::string Name( BaseGDL* p) // const
     {
@@ -592,6 +629,9 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
         return "<(ptr to undefined expression not found on the heap)>";
     }
 
+
+
+
     // compiler (lexer, parser, treeparser) def in dinterpreter.cpp
     static void ReportCompileError( GDLException& e, const std::string& file = "");
 
@@ -639,17 +679,22 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
     {
         DString msgPrefix = SysVar::MsgPrefix();
 
-        EnvStackT::reverse_iterator upEnv = callStack.rbegin();
-        //EnvStackT::reverse_iterator env = upEnv++;
-        upEnv++;
-        for(; 
-            upEnv != callStack.rend();
-            ++upEnv /*,++env*/)
+        // EnvStackT::reverse_iterator upEnv = callStack.rbegin();
+        // //EnvStackT::reverse_iterator env = upEnv++;
+        // upEnv++;
+        // for(; 
+        //     upEnv != callStack.rend();
+        //     ++upEnv /*,++env*/)
+        
+        long actIx = callStack.size() - 2;
+        for( ; actIx >= 0; --actIx)
         {
-            std::cerr << msgPrefix << std::right << std::setw( w) << "";
-            std::cerr << std::left << std::setw(16) << (*upEnv)->GetProName();
+            EnvStackT::pointer_type upEnv = callStack[ actIx]; 
 
-            std::string file = (*upEnv)->GetFilename();
+            std::cerr << msgPrefix << std::right << std::setw( w) << "";
+            std::cerr << std::left << std::setw(16) << upEnv->GetProName();
+
+            std::string file = upEnv->GetFilename();
             if( file != "")
             {              
 //                 ProgNodeP cNode= (*env)->CallingNode();
@@ -661,14 +706,13 @@ std::cout << add << " + <ObjHeapVar" << id << ">" << std::endl;
 //                 {
 //                     std::cerr << std::right << std::setw(6) << "";
 //                 }                
-
 //                 ProgNodeP cNode= (*env)->CallingNode();
 //                 if( cNode != NULL && cNode->getLine() != 0)
 //                 {       
 //                     (*upEnv)->SetLineNumber( cNode->getLine());
 //                 }
 
-                int lineNumber = (*upEnv)->GetLineNumber();
+                int lineNumber = upEnv->GetLineNumber();
                 if( lineNumber != 0)
                 {       
                     std::cerr << std::right << std::setw(6) << lineNumber;
@@ -755,21 +799,21 @@ public:
 	public: BaseGDL**  l_ret_expr(ProgNodeP _t);
 	public: BaseGDL*  expr(ProgNodeP _t);
 	public:  BaseGDL**  l_arrayexpr_mfcall_as_mfcall(ProgNodeP _t);
-	public:  BaseGDL**  l_function_call(ProgNodeP _t);
+	public:  BaseGDL**  l_function_call_internal(ProgNodeP _t);
 	public: BaseGDL*  tmp_expr(ProgNodeP _t);
-	public:  BaseGDL*  lib_function_call(ProgNodeP _t);
+	public:  BaseGDL*  lib_function_call_internal(ProgNodeP _t);
 	public: BaseGDL*  r_expr(ProgNodeP _t);
-	public: BaseGDL*  l_decinc_indexable_expr(ProgNodeP _t,
-		int dec_inc
+	public: BaseGDL**  l_decinc_indexable_expr(ProgNodeP _t,
+		int dec_inc, BaseGDL*& res
 	);
 	public: BaseGDL**  l_defined_simple_var(ProgNodeP _t);
 	public: BaseGDL**  l_sys_var(ProgNodeP _t);
-	public: BaseGDL*  l_decinc_array_expr(ProgNodeP _t,
-		int dec_inc
+	public: BaseGDL**  l_decinc_array_expr(ProgNodeP _t,
+		int dec_inc, BaseGDL*& res
 	);
 	public: ArrayIndexListT*  arrayindex_list(ProgNodeP _t);
-	public: BaseGDL*  l_decinc_dot_expr(ProgNodeP _t,
-		int dec_inc
+	public: BaseGDL**  l_decinc_dot_expr(ProgNodeP _t,
+		int dec_inc, BaseGDL*& res
 	);
 	public: void l_dot_array_expr(ProgNodeP _t,
 		DotAccessDescT* aD
@@ -777,12 +821,12 @@ public:
 	public: void tag_array_expr(ProgNodeP _t,
 		DotAccessDescT* aD
 	);
-	public: BaseGDL*  l_decinc_expr(ProgNodeP _t,
-		int dec_inc
+	public: BaseGDL**  l_decinc_expr(ProgNodeP _t,
+		int dec_inc, BaseGDL*& res
 	);
 	public: BaseGDL*  indexable_expr(ProgNodeP _t);
 	public: BaseGDL*  indexable_tmp_expr(ProgNodeP _t);
-	public: BaseGDL**  l_expr(ProgNodeP _t,
+	public: BaseGDL**  l_expr_internal(ProgNodeP _t,
 		BaseGDL* right
 	);
 	public: BaseGDL**  l_simple_var(ProgNodeP _t);
@@ -802,18 +846,16 @@ public:
 	public: BaseGDL*  r_dot_indexable_expr(ProgNodeP _t,
 		DotAccessDescT* aD
 	);
-	public: BaseGDL*  sys_var_nocopy(ProgNodeP _t);
 	public: void r_dot_array_expr(ProgNodeP _t,
 		DotAccessDescT* aD
 	);
-	public: BaseGDL*  dot_expr(ProgNodeP _t);
+	public: BaseGDL*  dot_expr_unused(ProgNodeP _t);
 	public: BaseGDL*  assign_expr(ProgNodeP _t);
 	public:  BaseGDL*  unused_function_call(ProgNodeP _t);
-	public:  BaseGDL*  lib_function_call_retnew(ProgNodeP _t);
-	public: BaseGDL*  constant(ProgNodeP _t);
+	public:  BaseGDL*  lib_function_call_retnew_internal(ProgNodeP _t);
+	public: BaseGDL*  sys_var_nocopy(ProgNodeP _t);
 	public: BaseGDL*  simple_var(ProgNodeP _t);
 	public: BaseGDL*  sys_var(ProgNodeP _t);
-	public: BaseGDL*  unused_constant_nocopy(ProgNodeP _t);
 	public: BaseGDL**  l_arrayexpr_mfcall_as_arrayexpr(ProgNodeP _t,
 		BaseGDL* right
 	);
@@ -823,7 +865,6 @@ public:
 	public: void parameter_def_nocheck(ProgNodeP _t,
 		EnvBaseT* actEnv
 	);
-	public: ArrayIndexListT*  arrayindex_list_noassoc(ProgNodeP _t);
 	public: void arrayindex_list_overload(ProgNodeP _t,
 		IxExprListT& indexList
 	);
@@ -839,10 +880,10 @@ protected:
 private:
 	static const char* tokenNames[];
 #ifndef NO_STATIC_CONSTS
-	static const int NUM_TOKENS = 235;
+	static const int NUM_TOKENS = 237;
 #else
 	enum {
-		NUM_TOKENS = 235
+		NUM_TOKENS = 237
 	};
 #endif
 	
