@@ -300,32 +300,32 @@ private:
     SizeT nDot=tag.size();
     SizeT d;
     for( d=0; d<nDot; ++d)
-      {
-	  if( ix[d] == NULL)
-	  { // loop over all elements
-		  if( dStruct[d]->N_Elements() > 1)
-			  dim >> dStruct[d]->Dim();
-	  }
-	  else
-	  {
-		  ix[d]->SetVariable( dStruct[d]);
-		  if( ix[d]->N_Elements() > 1)
-			  dim >> ix[d]->GetDim();
-	  }
+    {
+      if( ix[d] == NULL)
+      { // loop over all elements
+	if( dStruct[d]->N_Elements() > 1)
+		dim >> dStruct[d]->Dim();
       }
-//     dimension topDim;
+      else
+      {
+	ix[d]->SetVariable( dStruct[d]);
+	if( ix[d]->N_Elements() > 1)
+		dim >> ix[d]->GetDim();
+      }
+    }
+    //     dimension topDim;
     if( ix[d] == NULL)
     { // loop over all elements
-// 	topDim=top->Dim();
-// 	dim >> topDim;
-	dim >> top->Dim();
+    // 	topDim=top->Dim();
+    // 	dim >> topDim;
+	    dim >> top->Dim();
     }
     else
     {
-	ix[d]->SetVariable( top);
-// 	topDim=ix[d]->GetDim();
-// 	dim >> topDim;
-	dim >> ix[d]->GetDim();
+	    ix[d]->SetVariable( top);
+    // 	topDim=ix[d]->GetDim();
+    // 	dim >> topDim;
+	    dim >> ix[d]->GetDim();
     }
   }
   
@@ -364,7 +364,7 @@ public:
 
     BaseGDL* newData;
     // no zeroing, here the new variable is created 
-    // zero only for GDL_PTR and GDL_OBJ (refcounting)
+    // zero only for GDL_PTR and GDL_OBJ (because of ref counting)
     if( top->Type() == GDL_PTR || top->Type() == GDL_OBJ)
       newData=top->New( dim);//, BaseGDL::NOZERO);
     else
@@ -391,7 +391,7 @@ public:
     SizeT rRank=r->Rank();
 
 //     if( rRank > lRank)
-//       throw GDLException(NULL,"Conflicting data structures (rank).",true,false);
+//       throw GDLException(-1,NULL,"Conflicting data structures (rank).",true,false);
 
     SizeT topRank=top->Rank();
 
@@ -408,7 +408,7 @@ public:
       {
 	 // IDL seems to allow a maximum of one rank more for the r-value
 // 	if( rRank > (topRank+1))
-// 	  throw GDLException(NULL,"Conflicting data structures (top-rank).",true,false);
+// 	  throw GDLException(-1,NULL,"Conflicting data structures (top-rank).",true,false);
 
 	// inplace copy to every instance of top
 	// just loop over all top elements and insert (at appropriate indices)
@@ -418,7 +418,7 @@ public:
 	if( r->Type() != top->Type())
 	  {
 	    BaseGDL* rConv = r->Convert2( top->Type(), BaseGDL::COPY);
-	    std::auto_ptr<BaseGDL> conv_guard( rConv);
+	    Guard<BaseGDL> conv_guard( rConv);
 
 	    DoAssign( dStruct[0], rConv);
 	  }
@@ -431,7 +431,7 @@ public:
 	// all dimensions must match here
 	for( SizeT i=0; i<rRank; i++)
 	  if( dim[i] != r->Dim(i))
-	    throw GDLException(NULL,"Conflicting data structures (dim).",true,false);
+	    throw GDLException(-1,NULL,"Conflicting data structures (dim).",true,false);
 
 	// copy only topRank dimensions each time (topElem elements)
 	// topRank is the dim to start the outer loop with
@@ -445,7 +445,7 @@ public:
 	if( r->Type() != top->Type())
 	  {
 	    BaseGDL* rConv = r->Convert2( top->Type(), BaseGDL::COPY);
-	    std::auto_ptr<BaseGDL> conv_guard( rConv);
+	    Guard<BaseGDL> conv_guard( rConv);
 
 	    DoAssign( dStruct[0], rConv);
 	  }
@@ -491,7 +491,7 @@ public:
   void ADRoot( DStructGDL* s, ArrayIndexListT* ix_=NULL) // root
   {
 //     if( s->IsAssoc())
-//       throw GDLException(NULL,"File expression not allowed in this context.",true,false);
+//       throw GDLException(-1,NULL,"File expression not allowed in this context.",true,false);
     propertyAccess = false;
     dStruct.push_back(s);
     ix.push_back(ix_); 
@@ -507,20 +507,27 @@ public:
 // 	// must only have one time property access
 // 	if( propertyAccess)
 // 	{
-// 	  throw GDLException(NULL,"Cannot access tag: "+ tagName+" [of property: "+propertyName+"].",true,false);
+// 	  throw GDLException(-1,NULL,"Cannot access tag: "+ tagName+" [of property: "+propertyName+"].",true,false);
 // 	}
 // 	propertyName = tagName;
 // 	propertyAccess = true;
 // 	return; // no further change
 // 	// hence "Add( SizeT)" will fail next time as well -> no further action here
 //       }
-      throw GDLException(NULL,"Left side of a tag must be a STRUCT: "+tagName);
+      assert( top != NULL);
+      if( top->Type() == GDL_OBJ) 
+	throw GDLException(-1,NULL,"Nested structure references are not allowed with objects. Consider using parentheses: "+tagName);
+      else 
+	throw GDLException(-1,NULL,"Left side of a tag must be a STRUCT: "+tagName);
     }
    
     int t=dStruct.back()->Desc()->TagIndex( tagName);
     if( t == -1) 
-      throw GDLException(NULL,"Tag name: "+tagName+" is undefined for STRUCT.",true,false);
-    
+    {
+      // TODO: Check for call to Get/SetProperty
+      
+      throw GDLException(-1,NULL,"Tag name: "+tagName+" is undefined for STRUCT.",true,false);
+    }
     // call SizeT version
     SizeT tagIx=static_cast<SizeT>(t);
     ADAdd( tagIx);
@@ -532,35 +539,38 @@ public:
     DStructGDL* actTop=dStruct.back();
 
     if( actTop == NULL)
-      throw GDLException(NULL,"Expression must be a STRUCT in this context.",true,false);
+      throw GDLException(-1,NULL,"Expression must be a STRUCT in this context.",true,false);
     
     if( actTop->N_Elements() == 0) // maybe not needed
-      throw GDLException(NULL,"Error struct data empty.",true,false);
+      throw GDLException(-1,NULL,"Error struct data empty.",true,false);
     
     SizeT nTags=actTop->Desc()->NTags();
     
     if( tagN >= nTags)
-      throw GDLException(NULL,"Invalid tag number.",true,false);
+      throw GDLException(-1,NULL,"Invalid tag number.",true,false);
 
+    // TODO: Insert object struct for Get/SetProperty
+    // tagN == -1 (change type to int)?
+      
     top=actTop->GetTag( tagN, 0);
 
     // push struct onto struct stack
-    DStructGDL* newTop;
     if( top->Type() == GDL_STRUCT)
-      newTop = static_cast<DStructGDL*>(top);
+    {
+      DStructGDL* newTop=static_cast<DStructGDL*>(top);
+      dStruct.push_back( newTop);
+    }
     else
-      newTop = NULL;
-
-    //    if( newTop != NULL) dStruct.push_back( newTop);
-    dStruct.push_back( newTop);
-
+    {
+      dStruct.push_back( NULL);      
+    }
     tag.push_back(tagN);
   }
 
   void ADAddIx( ArrayIndexListT* ix_) // tags
   {
     if( propertyAccess && ix_ != NULL)
-	  throw GDLException(NULL,"Property must not be indexed: "+propertyName+".",true,false);      
+	  throw GDLException(-1,NULL,"Property must not be indexed: "+propertyName+".",true,false);      
     ix.push_back(ix_); 
   }
 
