@@ -78,10 +78,10 @@ public:
   virtual void Init( BaseGDL*, BaseGDL*, BaseGDL*) { assert( false);}
 
   virtual bool IsRange() { return false;} // default for non-ranges
-  virtual BaseGDL* OverloadIndexNew() { assert( false);}
-  virtual BaseGDL* OverloadIndexNew( BaseGDL*) { assert( false);}
-  virtual BaseGDL* OverloadIndexNew( BaseGDL*, BaseGDL*) { assert( false);}
-  virtual BaseGDL* OverloadIndexNew( BaseGDL*, BaseGDL*, BaseGDL*) { assert( false);}
+  virtual BaseGDL* OverloadIndexNew() { assert( false); return 0;}
+  virtual BaseGDL* OverloadIndexNew( BaseGDL*) { assert( false); return 0;}
+  virtual BaseGDL* OverloadIndexNew( BaseGDL*, BaseGDL*) { assert( false); return 0;}
+  virtual BaseGDL* OverloadIndexNew( BaseGDL*, BaseGDL*, BaseGDL*) { assert( false); return 0;}
 
   virtual void Clear() {}
   virtual ~ArrayIndexT() {}
@@ -133,6 +133,7 @@ return arrayIxArr[ ix];
 SizeT size() const { return sz;}
 void push_back( ArrayIndexT* aIx)
 {
+// if( sz >= MAXRANK) // debug
 assert( sz < MAXRANK);
 arrayIxArr[ sz++] = aIx;
 }
@@ -389,8 +390,8 @@ public:
 //   // if this is used, Init was NOT called before
 //   BaseGDL* Index( BaseGDL* var, IxExprListT& ixL)
 //   {
-//     if( s >= var->Size())
-//       throw GDLException(NULL,"Scalar subscript out of range [>].h1",true,false);
+//     if( s >= var->N_Elements()/*var->Size()*/)
+//       throw GDLException(-1,NULL,"Scalar subscript out of range [>].h1",true,false);
 //     return var->NewIx( s);
 //   }
 
@@ -404,13 +405,13 @@ public:
       s = sInit;
 
     if( s < 0)
-	throw GDLException(NULL,"Constant scalar subscript out of range [-i].",true,false);
+	throw GDLException(-1,NULL,"Constant scalar subscript out of range [-i].",true,false);
     if( s >= varDim && s > 0) // varDim == 0 && s == 0 ok
-	throw GDLException(NULL,"Constant scalar out of range [i].",true,false);
+	throw GDLException(-1,NULL,"Constant scalar out of range [i].",true,false);
     return 1;
 
 //     if( s >= varDim && s > 0) // varDim == 0 && s == 0 ok
-//       throw GDLException(NULL,"Scalar subscript out of range [>].h2",true,false);
+//       throw GDLException(-1,NULL,"Scalar subscript out of range [>].h2",true,false);
 //     return 1;
   }
 }; //class CArrayIndexScalar: public ArrayIndexT
@@ -528,7 +529,7 @@ public:
       // 	if( ret == -1) // index < 0
       // 	  {
       // 	    throw 
-      // 	      GDLException(NULL, "Subscript range values of the"
+      // 	      GDLException(-1,NULL, "Subscript range values of the"
       // 			    " form low:high must be >= 0, < size,"
       // 			    " with low <= high.",true,false);
       // 	  }
@@ -543,7 +544,7 @@ public:
 
     int typeCheck = DTypeOrder[ dType];
     if( typeCheck >= 100)
-      throw GDLException(NULL,"Type not allowed as subscript.",true,false);
+      throw GDLException(-1, NULL,"Type not allowed as subscript.",true,false);
 
     //SizeT nElem = ix_->N_Elements();
     //    ix = new SizeT[ nElem]; // allocate array
@@ -572,9 +573,9 @@ public:
 	s = sInit;
 
       if( s < 0)
-	throw GDLException(NULL,"Subscript out of range [-i].",true,false);
+	throw GDLException(-1, NULL,"Subscript out of range [-i].",true,false);
       if( s >= varDim && s > 0)
-	throw GDLException(NULL,"Subscript out of range [i].",true,false);
+	throw GDLException(-1, NULL,"Subscript out of range [i].",true,false);
       return 1;
     }
     // INDEXED
@@ -684,7 +685,7 @@ public:
     DType dType = rawData->Type();
     int typeCheck = DTypeOrder[ dType];
     if( typeCheck >= 100)
-      throw GDLException(NULL,"Type not allowed as subscript.",true,false);
+      throw GDLException(-1, NULL,"Type not allowed as subscript.",true,false);
 
     if( strictArrSubs)
       ix = new (ixBuf) AllIxIndicesStrictT( rawData);
@@ -715,9 +716,9 @@ public:
 	s = sInit;
 
       if( s < 0)
-	throw GDLException(NULL,"Subscript out of range [-i].",true,false);
+	throw GDLException(-1, NULL,"Subscript out of range [-i].",true,false);
       if( s >= varDim && s > 0)
-	throw GDLException(NULL,"Subscript out of range [i].",true,false);
+	throw GDLException(-1, NULL,"Subscript out of range [i].",true,false);
       return 1;
     }
     // INDEXED
@@ -778,7 +779,7 @@ public:
   BaseGDL* OverloadIndexNew( BaseGDL* s_) 
   {
     Init( s_);
-    DLong arr[3] = {sInit,-1,1};
+    DLong arr[3] = {static_cast<DLong>(sInit),-1,1};
     return new DLongGDL( arr, 3);
   }
   
@@ -803,18 +804,20 @@ public:
       {
 	if( s_->N_Elements() == 0)
 	  throw 
-	    GDLException(NULL,"Internal error: Scalar2RangeT:"
+	    GDLException(-1,NULL,"Internal error: Scalar2RangeT:"
 			  " 1st index empty",true,false); 
 	else
 	  throw 
-	    GDLException(NULL,"Expression must be a scalar"
+	    GDLException(-1,NULL,"Expression must be a scalar"
 			  " in this context.",true,false); 
       }
+//    s = sInit; // for assoc  
+
 // not with Scalar2RangeT():
 //     if( retMsg == -1) // index < 0
 //       {
 // 	throw 
-// 	  GDLException(NULL,"Subscript range values of the"
+// 	  GDLException(-1,NULL,"Subscript range values of the"
 // 			" form low:high must be >= 0, < size, "
 // 			"with low <= high.",true,false);
 //       }
@@ -823,12 +826,12 @@ public:
   SizeT NIter( SizeT varDim)
   {
     if( sInit >= varDim) // && s > 0)
-      throw GDLException(NULL,"Subscript out of range [s:*].",true,false);
+      throw GDLException(-1,NULL,"Subscript out of range [s:*].",true,false);
     if( sInit < 0)
     {
       s = sInit + varDim;
       if( s < 0)
-	      throw GDLException(NULL,"Subscript out of range [-s:*].",true,false);
+	      throw GDLException(-1,NULL,"Subscript out of range [-s:*].",true,false);
 	  
       return (varDim - s);
     }
@@ -848,7 +851,7 @@ public:
 
   BaseGDL* OverloadIndexNew()
   {
-    DLong arr[3] = {sInit,-1,1};
+    DLong arr[3] = {static_cast<DLong>(sInit),-1,1};
     return new DLongGDL( arr, 3);
   }
 
@@ -888,7 +891,7 @@ public:
   BaseGDL* OverloadIndexNew( BaseGDL* s_, BaseGDL* e_) 
   {
     Init( s_, e_);
-    DLong arr[3] = {sInit,eInit,1};
+    DLong arr[3] = {static_cast<DLong>(sInit),static_cast<DLong>(eInit),1};
     return new DLongGDL( arr, 3);
   }
 
@@ -909,22 +912,22 @@ public:
 
   void Init( BaseGDL* s_, BaseGDL* e_)
   {
-// 	SizeT varSize = var->Size();
+// 	SizeT varSize = var->N_Elements()/*var->Size()*/;
     
     int retMsg=s_->Scalar2RangeT(sInit);
     if( retMsg == 0) // index empty or array
       {
 	if( s_->N_Elements() == 0)
 	  throw 
-	    GDLException(NULL,"Internal error: Scalar2RangeT: 1st index empty.",true,false);
+	    GDLException(-1,NULL,"Internal error: Scalar2RangeT: 1st index empty.",true,false);
 	else
 	  throw 
-	    GDLException(NULL,"Expression must be a scalar in this context.",true,false); 
+	    GDLException(-1,NULL,"Expression must be a scalar in this context.",true,false); 
       }
 //     if( retMsg == -1) // index < 0
 //       {
 // 	throw 
-// 	  GDLException(NULL,"Subscript range values of the form low:high " 
+// 	  GDLException(-1,NULL,"Subscript range values of the form low:high " 
 // 			"must be >= 0, < size, with low <= high.",true,false);
 //       }
     
@@ -933,10 +936,10 @@ public:
       {
 	if( e_->N_Elements() == 0)
 	  throw 
-	    GDLException(NULL,"Internal error: Scalar2RangeT: 2nd index empty.",true,false);
+	    GDLException(-1,NULL,"Internal error: Scalar2RangeT: 2nd index empty.",true,false);
 	else
 	  throw 
-	    GDLException(NULL,"Expression must be a scalar in this context.",true,false); 
+	    GDLException(-1,NULL,"Expression must be a scalar in this context.",true,false); 
       }
   }
 
@@ -949,7 +952,7 @@ public:
     {
 	    s = sInit + varDim;
 	    if( s < 0)
-		    throw GDLException(NULL,"Subscript out of range [S:e].",true,false);
+		    throw GDLException(-1,NULL,"Subscript out of range [S:e].",true,false);
     }
     else
 	    s = sInit;
@@ -957,17 +960,17 @@ public:
     {
 	    e = eInit + varDim;
 	    if( e < 0)
-		    throw GDLException(NULL,"Subscript out of range [s:E].",true,false);
+		    throw GDLException(-1,NULL,"Subscript out of range [s:E].",true,false);
     }
     else
 	    e = eInit;
 
     if( s > e)
       throw 
-	GDLException(NULL,"Subscript range values of the form low:high "
+	GDLException(-1,NULL,"Subscript range values of the form low:high "
 		"must be < size, with low <= high",true,false);
     if( e >= varDim) // && e > 0)
-		throw GDLException(NULL,"Subscript out of range [s:e].",true,false);
+		throw GDLException(-1,NULL,"Subscript out of range [s:e].",true,false);
     return (e - s + 1);
   }
 };
@@ -983,7 +986,7 @@ public:
 
   BaseGDL* OverloadIndexNew()
   {
-    DLong arr[3] = {sInit,eInit,1};
+    DLong arr[3] = {static_cast<DLong>(sInit),static_cast<DLong>(eInit),1};
     return new DLongGDL( arr, 3);
   }
 
@@ -1026,7 +1029,7 @@ public:
   BaseGDL* OverloadIndexNew( BaseGDL* s_, BaseGDL* stride_) 
   {
     Init( s_, stride_);
-    DLong arr[3] = {sInit,-1,stride};
+    DLong arr[3] = {static_cast<DLong>(sInit),-1,static_cast<DLong>(stride)};
     return new DLongGDL( arr, 3);
   }
 
@@ -1066,12 +1069,12 @@ public:
 // 			" form low:high must be >= 0, < size, with low <= high.",true,false);
 //       }
     // stride
-    retMsg=stride_->Scalar2index( stride);
+    retMsg=stride_->Scalar2Index( stride);
     if( retMsg == 0) // index empty or array
       {
 			if( stride_->N_Elements() == 0)
 			throw
-				GDLException(  "Internal error: Scalar2index:"
+				GDLException(  "Internal error: Scalar2Index:"
 					" stride index empty",true,false);
 			else
 			throw
@@ -1093,14 +1096,14 @@ public:
       {
 	s = sInit + varDim;
 	if( s < 0)
-		throw GDLException(NULL,"Subscript out of range [-S:*:stride].",true,false);
+		throw GDLException(-1,NULL,"Subscript out of range [-S:*:stride].",true,false);
 	return (varDim - s + stride - 1)/stride;
       }
     else
       s= sInit;
 
     if( s >= varDim) // && s > 0)
-      throw GDLException(NULL,"Subscript out of range [s:*:stride].",true,false);
+      throw GDLException(-1,NULL,"Subscript out of range [s:*:stride].",true,false);
     return (varDim - s + stride - 1)/stride;
   }
 };
@@ -1114,7 +1117,7 @@ public:
 
   BaseGDL* OverloadIndexNew()
   {
-    DLong arr[3] = {sInit,-1,stride};
+    DLong arr[3] = {static_cast<DLong>(sInit),-1,static_cast<DLong>(stride)};
     return new DLongGDL( arr, 3);
   }
 
@@ -1166,7 +1169,7 @@ public:
   BaseGDL* OverloadIndexNew( BaseGDL* s_, BaseGDL* e_, BaseGDL* stride_)
   {
     Init( s_, e_, stride_);
-    DLong arr[3] = {sInit,eInit,stride};
+    DLong arr[3] = {static_cast<DLong>(sInit),static_cast<DLong>(eInit),static_cast<DLong>(stride)};
     return new DLongGDL( arr, 3);
   }
 
@@ -1218,12 +1221,12 @@ public:
       }
                             
     // stride
-    retMsg=stride_->Scalar2index(stride);
+    retMsg=stride_->Scalar2Index(stride);
     if( retMsg == 0) // index empty or array
       {
 	if( stride_->N_Elements() == 0)
 	  throw 
-	    GDLException(  "Internal error: Scalar2index:"
+	    GDLException(  "Internal error: Scalar2Index:"
 			  " stride index empty",true,false); 
 	else
 	  throw 
@@ -1246,7 +1249,7 @@ public:
 	    {
 	      s = sInit + varDim;
 	      if( s < 0)
-		      throw GDLException(NULL,"Subscript out of range [-S:e:stride].",true,false);
+		      throw GDLException(-1,NULL,"Subscript out of range [-S:e:stride].",true,false);
 	    }
     else
 	    s = sInit;
@@ -1254,19 +1257,19 @@ public:
 	    {
 	      e = eInit + varDim;
 	      if( e < 0)
-		      throw GDLException(NULL,"Subscript out of range [s:-E:stride].",true,false);
+		      throw GDLException(-1,NULL,"Subscript out of range [s:-E:stride].",true,false);
 	    }
     else
 	    e = eInit;
     
     if( s > e)
 		throw 
-			GDLException(NULL,"Subscript range values of the form low:high "
+			GDLException(-1,NULL,"Subscript range values of the form low:high "
 				"must be < size, with low <= high",true,false);
     
     if( e >= varDim) // && e > 0)
       {
-		throw GDLException(NULL,"Subscript out of range [s:E:st].",true,false);
+		throw GDLException(-1,NULL,"Subscript out of range [s:E:st].",true,false);
       }
     return (e - s + stride)/stride;
   }
@@ -1281,7 +1284,7 @@ public:
 
   BaseGDL* OverloadIndexNew()
   {
-    DLong arr[3] = {sInit,eInit,stride};
+    DLong arr[3] = {static_cast<DLong>(sInit),static_cast<DLong>(eInit),static_cast<DLong>(stride)};
     return new DLongGDL( arr, 3);
   }
 
