@@ -43,21 +43,27 @@ public:
 
   virtual ArrayIndexListT* Clone() { assert( 0); return NULL;}
 
-  virtual void InitAsOverloadIndex( IxExprListT& ixIn, IxExprListT* cleanupIx, IxExprListT& ixOut) 
+  virtual void InitAsOverloadIndex( IxExprListT& ixIn, /*IxExprListT* cleanupIx,*/ IxExprListT& ixOut) 
   { 
-    throw GDLException( NULL,"Internal error: ArrayIndexListT::InitAsOverladIndex( IxExprListT& ixInOut) called.",true,false);    
+    throw GDLException( -1, NULL,"Internal error: ArrayIndexListT::InitAsOverladIndex( IxExprListT& ixInOut) called.",true,false);    
   }
-  virtual void Init( IxExprListT& ix, IxExprListT* cleanupIx) 
+  virtual void Init( IxExprListT& ix)//, IxExprListT* cleanupIx) 
   { 
     assert( 0);
-    throw GDLException( NULL,"Internal error: ArrayIndexListT::Init( IxExprListT& ix, IxExprListT* cleanupIx) called.",true,false);    
+    throw GDLException( -1, NULL,"Internal error: ArrayIndexListT::Init( IxExprListT& ix, IxExprListT* cleanupIx) called.",true,false);    
   }
+  virtual IxExprListT* GetCleanupIx()
+  { 
+    assert( 0);
+    throw GDLException( -1, NULL,"Internal error: ArrayIndexListT::GetCleanupIx() called.",true,false);    
+  }
+
   virtual void Init() {}
   
    virtual bool ToAssocIndex( SizeT& lastIx) 
    { 
     assert( 0);       
-    throw GDLException( NULL,"Internal error: ArrayIndexListT::ToAssocIndex( SizeT& lastIx) called.",true,false);
+    throw GDLException( -1, NULL,"Internal error: ArrayIndexListT::ToAssocIndex( SizeT& lastIx) called.",true,false);
    }
 //  virtual bool ToAssocIndex( RangeT& lastIx) = 0;
 
@@ -107,7 +113,8 @@ private:
   char allIxInstance[ AllIxMaxSize];
   
 public:    
-  
+  IxExprListT* GetCleanupIx() { return &cleanupIx;}
+    
   ~ArrayIndexListOneT()
   {
 //     delete allIx;
@@ -147,18 +154,18 @@ public:
 //     allIxMulti.Clear();
     
     ix->Clear();
-	cleanupIx.Cleanup();
+    cleanupIx.Cleanup();
   }
 
   ArrayIndexListT* Clone() { return new ArrayIndexListOneT( *this);}
 
-  void Init( IxExprListT& ix_, IxExprListT* cleanupIxIn)
+  void Init( IxExprListT& ix_)//, IxExprListT* cleanupIxIn)
   {
     assert( allIx == NULL);
     assert( ix_.size() == nParam);
 
-    if( cleanupIxIn != NULL)
-      cleanupIx = *cleanupIxIn;
+//     if( cleanupIxIn != NULL)
+//       cleanupIx = *cleanupIxIn;
     
     if( nParam == 0) return;
     if( nParam == 1) 
@@ -188,10 +195,10 @@ public:
     // cannot be ArrayIndexScalar[VP] ix->Init();
     RangeT lastValIx;
     if( !ix->Scalar( lastValIx))
-      throw GDLException( NULL,"Record number must be a scalar in this context.",true,false);
+      throw GDLException( -1, NULL,"Record number must be a scalar in this context.",true,false);
 
     if( lastValIx < 0)
-      throw GDLException( NULL,"Record number must be a scalar > 0 in this context.",true,false);
+      throw GDLException( -1, NULL,"Record number must be a scalar > 0 in this context.",true,false);
 
     lastIx = lastValIx;
     return true;
@@ -203,13 +210,15 @@ public:
     assert( allIx == NULL);
 
     // for assoc variables last index is the record
-    // we cannot return here as sInit is not yet copied to s
-    //if( var->IsAssoc()) return;
-
+    if( var->IsAssoc())
+    {
+	// note: s is copied from sIter in ArrayIndex::Init
+       return;
+    }
     // ArrayIndexScalar[VP] are not initialized
     // they need the NIter call, but
     // for only one index they have their own ArrayIndexListT
-    nIx=ix->NIter( var->Size());
+    nIx=ix->NIter( var->N_Elements()/*var->Size()*/);
   }
   
   // structure of indexed expression
@@ -304,7 +313,7 @@ public:
   {
     // scalar case
     if( right->N_Elements() == 1 && !var->IsAssoc() &&
-	ix->NIter( var->Size()) == 1)// && var->Type() != GDL_STRUCT) 
+	ix->NIter( var->N_Elements()/*var->Size()*/) == 1)// && var->Type() != GDL_STRUCT) 
       {
 	var->AssignAtIx( ix->GetIx0(), right);
 	return;
@@ -319,7 +328,7 @@ public:
     else
       {
 	BaseGDL* rConv = right->Convert2( var->Type(), BaseGDL::COPY);
-	std::auto_ptr<BaseGDL> conv_guard( rConv);
+	Guard<BaseGDL> conv_guard( rConv);
 	
 	var->AssignAt( rConv, this); // assigns inplace
       }
@@ -328,15 +337,19 @@ public:
   // optimized for one dimensional access
   BaseGDL* Index( BaseGDL* var, IxExprListT& ix_)
   {
-    Init( ix_, NULL);
-    if( !var->IsAssoc() && ix->Scalar()) //ix->NIter( var->Size()) == 1)// && var->Type() != GDL_STRUCT) 
+    Init( ix_);//, NULL);
+
+    if( var->IsAssoc()) // deault case
+      return var->Index( this);
+    
+    if( /*!var->IsAssoc() &&*/ ix->Scalar()) //ix->NIter( var->N_Elements()/*var->Size()*/) == 1)// && var->Type() != GDL_STRUCT) 
       {
-	SizeT assertValue = ix->NIter( var->Size());
+	SizeT assertValue = ix->NIter( var->N_Elements()/*var->Size()*/);
 	assert( assertValue == 1);
 
 	return var->NewIx( ix->GetIx0());
       }
-    // normal case
+    // normal case, no assoc
     SetVariable( var);
     return var->Index( this);
   }
@@ -535,7 +548,7 @@ public:
   {
     sInit = varPtr->Data()->LoopIndex();
     if( sInit < 0)
-      throw GDLException( NULL,"Record number must be a scalar > 0 in this context.",true,false);
+      throw GDLException( -1, NULL,"Record number must be a scalar > 0 in this context.",true,false);
     s = sInit;
     lastIx = s;
     return true;
@@ -552,10 +565,10 @@ public:
     
     // for assoc variables last index is the record
     if( var->IsAssoc()) return;
-    if( s >= var->Size())
-      throw GDLException(NULL,"Scalar subscript out of range [>].1",true,false);
+    if( s >= var->N_Elements()/*var->Size()*/)
+      throw GDLException(-1, NULL,"Scalar subscript out of range (>).",true,false);
     if( s < 0)
-      throw GDLException(NULL,"Scalar subscript out of range [<].1",true,false);
+      throw GDLException(-1,NULL,"Scalar subscript out of range (<-1).",true,false);
   }
   
   // structure of indexed expression
@@ -599,8 +612,8 @@ public:
     if( right->N_Elements() == 1 && !var->IsAssoc()) // && var->Type() != GDL_STRUCT) 
       {
 	s = varPtr->Data()->LoopIndex();
-	if( s >= var->Size())
-	  throw GDLException(NULL,"Scalar subscript out of range [>].2",true,false);
+	if( s >= var->N_Elements()/*var->Size()*/)
+	  throw GDLException(-1,NULL,"Scalar subscript out of range [>].2",true,false);
 	var->AssignAtIx( s, right);
 	return;
       }
@@ -613,7 +626,7 @@ public:
     else
       {
 	BaseGDL* rConv = right->Convert2( var->Type(), BaseGDL::COPY);
-	std::auto_ptr<BaseGDL> conv_guard( rConv);
+	Guard<BaseGDL> conv_guard( rConv);
 	
 	var->AssignAt( rConv, this); // assigns inplace
       }
@@ -714,7 +727,7 @@ public:
   bool ToAssocIndex( SizeT& lastIx)
   {
     if( sInit < 0)
-      throw GDLException( NULL,"Record number must be a scalar > 0 in this context.",true,false);      
+      throw GDLException(-1,NULL,"Record number must be a scalar > 0 in this context.",true,false);      
     lastIx = sInit;
     return true;
   }
@@ -724,12 +737,12 @@ public:
   {
     if( var->IsAssoc()) return;
     if( sInit < 0)
-      s = sInit + var->Size();
+      s = sInit + var->N_Elements()/*var->Size()*/;
     // for assoc variables last index is the record
     if( s < 0)
-      throw GDLException(NULL,"Scalar subscript out of range [<].1",true,false);
-    if( s >= var->Size())
-      throw GDLException(NULL,"Scalar subscript out of range [>].1",true,false);
+      throw GDLException(-1,NULL,"Scalar subscript out of range [<].1",true,false);
+    if( s >= var->N_Elements()/*var->Size()*/)
+      throw GDLException(-1,NULL,"Scalar subscript out of range [>].1",true,false);
   }
 
   // returns one dim long ix in case of one element array index
@@ -750,11 +763,11 @@ public:
     if( right->N_Elements() == 1 && !var->IsAssoc())// && var->Type() != GDL_STRUCT) 
       {
 	if( sInit < 0)
-	  s = sInit + var->Size();
+	  s = sInit + var->N_Elements()/*var->Size()*/;
 	if( s < 0)
-	  throw GDLException(NULL,"Scalar subscript out of range [<].2",true,false);
-	if( s >= var->Size())
-	  throw GDLException(NULL,"Scalar subscript out of range [>].2",true,false);
+	  throw GDLException(-1,NULL,"Scalar subscript out of range [<].2",true,false);
+	if( s >= var->N_Elements()/*var->Size()*/)
+	  throw GDLException(-1,NULL,"Scalar subscript out of range [>].2",true,false);
 	var->AssignAtIx( s, right);
 	return;
       }
@@ -767,7 +780,7 @@ public:
     else
       {
 		BaseGDL* rConv = right->Convert2( var->Type(), BaseGDL::COPY);
-		std::auto_ptr<BaseGDL> conv_guard( rConv);
+		Guard<BaseGDL> conv_guard( rConv);
 	
 		var->AssignAt( rConv, this); // assigns inplace
       }
@@ -780,13 +793,13 @@ public:
     if( !var->IsAssoc())// && var->Type() != GDL_STRUCT)
       {
 	if( sInit < 0)
-	  s = sInit + var->Size();
+	  s = sInit + var->N_Elements()/*var->Size()*/;
 	if( s < 0)
-		throw GDLException(NULL,"Scalar subscript out of range [<].3",true,false);
-	if( s >= var->Size())
+		throw GDLException(-1,NULL,"Scalar subscript out of range [<].3",true,false);
+	if( s >= var->N_Elements()/*var->Size()*/)
 	{
-// 	    std::cout << s << " var->Size():" << var->Size() << std::endl;
-		throw GDLException(NULL,"Scalar subscript out of range [>].3",true,false);
+// 	    std::cout << s << " var->N_Elements()/*var->Size()*/:" << var->N_Elements()/*var->Size()*/ << std::endl;
+		throw GDLException(-1,NULL,"Scalar subscript out of range [>].3",true,false);
 	}
 	
 	return var->NewIx( s);
@@ -879,7 +892,7 @@ public:
     assert( ixList.size() > 1); // must be, from compiler
     
     if( ix->size() > MAXRANK)
-      throw GDLException(NULL,"Maximum of "+MAXRANK_STR+" dimensions allowed.",true,false);
+      throw GDLException(-1,NULL,"Maximum of "+MAXRANK_STR+" dimensions allowed.",true,false);
 
     nParam = 0;
     for( SizeT i=0; i<ixList.size(); ++i)
@@ -941,7 +954,7 @@ public:
     ixListEnd->Scalar( lastIxVal); // always scalar
 
     if( lastIxVal < 0)
-      throw GDLException( NULL,"Record number must be a scalar > 0 in this context.",true,false);
+      throw GDLException(-1,NULL,"Record number must be a scalar > 0 in this context.",true,false);
     
     lastIx = lastIxVal;
     return false; // multi dim
@@ -1047,7 +1060,7 @@ public:
     else
       {
 	BaseGDL* rConv = right->Convert2( var->Type(), BaseGDL::COPY);
-	std::auto_ptr<BaseGDL> conv_guard( rConv);
+	Guard<BaseGDL> conv_guard( rConv);
 	
 	var->AssignAt( rConv, this); // assigns inplace (not only scalar)
       }
@@ -1164,7 +1177,8 @@ protected:
   bool indexed; // is the variable index indexed?
 
 public:    
-  
+    IxExprListT* GetCleanupIx() { return &cleanupIx;}
+
   ~ArrayIndexListMultiT()
   {
 //     delete allIx;
@@ -1209,7 +1223,7 @@ public:
     assert( ix->size() != 0); // must be, from compiler
 
     if( ixList.size() > MAXRANK)
-      throw GDLException(NULL,"Maximum of "+MAXRANK_STR+" dimensions allowed.",true,false);
+      throw GDLException(-1,NULL,"Maximum of "+MAXRANK_STR+" dimensions allowed.",true,false);
 
     nParam = 0;
     for( SizeT i=0; i<ix->size(); ++i)
@@ -1276,13 +1290,13 @@ public:
   ArrayIndexListT* Clone() { return new ArrayIndexListMultiT( *this);}
 
 
-  void Init( IxExprListT& ix, IxExprListT* cleanupIxIn)
+  void Init( IxExprListT& ix)//, IxExprListT* cleanupIxIn)
   {
     assert( allIx == NULL);
     assert( ix.size() == nParam);
     	
-	if( cleanupIxIn != NULL)
-		cleanupIx = *cleanupIxIn;
+// 	if( cleanupIxIn != NULL)
+// 		cleanupIx = *cleanupIxIn;
 
     SizeT pIX = 0;
     for( SizeT i=0; i<ixList.size(); ++i)
@@ -1326,10 +1340,10 @@ public:
     
     RangeT lastValIx;
     if( !ixListEndTmp->Scalar( lastValIx))
-      throw GDLException(NULL ,"Record number must be a scalar in this context.",true,false);
+      throw GDLException(-1, NULL ,"Record number must be a scalar in this context.",true,false);
 
     if( lastValIx < 0)
-      throw GDLException( NULL,"Record number must be a scalar > 0 in this context.",true,false);
+      throw GDLException(-1, NULL,"Record number must be a scalar > 0 in this context.",true,false);
 
     lastIx = lastValIx;
    
@@ -1463,7 +1477,7 @@ public:
 	      {
 		SizeT nIter = ixList[i]->NIter( (i<varRank)?varDim[i]:1);
 		if( nIter != nIx)
-			throw GDLException(NULL, "All array subscripts must be of same size.", true, false);
+			throw GDLException(-1,NULL, "All array subscripts must be of same size.", true, false);
 	      }
 
       // in this case, having more index dimensions does not matter
@@ -1668,7 +1682,7 @@ public:
     else
       {
 	BaseGDL* rConv = right->Convert2( var->Type(), BaseGDL::COPY);
-	std::auto_ptr<BaseGDL> conv_guard( rConv);
+	Guard<BaseGDL> conv_guard( rConv);
 	
 	var->AssignAt( rConv, this); // assigns inplace
       }
@@ -1678,7 +1692,7 @@ public:
   BaseGDL* Index( BaseGDL* var, IxExprListT& ix)
   {
     // normal case
-    Init( ix, NULL);
+    Init( ix);//, NULL);
     SetVariable( var);
     if( nIx == 1 && !var->IsAssoc())
     {
@@ -1738,7 +1752,7 @@ class ArrayIndexListMultiNoneIndexedT: public ArrayIndexListMultiT
     assert( ix->size() != 0); // must be, from compiler
 
     if( ixList.size() > MAXRANK)
-      throw GDLException(NULL,"Maximum of "+MAXRANK_STR+" dimensions allowed.",true,false);
+      throw GDLException(-1,NULL,"Maximum of "+MAXRANK_STR+" dimensions allowed.",true,false);
 
     nParam = 0;
     for( SizeT i=0; i<ix->size(); ++i)
@@ -1976,7 +1990,7 @@ public:
     assert( ix->size() != 0); // must be, from compiler
 
     if( ixList.size() > MAXRANK)
-      throw GDLException(NULL,"Maximum of "+MAXRANK_STR+" dimensions allowed.",true,false);
+      throw GDLException(-1,NULL,"Maximum of "+MAXRANK_STR+" dimensions allowed.",true,false);
 
     nParam = 0;
     for( SizeT i=0; i<ix->size(); ++i)
@@ -2066,7 +2080,7 @@ public:
 			{
 				SizeT nIter = ixList[i]->NIter( (i<varRank)?varDim[i]:1);
 				if( nIter != nIx)
-					throw GDLException(NULL, "All array subscripts must be of same size.", true, false);
+					throw GDLException(-1,NULL, "All array subscripts must be of same size.", true, false);
 			}
 
 		// in this case, having more index dimensions does not matter
