@@ -31,7 +31,7 @@ class Assoc_: public Parent_
 
 public:
 	// memory management optimization
-static std::deque< void*> freeList;
+static std::vector< void*> freeList;
 
 	// operator new and delete
 static 	void* operator new( size_t bytes);
@@ -50,7 +50,7 @@ static	void operator delete( void *ptr);
 			     sliceSize( cp.sliceSize)
   {}
 
- Parent_* Dup() { return new Assoc_(*this);}
+  Assoc_* Dup() const { return new Assoc_(*this);}
 
   void AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList, SizeT offset);
   void AssignAt( BaseGDL* srcIn, ArrayIndexListT* ixList);
@@ -80,20 +80,25 @@ static	void operator delete( void *ptr);
     throw GDLException("Assoc_::NBytes() called.");
   }
   
-  std::ostream& Write( std::ostream& os, bool swapEndian)
+  std::ostream& Write( std::ostream& os, bool swapEndian, 
+			bool compress, XDR *xdrs)
   {
     throw GDLException("Assoc_::Write(...) called.");
   }
-  std::istream& Read( std::istream& os, bool swapEndian)
-  {
+  std::istream& Read( std::istream& os, bool swapEndian, 
+		       bool compress, XDR *xdrs)
+ {
     throw GDLException("Assoc_::Read(...) called.");
   }
 
   //  std::ostream& ToStream(std::ostream& o)
-  std::ostream& ToStream(std::ostream& o, SizeT width = 0, SizeT* actPosPtr = NULL)
-  {
-    throw GDLException("File expression not allowed in this context.");
-  }
+  std::ostream& ToStream(std::ostream& o, SizeT width = 0, SizeT* actPosPtr = NULL);
+//   {
+//   
+//     os << "File<"+/dev/zero+"> "; 
+//     return Parent_::ToStream( o, width, actPosPtr);
+// //     throw GDLException("File expression not allowed in this context.");
+//   }
   std::istream& FromStream(std::istream& i)
   {
     throw GDLException("File expression not allowed in this context.");
@@ -134,7 +139,7 @@ static	void operator delete( void *ptr);
     throw GDLException("Assoc_::GetInstance(...) called.");
   }
 
-  int Scalar2index(SizeT& ret) const 
+  int Scalar2Index(SizeT& ret) const 
   { 
     throw GDLException("File expression not allowed in this context.");
   }
@@ -169,12 +174,12 @@ static	void operator delete( void *ptr);
     throw GDLException("File expression not allowed in this context.");
   }
 
-  bool Equal( BaseGDL*)
+  bool Equal( BaseGDL*) const
   {
     throw GDLException("File expression not allowed in this context.");
   }
 
-  bool EqualNoDelete( const BaseGDL*)
+  bool EqualNoDelete( const BaseGDL*) const
   {
     throw GDLException("File expression not allowed in this context.");
   }
@@ -346,7 +351,7 @@ static	void operator delete( void *ptr);
     throw GDLException("File expression not allowed in this context.");
   }
 
-  Parent_* MatrixOp( BaseGDL* r)   
+  Parent_* MatrixOp( BaseGDL* r, bool, bool)   
   {
     throw GDLException("File expression not allowed in this context.");
   }
@@ -385,6 +390,11 @@ static	void operator delete( void *ptr);
   {
     throw GDLException("File expression not allowed in this context.");
   }
+  SizeT OFmtCal( std::ostream* os, SizeT offs, SizeT num, int width, 
+	int minN, char fill, BaseGDL::Cal_IOMode oM = BaseGDL::DEFAULT)
+  {
+    throw GDLException("File expression not allowed in this context.");
+  }
   SizeT IFmtA( std::istream* os, SizeT offset, SizeT num, int width)
   {
     throw GDLException("File expression not allowed in this context.");
