@@ -38,9 +38,19 @@ namespace lib {
 
   using namespace std;
   using namespace antlr;
-
+ 
+  
   BaseGDL* size( EnvT* e) 
   {
+    static int L64Ix = e->KeywordIx( "L64");
+    static int dimIx = e->KeywordIx( "DIMENSIONS");
+    static int FILE_LUNIx = e->KeywordIx( "FILE_LUN");
+    static int N_DIMENSIONSIx = e->KeywordIx( "N_DIMENSIONS");
+    static int N_ELEMENTSIx = e->KeywordIx( "N_ELEMENTS");
+    static int STRUCTUREIx = e->KeywordIx( "STRUCTURE");
+    static int TNAMEIx = e->KeywordIx( "TNAME");
+    static int TYPEIx = e->KeywordIx( "TYPE");
+
     e->NParam( 1); // might be GDL_UNDEF, but must be given
 
     // BaseGDL* p0 = e->GetParDefined( 0); //, "SIZE");
@@ -57,18 +67,16 @@ namespace lib {
     }
 
     // DIMENSIONS
-    static int dimIx = e->KeywordIx( "DIMENSIONS");
-
     if( e->KeywordSet( dimIx)) { 
-      if( Rank == 0) 
-	if( e->KeywordSet(0))
+      if( Rank == 0) {
+	if( e->KeywordSet(L64Ix))
 	  return new DLong64GDL( 0);
 	else
 	  return new DLongGDL( 0);
-
+      }
       dimension dim( Rank);
 
-      if( e->KeywordSet(0)) { // L64
+      if( e->KeywordSet(L64Ix)) { // L64
 	DLong64GDL* res = new DLong64GDL( dim, BaseGDL::NOZERO);
 	(*res)[0] = 0;
 	for( SizeT i=0; i<Rank; ++i) (*res)[ i] = p0->Dim(i);
@@ -81,17 +89,17 @@ namespace lib {
       }
 
     // FILE_LUN
-    } else if( e->KeywordSet(2)) { 
+    } else if( e->KeywordSet(FILE_LUNIx)) { 
 
       e->Throw( "FILE_LUN not supported yet.");
 
     // N_DIMENSIONS
-    } else if( e->KeywordSet(3)) { 
+    } else if( e->KeywordSet(N_DIMENSIONSIx)) { 
 
       return new DLongGDL( Rank);
 
     //N_ELEMENTS
-    } else if( e->KeywordSet(4)) { 
+    } else if( e->KeywordSet(N_ELEMENTSIx)) { 
 
       if( e->KeywordSet(0))
 	return new DULongGDL( nEl);
@@ -99,7 +107,7 @@ namespace lib {
 	return new DLongGDL( nEl);
 
     // STRUCTURE
-    } else if( e->KeywordSet(5)) { 
+    } else if( e->KeywordSet(STRUCTUREIx)) { 
 
 
       DStructGDL* res = new DStructGDL( "IDL_SIZE");
@@ -141,7 +149,7 @@ namespace lib {
       //e->Throw( "STRUCTURE not supported yet.");
 
     // TNAME
-    } else if( e->KeywordSet(6)) { 
+    } else if( e->KeywordSet(TNAMEIx)) { 
 
       if( p0 == NULL)
 	return new DStringGDL( "UNDEFINED");
@@ -149,7 +157,7 @@ namespace lib {
       return new DStringGDL( p0->TypeStr());
 
     // TYPE
-    } else if( e->KeywordSet(7)) { 
+    } else if( e->KeywordSet(TYPEIx)) { 
 
       return new DLongGDL( vType );
 
@@ -157,7 +165,7 @@ namespace lib {
 
       dimension dim( 3 + Rank);
 
-      if( e->KeywordSet(0)) {
+      if( e->KeywordSet(L64Ix)) {
 	DLong64GDL* res = new DLong64GDL( dim, BaseGDL::NOZERO);
 	(*res)[ 0] = Rank;
 	for( SizeT i=0; i<Rank; ++i) (*res)[ i+1] = p0->Dim(i);
@@ -320,6 +328,8 @@ namespace lib {
       {
 	e->Throw( ex.getMessage());
       }
+    assert(false);      
+    return NULL;
   }
 
   BaseGDL* make_array( EnvT* e)
@@ -329,7 +339,7 @@ namespace lib {
     e->AssureLongScalarKWIfPresent( typeIx, type);
 
     DLongGDL* dimKey=NULL;
-    auto_ptr<DLongGDL> dimKey_guard;
+    Guard<DLongGDL> dimKey_guard;
 
     static int sizeix = e->KeywordIx( "SIZE"); 
     static int dimensionix = e->KeywordIx( "DIMENSION"); 
@@ -344,7 +354,7 @@ namespace lib {
 	  {
 	    dimension dim(l_dimension->N_Elements(),1);
 	    dimKey=new DLongGDL(dim, BaseGDL::NOZERO);
-	    dimKey_guard.reset( dimKey); //e->Guard( dimKey);
+	    dimKey_guard.Reset( dimKey); //e->Guard( dimKey);
 	    for (int i=0;i<l_dimension->N_Elements();++i)
 	      (*dimKey)[i]=(*l_dimension)[i];
 	  }
@@ -361,7 +371,7 @@ namespace lib {
 	  {
 	    dimension dim((*l_size)[0],1);
 	    dimKey=new DLongGDL(dim, BaseGDL::NOZERO);
-	    dimKey_guard.reset( dimKey); //e->Guard( dimKey);
+	    dimKey_guard.Reset( dimKey); //e->Guard( dimKey);
 	    //	    e->Guard( dimKey);
 	    for (int i=1;i<=(*l_size)[0];++i)
 	      (*dimKey)[i-1]=(*l_size)[i];
@@ -387,7 +397,7 @@ namespace lib {
 
     static int valueix = e->KeywordIx( "VALUE"); 
     BaseGDL* value = e->GetKW( valueix);
-    auto_ptr<BaseGDL> value_guard;
+    Guard<BaseGDL> value_guard;
     if( value != NULL)
       {
 	if( !value->Scalar())
@@ -398,7 +408,7 @@ namespace lib {
 	else
 	  {
 	    value = value->Convert2( static_cast<DType>(type), BaseGDL::COPY);
-	    value_guard.reset(value);//e->Guard( value);
+	    value_guard.Reset(value);//e->Guard( value);
 	  }
       }
 
@@ -506,14 +516,8 @@ namespace lib {
     return 0;
   }
 
-//   template< typename T>
-//   BaseGDL* reform_template( EnvT* e, dimension dim)
-//   {
-//     T* res = static_cast<T*>( e->GetParDefined( 0)->Dup());
-//     res->SetDim(dim);
-//     return res;
-//   }
 
+  
   BaseGDL* reform( EnvT* e)
   {
     SizeT nParam=e->NParam(1);
@@ -522,13 +526,12 @@ namespace lib {
     BaseGDL* p0 = *p0P;
 
     SizeT nEl = p0->N_Elements();
-    SizeT Rank = p0->Rank();
+//     SizeT Rank = p0->Rank();
 //     if( Rank == 0)
 //       e->Throw( "Parameter must be an array in this context: " 
 // 		+ e->GetParString( 0));
 
-
-    SizeT Type = p0->Type();
+//     SizeT Type = p0->Type();
 
     dimension dim;
 
@@ -548,92 +551,39 @@ namespace lib {
       if( dim.Rank() == 0)
 	dim << 1;
       //     dim.Set(0, j);
-    } else arr( e, dim, 1);
-
+    } 
+    else 
+      arr( e, dim, 1);
 
     if (dim.NDimElements() != nEl) 
       e->Throw( "New subscripts must not change the number of elements in " 
 		+ e->GetParString( 0));
 
-
-    if (e->KeywordSet( "OVERWRITE")) {
-
-      // make a copy if p0 is not global
-      //      if( !e->GlobalPar( 0))
-      //	p0 = p0->Dup();
-      // better: steal p0
-      if( !e->GlobalPar( 0))
-	{
-	  bool success = e->StealLocalPar( 0); 
-	  //*p0P = NULL; // prevent local parameter form deletion
-	  assert( success);
-	}
-
-      p0->SetDim(dim);
-      return p0;
-    }
-
-    // steal local parmeter
+    // make a copy if p0 is not global
+    //      if( !e->GlobalPar( 0))
+    //	p0 = p0->Dup();
+    // better: steal p0
     if( !e->GlobalPar( 0))
       {
 	bool success = e->StealLocalPar( 0); //*p0P = NULL;
+	//*p0P = NULL; // prevent local parameter form deletion
 	assert( success);
-	p0->SetDim( dim);
+	p0->SetDim(dim);
 	return p0;
       }
 
+    static int overwriteIx = e->KeywordIx("OVERWRITE");
+    if (e->KeywordSet( overwriteIx)) 
+    {
+      p0->SetDim(dim);
+      e->SetPtrToReturnValue( p0P);
+      return p0;
+    }
+
     // global paramter - make a copy
     BaseGDL* res = p0->Dup();
     res->SetDim(dim);
     return res;
-
-//     // GDL_BYTE
-//     if (Type == GDL_BYTE) {
-    
-//       return reform_template< DByteGDL>( e, dim);
-    
-//       //GDL_INT
-//     } else if (Type || Type == GDL_INT) {
-    
-//       return reform_template< DIntGDL>( e, dim);
-    
-//       // GDL_UINT
-//     } else if (Type == GDL_UINT) {
-    
-//       return reform_template< DUIntGDL>( e, dim);
-    
-//       // GDL_LONG
-//     } else if (Type == GDL_LONG) {
-    
-//       return reform_template< DLongGDL>( e, dim);
-    
-//       // GDL_ULONG
-//     } else if (Type == GDL_ULONG) {
-    
-//       return reform_template< DULongGDL>( e, dim);
-    
-//       // GDL_LONG64
-//     } else if (Type == GDL_LONG64) {
-    
-//       return reform_template< DLong64GDL>( e, dim);
-    
-//       // GDL_ULONG64
-//     } else if (Type == GDL_ULONG64) {
-    
-//       return reform_template< DULong64GDL>( e, dim);
-      
-//       // GDL_FLOAT
-//     } else if (Type == GDL_FLOAT) {
-    
-//       return reform_template< DFloatGDL>( e, dim);
-    
-//       // GDL_DOUBLE
-//     } else if (Type == GDL_DOUBLE) {
-    
-//       return reform_template< DDoubleGDL>( e, dim);
-    
-//     }
-    
   }
 
   
@@ -650,7 +600,7 @@ namespace lib {
 	DLong curlevnum = callStack.size();
 
     if (e->KeywordSet( "S_FUNCTIONS")) {
-      deque<DString> subList;
+      vector<DString> subList;
 
       SizeT nFun = libFunList.size();
       for( SizeT i = 0; i<nFun; ++i) {
@@ -671,7 +621,7 @@ namespace lib {
     }
 
     if (e->KeywordSet( "S_PROCEDURES")) {
-      deque<DString> subList;
+      vector<DString> subList;
 
       SizeT nPro = libProList.size();
       for( SizeT i = 0; i<nPro; ++i) {
@@ -851,7 +801,7 @@ namespace lib {
       DLong n = proList.size() + funList.size() + 1;
 
       // Add $MAIN$ to list
-      deque<DString> pfList;
+      vector<DString> pfList;
       pfList.push_back("$MAIN$");
 
       // Procedures
