@@ -235,51 +235,56 @@ namespace lib {
     }
   }
 
-  void kwtest( EnvT* e)
-  {
-    StackGuard<EnvStackT> guard( e->Interpreter()->CallStack());
-
-    // here first parameter is the function name
-    // set callF to the function you want to call
-    int nParam=e->NParam();
-    if( nParam == 0)
-     e->Throw( "No function specified.");
-    DString callF;
-    e->AssureScalarPar<DStringGDL>( 0, callF);
- 
-    // this is a function name -> convert to UPPERCASE
-    callF = StrUpCase( callF);
-
-	SizeT funIx = GDLInterpreter::GetFunIx( callF);
-	
-	EnvUDT* newEnv= new EnvUDT( e->CallingNode(), funList[ funIx], (BaseGDL**)NULL);
-
-	// add parameter
-	SizeT widgetID = 999;
-      
-    newEnv->SetNextPar( new DLongGDL(widgetID)); // pass as local
-
-	e->Interpreter()->CallStack().push_back( newEnv);
-	
-	// make the call
-	BaseGDL* res = e->Interpreter()->
-	  call_fun(static_cast<DSubUD*>(newEnv->GetPro())->GetTree());
-
-	// set the keyword to the function's return value
-    static int testIx = e->KeywordIx( "TEST");
-    e->SetKW( testIx, res);
-  }
+//   void kwtest( EnvT* e)
+//   {
+//     StackGuard<EnvStackT> guard( e->Interpreter()->CallStack());
+// 
+//     // here first parameter is the function name
+//     // set callF to the function you want to call
+//     int nParam=e->NParam();
+//     if( nParam == 0)
+//      e->Throw( "No function specified.");
+//     DString callF;
+//     e->AssureScalarPar<DStringGDL>( 0, callF);
+//  
+//     // this is a function name -> convert to UPPERCASE
+//     callF = StrUpCase( callF);
+// 
+// 	SizeT funIx = GDLInterpreter::GetFunIx( callF);
+// 	
+// 	EnvUDT* newEnv= new EnvUDT( e->CallingNode(), funList[ funIx], (BaseGDL**)NULL);
+// 
+// 	// add parameter
+// 	SizeT widgetID = 999;
+//       
+//     newEnv->SetNextPar( new DLongGDL(widgetID)); // pass as local
+// 
+// 	e->Interpreter()->CallStack().push_back( newEnv);
+// 	
+// 	// make the call
+// 	BaseGDL* res = e->Interpreter()->
+// 	  call_fun(static_cast<DSubUD*>(newEnv->GetPro())->GetTree());
+// 
+// 	// set the keyword to the function's return value
+//     static int testIx = e->KeywordIx( "TEST");
+//     e->SetKW( testIx, res);
+//   }
 
   // CALL_EXTERNAL by Christoph Fuchs
+  //AC #ifdef USE_EIGEN
+  //AC SizeT defaultAlign = 16;
+  //AC #else  
   typedef struct {
     char      c;
     long long l;
   } testAlign;
   SizeT defaultAlign = (SizeT)( sizeof(testAlign)-sizeof(long long) );
-
+  //AC #endif
+  
   BaseGDL* call_external( EnvT* e)
   {
     DString image, entry;
+    static std::string s;
     SizeT myAlign      = defaultAlign;
     DType myReturnType = GDL_UNDEF;
 
@@ -341,10 +346,11 @@ namespace lib {
 	e->Throw("Conflicting keywords ALL_VALUE and ALL_GDL");
     }
 
-    short* byValue = (short*) malloc( (nParam-2) * sizeof(short) );
-    if (byValue == NULL) {
-	e->Throw("Internal error allocating memory for byValue");
-    }
+//     short* byValue = (short*) malloc( (nParam-2) * sizeof(short) );
+//     if (byValue == NULL) {
+// 	e->Throw("Internal error allocating memory for byValue");
+//     }
+    vector<short> byValue(nParam-2,0);
 
     for (SizeT i=0; i<nParam-2;i++) {
 	byValue[i] = flagAllValue ? 1 : flagAllGdl ? -1 : 0;
@@ -368,11 +374,13 @@ namespace lib {
     e->AssureStringScalarPar( (SizeT)1, entry);
 
     int argc      = nParam-2;
+    // must be void** for dl... stuff
     void **argv   = (void**)malloc((nParam-2) * sizeof(void*) );
     if (argv == NULL) {
 	e->Throw("Internal error allocating memory for argv");
     }
-
+    GDLGuard<void*,void,void> argvGuard(argv, free);
+    
     // Fill argv with the parameters
 
     for(SizeT i =2; i < nParam; i++){
@@ -386,13 +394,13 @@ namespace lib {
 		);
 	    }
 
-	    if (IsNumericType[pType]) {
+	    if (NumericType(pType)) {
 		if (par->Sizeof() > sizeof(void*)) {
 		    e->Throw("Parameter is larger than pointer: "
 			     + e->GetParString(i)
 		    );
 		}
-		memcpy(argv+i-2, (void*) par->DataAddr(), par->Sizeof());
+		memcpy(&argv[i-2], (void*) par->DataAddr(), par->Sizeof());
 	    }
 	    else if (pType == GDL_STRING) {
 		argv[i-2] = (void*) (*(DStringGDL*)(par))[0].c_str();
@@ -407,7 +415,7 @@ namespace lib {
 	    argv[i-2] = (void*) par;
 	}
 	else {					// By reference (default)
-	    if (IsNumericType[pType] || pType == GDL_PTR || pType == GDL_OBJ ) {
+	    if (NumericType(pType) || pType == GDL_PTR || pType == GDL_OBJ ) {
 		argv[i-2] = (void*) par->DataAddr();
 	    }
 	    else if (pType == GDL_STRING) {
@@ -427,7 +435,7 @@ namespace lib {
 #ifdef _MSC_VER
     HMODULE handle =  LoadLibrary((LPCSTR)image.c_str());
 #else
-    void* handle =  dlopen(image.c_str(),  RTLD_NOW || RTLD_GLOBAL);
+    void* handle =  dlopen(image.c_str(),  RTLD_NOW | RTLD_GLOBAL);
 #endif
     if (handle == NULL) {
 #ifndef _MSC_VER
@@ -477,9 +485,9 @@ namespace lib {
 		      break;
 	case GDL_ULONG64: ret.d_ulong64 = ((DULong64(*)(int, void**))func)(argc, argv);
 		      break;
-	case GDL_STRING:  ret.d_string  = ((char*(*)   (int, void**))func)(argc, argv);
+	case GDL_STRING:  ret.d_string  = ((char*(*)   (int, void**))func)(argc, argv); 
 		      break;
-	default:      e->Throw("Return type not supported: " + myReturnType );
+	default:      e->Throw("Return type not supported: " + i2s(myReturnType) );
 		      break;
     }
 
@@ -490,22 +498,24 @@ namespace lib {
 	while (! dlclose(handle) ) {}
 #endif
     }
-
+// necessary since struct is freed below, i do not see how??? (FIXME)
+    if (myReturnType == GDL_STRING) {s=ret.d_string;}
     // Copy strings and structures back to GDL, free memory
 
-    for(SizeT i = nParam-1; i >= 2; i--){
+    for (SizeT i = nParam - 1; i >= 2; i--) {
 	if (byValue[i-2] != 0) {continue;}
-	BaseGDL* par = e->GetParDefined(i);
-	SizeT pType  = par->Type();
-	if (pType == GDL_STRING) {
-	    ce_StringIDLtoGDL((EXTERN_STRING*) argv[i-2], par, 1);
-	}
-	else if (pType == GDL_STRUCT) {
-	    ce_StructIDLtoGDL( e, argv[i-2], par, 1, myAlign);
-	}
+      BaseGDL* par = e->GetParDefined(i);
+      SizeT pType = par->Type();
+      if (pType == GDL_STRING) {
+        ce_StringIDLtoGDL((EXTERN_STRING*) argv[i - 2], par, 1);
+      }
+      else if (pType == GDL_STRUCT) {
+        ce_StructIDLtoGDL(e, argv[i - 2], par, 1, myAlign);
+      }
     }
 
-    free(argv);
+    // now guarded. s. a.
+    //free(argv);
 
     // Return the return value
 
@@ -528,10 +538,10 @@ namespace lib {
 			break;
         case GDL_ULONG64:   return new DULong64GDL(ret.d_ulong64);
 			break;
-        case GDL_STRING:    return new DStringGDL(ret.d_string);
+        case GDL_STRING:    return new DStringGDL(s);
 			break;
     }
-	    
+    assert( false);
   }
 
 
@@ -606,7 +616,7 @@ namespace lib {
 		SizeT sizeOf;
 		void* source;
 		int   doFree = 0;
-		if (IsNumericType[pType] || pType == GDL_PTR || pType == GDL_OBJ) {
+		if (NumericType(pType) || pType == GDL_PTR || pType == GDL_OBJ) {
 		    source = (void*) member->DataAddr();
 		    length = member->NBytes();
 		    sizeOf = member->Sizeof();
@@ -623,7 +633,7 @@ namespace lib {
 		    sizeOf = 8;
 		}
 		else {
-		    e->Throw("Unsupported type in structure: " + pType );
+		    e->Throw("Unsupported type in structure: " + i2s(pType) );
 		}
 		SizeT align = sizeOf < myAlign ? sizeOf : myAlign;
 		if ((SizeT)p % align) {
@@ -658,7 +668,7 @@ namespace lib {
 		SizeT length;
 		SizeT sizeOf;
 		void* dest;
-		if (IsNumericType[pType]) {
+		if (NumericType(pType)) {
 		    sizeOf = member->Sizeof();
 		}
 		else {
@@ -670,7 +680,7 @@ namespace lib {
 		    p += space;
 		}
 
-		if (IsNumericType[pType] || pType == GDL_PTR || pType == GDL_OBJ) {
+		if (NumericType(pType) || pType == GDL_PTR || pType == GDL_OBJ) {
 		    length = member->NBytes();
 		    dest   = (void*) member->DataAddr();
 		    memcpy(dest, p, length);
@@ -709,7 +719,9 @@ namespace lib {
 	for (SizeT iTag=0; iTag < nTags; iTag++) {
 	    BaseGDL* member = s->GetTag(iTag);
 	    DType    pType  = member->Type();
-	    if (IsNumericType[pType] || pType == GDL_PTR || pType == GDL_OBJ) {
+	    // there is probably no point transfering PTR and OBJ
+	    // but if it fits why restrict it? They should never be used though
+	    if (NumericType(pType) || pType == GDL_PTR || pType == GDL_OBJ) {
 		totalSize += member->NBytes();
 		sizeOf    =  member->Sizeof();
 	    }
