@@ -30,7 +30,7 @@ struct EnvType
   void NullP() { p = NULL;}
   void NullPP() { pp = NULL;}
   
-  bool IsSet() { return (p != NULL || pp != NULL);}
+  bool IsSet() const { return (p != NULL || pp != NULL);}
 
   bool IsP() const { return p != NULL;}
   bool IsPP() const { return pp != NULL;}
@@ -44,9 +44,9 @@ struct EnvType
   BaseGDL* P() const { return p;}
   BaseGDL** PP() const { return pp;}
   BaseGDL*& PRef() { return p;}
-  BaseGDL**& PPRef() { return pp;}
+//   BaseGDL**& PPRef() { return pp;}
   BaseGDL* const& PRefConst() const { return p;}
-  BaseGDL** const& PPRefConst() const { return pp;}
+//   BaseGDL** const& PPRefConst() const { return pp;}
 
 private:
   BaseGDL* p;
@@ -133,15 +133,15 @@ bool Contains( BaseGDL* p) const
     }
   return false;
 }
-void RemoveLoc( BaseGDL* p) 
-  {
-    for( SizeT i=0; i<sz; ++i)
-      if( eArr[i].P() == p) 
-	{
-	  eArr[i].NullP(); 
-	  return;
-	}
-  }
+// void RemoveLoc( BaseGDL* p) 
+//   {
+//     for( SizeT i=0; i<sz; ++i)
+//       if( eArr[i].P() == p) 
+// 	{
+// 	  eArr[i].NullP(); 
+// 	  return;
+// 	}
+//   }
 SizeT size() const { return sz;}
 iterator begin()  { return &eArr[0];}
 iterator end()  { return &eArr[sz];}
@@ -214,10 +214,10 @@ public:
     return env.Contains( p);
   }
 
-  void RemoveLoc( BaseGDL* p) 
-  {
-    env.RemoveLoc( p);
-  }
+//   void RemoveLoc( BaseGDL* p) 
+//   {
+//     env.RemoveLoc( p);
+//   }
 
   void push_back( BaseGDL* p)
   {
@@ -310,13 +310,13 @@ public:
     return NULL;
  }
 
-  // finds the local variable pp points to
-  int FindLocal( BaseGDL** pp)
-  {
-    for( SizeT i=0; i<env.size(); i++)
-      if( &env[i].PRef() == pp) return static_cast<int>(i);
-    return -1;
-  }
+//   // finds the local variable pp points to
+//   int FindLocal( BaseGDL** pp)
+//   {
+//     for( SizeT i=0; i<env.size(); i++)
+//       if( &env[i].PRef() == pp) return static_cast<int>(i);
+//     return -1;
+//   }
 
   // finds the gloabl variable pp
   int FindGlobal( BaseGDL** pp)
@@ -337,11 +337,11 @@ public:
     return NULL;
   }
 
-  BaseGDL* Loc( SizeT ix)
+  BaseGDL* Loc( SizeT ix) const
   {
     return env[ ix].P();
   }
-  BaseGDL** Env( SizeT ix)
+  BaseGDL** Env( SizeT ix) const
   {
     return env[ ix].PP();
   }
