@@ -42,6 +42,7 @@ class DCompiler: public GDLTokenTypes
   DSubUD *pro;                    // actual processed pro/function
 
   bool            activeProCompiled;
+  int             nCompileErrors; // number of compilation errors
 
   //  std::map<std::string,std::deque<RefDNode> > labelList; // resolving goto/label
   CommonListT   ownCommonList; // common blocks not yet added to commonList
@@ -76,7 +77,7 @@ public:
   DCommonBase* CommonDef(const std::string&); // Common block (re)definition
   void CommonDecl(const std::string&);        // Common block declaration
   void CommonVar(DCommonBase*, const std::string&); // Common block variable
-  BaseGDL* Constant( RefDNode n);
+  BaseGDL* ConstantIndex( RefDNode n);
   RefDNode ByReference(RefDNode n_);  // for parameter passing 
   //  bool ByReferenceMaybe(RefDNode n_); // for parameter passing 
   bool IsVar(const std::string&); // variable already defined in actual context?
@@ -89,6 +90,7 @@ public:
   int  NDefLabel();
   bool IsActivePro( DSub* p);
   bool ActiveProCompiled() const { return activeProCompiled;}
+  int NCompileErrors() const { return nCompileErrors;}
   EnvBaseT* GetEnv() const { return env;}
   void SetEnv( EnvBaseT* e) { env = e;}
 };
