@@ -55,22 +55,30 @@ class DeviceSVG : public Graphics
     // we want color
     actStream->scolor( 1);
 
-    // set color map
+    // avoid to set color map 0 -- makes plplot very slow (?)
     PLINT r[ctSize], g[ctSize], b[ctSize];
     actCT.Get( r, g, b);
-    //    actStream->scmap0( r, g, b, ctSize); 
-    actStream->scmap1( r, g, b, ctSize); 
+//    actStream->scmap0( r, g, b, ctSize);
+    actStream->scmap1( r, g, b, ctSize);
 
     actStream->SETOPT( "drvopt","text_clipping=1"); // clear drvopt
 
     actStream->Init();
     
+    // need to be called initially. permit to fix things
+    actStream->ssub(1,1);
+    actStream->adv(0);
     // load font
     actStream->font( 1);
+    actStream->vpor(0,1,0,1);
+    actStream->wind(0,1,0,1);
     actStream->DefaultCharSize();
+   //in case these are not initalized, here is a good place to do it.
+    if (actStream->updatePageInfo()==true)
+    {
+        actStream->GetPlplotDefaultCharSize(); //initializes everything in fact..
 
-    //    (*pMulti)[ 0] = 0;
-    actStream->adv(0);
+    }
   }
 
 public:
