@@ -25,8 +25,6 @@
 #include <vector>
 #include <cstring>
 
-//#include <plplot/plstream.h>
-#include <plplot/plplotP.h>
 #include <plplot/drivers.h>
 
 #include "gdlxstream.hpp"
@@ -55,12 +53,11 @@ class DeviceX: public Graphics
   int decomposed; // false -> use color table
 
 
-  void plimage_gdl(unsigned char *idata, PLINT nx, PLINT ny, 
+  void plimage_gdl(PLStream* pls, unsigned char *idata, PLINT nx, PLINT ny, 
 		   DLong tru, DLong chan)
   {
-    PLINT ix, iy, xm, ym;
-
-    XwDev *dev = (XwDev *) plsc->dev;
+    PLINT ix, iy;
+    XwDev *dev = (XwDev *) pls->dev;
     XwDisplay *xwd = (XwDisplay *) dev->xwd;
     XImage *ximg = NULL, *ximg_pixmap = NULL;
 
@@ -68,13 +65,14 @@ class DeviceX: public Graphics
 
     int (*oldErrorHandler)(Display*, XErrorEvent*);
 
-    if (plsc->level < 3) {
-      plabort("plimage: window must be set up first");
+    //the following 2 tests cannot happen i think. I keep them for safety.
+    if (pls->level < 3) {
+      std::cerr<<"plimage: window must be set up first"<<std::endl; //plabort() not available anymore!
       return ;
     }
 
     if (nx <= 0 || ny <= 0) {
-      plabort("plimage: nx and ny must be positive");
+      std::cerr<<"plimage: nx and ny must be positive"<<std::endl;
       return;
     }
 
@@ -114,16 +112,19 @@ class DeviceX: public Graphics
 
       ncolors = 256;
 
-#if PL_RGB_COLOR == -1
-      free_mem(xwd->cmap1);
-      xwd->cmap1 = (XColor *) calloc(ncolors, (size_t) sizeof(XColor));
-#endif
+//#if PL_RGB_COLOR == -1 //was (always?) set by plplotP.h which we do not use anymore. 
+      if (xwd->ncol1 != ncolors)
+      {
+        free_mem(xwd->cmap1);
+        xwd->cmap1 = (XColor *) calloc(ncolors, (size_t) sizeof(XColor));
+      }
+//#endif
 
       for( SizeT i = 0; i < ncolors; i++ ) {
 
-	xwd->cmap1[i].red   = ToXColor(plsc->cmap1[i].r);
-	xwd->cmap1[i].green = ToXColor(plsc->cmap1[i].g);
-	xwd->cmap1[i].blue  = ToXColor(plsc->cmap1[i].b);
+	xwd->cmap1[i].red   = ToXColor(pls->cmap1[i].r);
+	xwd->cmap1[i].green = ToXColor(pls->cmap1[i].g);
+	xwd->cmap1[i].blue  = ToXColor(pls->cmap1[i].b);
 	xwd->cmap1[i].flags = DoRed | DoGreen | DoBlue;
 	
 	if ( XAllocColor( xwd->display, xwd->map, &xwd->cmap1[i]) == 0)
@@ -132,8 +133,8 @@ class DeviceX: public Graphics
       xwd->ncol1 = ncolors;
     }
 
-    PLINT xoff = (PLINT) (plsc->wpxoff/32767 * dev->width  + 1);
-    PLINT yoff = (PLINT) (plsc->wpyoff/24575 * dev->height + 1);
+    PLINT xoff = (PLINT) (pls->wpxoff/32767 * dev->width  + 1);
+    PLINT yoff = (PLINT) (pls->wpyoff/24575 * dev->height + 1);
     PLINT kx, ky;
 
     XColor curcolor;
@@ -233,7 +234,8 @@ SizeT nOp = kxLimit * kyLimit;
       {
 	long xsize,ysize,xoff,yoff;
 	winList[ wIx]->GetGeometry( xsize, ysize, xoff, yoff);
-
+    PLStream* pls;
+    plgpls( &pls);
 	// window size and pos
 // 	PLFLT xp; PLFLT yp; 
 // 	PLINT xleng; PLINT yleng;
@@ -246,7 +248,7 @@ SizeT nOp = kxLimit * kyLimit;
 
         // number of colors (based on the color depth from PLPlot)
         (*static_cast<DLongGDL*>( dStruct->GetTag( n_colorsTag)))[0] = 
-          1 << (((static_cast<XwDisplay*>((static_cast<XwDev*>(plsc->dev))->xwd))->depth));
+          1 << (((static_cast<XwDisplay*>((static_cast<XwDev*>(pls->dev))->xwd))->depth));
       }	
 
     // window number
@@ -424,7 +426,7 @@ public:
     PLFLT xp; PLFLT yp; 
     PLINT xleng; PLINT yleng;
     PLINT xoff; PLINT yoff;
-    winList[ wIx]->gpage( xp, yp, xleng, yleng, xoff, yoff);
+    winList[ wIx]->plstream::gpage( xp, yp, xleng, yleng, xoff, yoff);
 
     int debug=0;
     if (debug) cout <<xp<<" "<<yp<<" "<<xleng<<" "<<yleng<<" "<<xoff<<" "<<yoff<<endl;
@@ -434,12 +436,25 @@ public:
 
     xleng = xSize;
     yleng = ySize;
-    xoff  = xPos;
-    yoff  = yMaxSize-(yPos+ySize);
+
+    bool noPos=(xPos==-1 && yPos==-1);
+    xPos=max(0,xPos);
+    yPos=max(0,yPos);
+    static PLINT Quadx[4]={xMaxSize-xSize,0,0,xMaxSize-xSize};
+    static PLINT Quady[4]={0,0,yMaxSize-ySize,yMaxSize-ySize};
+    if (noPos)
+    { //no init given, use 4 quadrants:
+        xoff = Quadx[wIx%4];
+        yoff = Quady[wIx%4];
+    } else {
+      xoff  = xPos==0?xMaxSize-xSize:xPos;
+      yoff  = yPos==0?yPos:yMaxSize-(yPos+ySize);
+    }
     if (yoff <= 0) yoff=1;
     
     if (debug) cout <<xp<<" "<<yp<<" "<<xleng<<" "<<yleng<<" "<<xoff<<" "<<yoff<<endl;
-
+    xp=max(xp,1.0);
+    yp=max(yp,1.0);
     winList[ wIx]->spage( xp, yp, xleng, yleng, xoff, yoff);
 
     // no pause on win destruction
@@ -460,23 +475,30 @@ public:
     // we use our own window handling
     winList[ wIx]->SETOPT( "drvopt","usepth=0");
 
-    // set color map
+    // avoid to set color map 0 -- makes plplot very slow (?)
     PLINT r[ctSize], g[ctSize], b[ctSize];
     actCT.Get( r, g, b);
-    //    winList[ wIx]->scmap0( r, g, b, ctSize); 
-    winList[ wIx]->scmap1( r, g, b, ctSize); 
+//    winList[ wIx]->scmap0( r, g, b, ctSize);
+    winList[ wIx]->scmap1( r, g, b, ctSize);
 
     winList[ wIx]->Init();
-    
+// get actual size, and resize to it (overcomes some window managers problems, solves bug #535)
+    bool success = WSize( actWin ,&xleng, &yleng, &xoff, &yoff);
+    ResizeWin((UInt)xleng, (UInt) yleng);
+    // need to be called initially. permit to fix things
+    winList[ wIx]->ssub(1,1);
+    winList[ wIx]->adv(0);
     // load font
     winList[ wIx]->font( 1);
+    winList[ wIx]->vpor(0,1,0,1);
+    winList[ wIx]->wind(0,1,0,1);
     winList[ wIx]->DefaultCharSize();
+    //in case these are not initalized, here is a good place to do it.
+    if (winList[ wIx]->updatePageInfo()==true)
+    {
+        winList[ wIx]->GetPlplotDefaultCharSize(); //initializes everything in fact..
 
-    //    (*pMulti)[ 0] = nx*ny;
-
-    // need to be called initially
-    winList[ wIx]->adv(0);
-
+    }
     // sets actWin and updates !D
     SetActWin( wIx);
 
@@ -561,7 +583,7 @@ public:
 	DString title = "GDL 0";
         DLong xSize, ySize;
         DefaultXYSize(&xSize, &ySize);
-	bool success = WOpen( 0, title, xSize, ySize, 0, 0);
+	bool success = WOpen( 0, title, xSize, ySize, -1, -1);
 	if( !success)
 	  return NULL;
 	if( actWin == -1)
@@ -603,6 +625,118 @@ public:
     if( decomposed) return 1;
     return 0;
   }
+  
+  int OperateCG(XGCValues *gcValues, unsigned long valuemask, bool write) 
+  {
+    PLStream* pls;
+    plgpls( &pls);
+    XwDev *dev = (XwDev *) pls->dev;
+    if( dev == NULL || dev->xwd == NULL)
+    {
+      Graphics* actDevice = Graphics::GetDevice();
+      GDLGStream* newStream = actDevice->GetStream();
+      plgpls( &pls);
+      dev = (XwDev *) pls->dev;
+      if( dev == NULL) 
+      {
+        std::cerr<<"Device not open."<<std::endl;
+        return 0;
+      }
+    }
+    XwDisplay *xwd = (XwDisplay *) dev->xwd;
+    if (write)
+    {
+      XChangeGC(xwd->display, dev->gc, valuemask, gcValues);
+    }
+    else
+    {
+      XGetGCValues(xwd->display, dev->gc, valuemask, gcValues);
+    }
+    return 1;
+  }
+  bool SetGraphicsFunction( DLong value)                
+  { 
+    XGCValues gcValues;
+    gcValues.function   = max(0,min(value,15));
+    if (OperateCG(&gcValues, GCFunction, true)) return true;
+    else return false;
+  }
+  DLong GetGraphicsFunction()                
+  {
+    XGCValues gcValues;
+    if (OperateCG(&gcValues, GCFunction, false)) return (DLong)gcValues.function;
+    else return -1;
+  }
+  bool CursorStandard(int cursorNumber)
+  {
+    PLStream* pls;
+    plgpls( &pls);
+    int num=max(0,min(XC_num_glyphs-1,cursorNumber));
+    XwDev *dev = (XwDev *) pls->dev;
+    if( dev == NULL || dev->xwd == NULL)
+    {
+      Graphics* actDevice = Graphics::GetDevice();
+      GDLGStream* newStream = actDevice->GetStream();
+      plgpls( &pls);
+      dev = (XwDev *) pls->dev;
+      if( dev == NULL)
+      {
+        std::cerr<<"Device not open."<<std::endl;
+        return 0;
+      }
+    }
+    XwDisplay *xwd = (XwDisplay *) dev->xwd;
+    XDefineCursor(xwd->display,dev->window,XCreateFontCursor(xwd->display,num));
+    return true;
+  }
+  bool CursorCrosshair()
+  {
+    return CursorStandard(XC_crosshair);
+  }
+  
+  void ResizeWin(UInt width, UInt height)
+  {
+    PLStream* pls;
+    plgpls( &pls);
+    XwDev *dev = (XwDev *) pls->dev;
+    if( dev == NULL) return;
+    XwDisplay *xwd = (XwDisplay *) dev->xwd;
+    XResizeWindow(xwd->display, dev->window, width, height);
+  }
+  
+  bool UnsetFocus()
+  {
+    PLStream* pls;
+    plgpls( &pls);
+    XwDev *dev = (XwDev *) pls->dev;
+    if( dev == NULL) return false;
+    XwDisplay *xwd = (XwDisplay *) dev->xwd;
+    XWMHints gestw;
+    gestw.input = FALSE;
+    gestw.flags = InputHint;
+    XSetWMHints(xwd->display, dev->window, &gestw);
+    return true;
+  }
+  bool EnableBackingStore(bool enable)
+   {
+    PLStream* pls;
+    plgpls( &pls);
+    XwDev *dev = (XwDev *) pls->dev;
+    if( dev == NULL) return false;
+    XwDisplay *xwd = (XwDisplay *) dev->xwd;
+ 	XSetWindowAttributes attr;
+    if (enable)
+    {
+	 attr.backing_store = Always;
+    }
+    else
+    {
+	 attr.backing_store = NotUseful;
+    }
+    XChangeWindowAttributes(xwd->display, dev->window,CWBackingStore,&attr);
+    return true;
+  }
+
 
   int MaxWin() { ProcessDeleted(); return winList.size();}
   int ActWin() { ProcessDeleted(); return actWin;}
@@ -621,13 +755,15 @@ public:
     //BadMatch error, and if you read the XGetImage doc you'll see that such errors are prone to happen
     //as soon as part of the window is obscured.
     int (*oldErrorHandler)(Display*, XErrorEvent*);
-
-    XwDev *dev = (XwDev *) plsc->dev;
+    PLStream* pls;
+    plgpls( &pls);
+    XwDev *dev = (XwDev *) pls->dev;
     if( dev == NULL || dev->xwd == NULL)
     {
       GDLGStream* newStream = actDevice->GetStream();
       //already done: newStream->Init();
-      dev = (XwDev *) plsc->dev;
+      plgpls( &pls);
+      dev = (XwDev *) pls->dev;
       if( dev == NULL) e->Throw( "Device not open.");
     }
 
@@ -799,12 +935,14 @@ public:
       perm[1] = 0;
       perm[2] = 2;
       return res->Transpose( perm);
-    } else if (tru = 3) {
+    } else if (tru == 3) {
       perm[0] = 1;
       perm[1] = 2;
       perm[2] = 0;
       return res->Transpose( perm);
     }
+    assert( false);
+    return NULL;
   }
 
   void TV( EnvT* e)
@@ -812,6 +950,7 @@ public:
     //    Graphics* actDevice = Graphics::GetDevice();
 
     SizeT nParam=e->NParam( 1); 
+    PLStream* pls;
 
     GDLGStream* actStream = GetStream();
     if( actStream == NULL)
@@ -822,8 +961,8 @@ public:
 
     //    actStream->NextPlot( false);
     actStream->NoSub();
-
-    XwDev *dev = (XwDev *) plsc->dev;
+    plgpls( &pls);
+    XwDev *dev = (XwDev *) pls->dev;
     XwDisplay *xwd = (XwDisplay *) dev->xwd;
 
     int xSize, ySize, xPos, yPos;
@@ -843,7 +982,7 @@ public:
     //DByteGDL* p0B = e->GetParAs<DByteGDL>( 0);
     DByteGDL* p0B;
     p0B =static_cast<DByteGDL*>(p0->Convert2(GDL_BYTE,BaseGDL::COPY));
-    e->Guard( p0B);
+    e->DeleteAtExit( p0B);
     
     int width, height;
     DLong tru=0;
@@ -943,9 +1082,9 @@ public:
     if (channel < 0 || channel > 3)
       e->Throw("Value of Channel is out of allowed range.");
 
-    std::auto_ptr<BaseGDL> chan_guard;
+    Guard<BaseGDL> chan_guard;
     if (channel == 0) {
-      plimage_gdl(&(*p0B)[0], width, height, tru, channel);
+      plimage_gdl(pls, &(*p0B)[0], width, height, tru, channel);
     } else if (rank == 3) {
       // Rank == 3 w/channel
       SizeT dims[2];
@@ -957,11 +1096,11 @@ public:
 	(*p0B_chan)[i/3] = (*p0B)[i];
       }
       // Send just single channel
-      plimage_gdl(&(*p0B_chan)[0], width, height, tru, channel);
-      chan_guard.reset( p0B_chan); // delete upon exit
+      plimage_gdl(pls, &(*p0B_chan)[0], width, height, tru, channel);
+      chan_guard.Init( p0B_chan); // delete upon exit
     } else if (rank == 2) {
       // Rank = 2 w/channel
-      plimage_gdl(&(*p0B)[0], width, height, tru, channel);
+      plimage_gdl(pls, &(*p0B)[0], width, height, tru, channel);
     }
   }
 
@@ -985,8 +1124,9 @@ public:
 
   static void DefaultXYSize(DLong *xSize, DLong *ySize)
   {
-    *ySize = 640;
+    *xSize = 640;
     *ySize = 512;
+    //GD: normally here we always have HAVE_X true, no? 
 #ifdef HAVE_X
     Display* display = XOpenDisplay(NULL);
     if (display != NULL)
@@ -996,6 +1136,13 @@ public:
       XCloseDisplay(display);
     }   
 #endif
+    bool noQscreen=true;
+    string gdlQscreen=GetEnvString("GDL_GR_X_QSCREEN");
+    if( gdlQscreen == "1") noQscreen=false;
+    string gdlXsize=GetEnvString("GDL_GR_X_WIDTH");
+    if( gdlXsize != "" && noQscreen ) *xSize=atoi(gdlXsize.c_str()); 
+    string gdlYsize=GetEnvString("GDL_GR_X_HEIGHT");
+    if( gdlYsize != "" && noQscreen) *ySize=atoi(gdlYsize.c_str()); 
   }
   
   static void MaxXYSize(DLong *xSize, DLong *ySize)
