@@ -154,13 +154,20 @@ class DeviceZ: public Graphics
     actStream->SETOPT( "drvopt","text=0"); // clear drvopt
 
     actStream->Init();
-    
+   // need to be called initially. permit to fix things
+    actStream->ssub(1,1);
+    actStream->adv(0);
     // load font
     actStream->font( 1);
+    actStream->vpor(0,1,0,1);
+    actStream->wind(0,1,0,1);
     actStream->DefaultCharSize();
+   //in case these are not initalized, here is a good place to do it.
+    if (actStream->updatePageInfo()==true)
+    {
+        actStream->GetPlplotDefaultCharSize(); //initializes everything in fact..
 
-    //    (*pMulti)[ 0] = 0;
-    actStream->adv(0);
+    }
   }
 
 public:
@@ -316,8 +323,8 @@ public:
       }
     }
 
-    //    actStream->vpor( 0, 1.0, 0, 1.0);
-    // actStream->wind( 1-xLL, xsize-xLL, 1-yLL, ysize-yLL);
+    actStream->vpor( 0, 1.0, 0, 1.0);
+    actStream->wind( 1-xLL, xsize-xLL, 1-yLL, ysize-yLL);
 
     DByteGDL* p0B = e->GetParAs<DByteGDL>( 0);
     SizeT rank = p0B->Rank();
