@@ -68,6 +68,8 @@ DLong                   GDLInterpreter::stepCount;
 ProgNode GDLInterpreter::NULLProgNode;
 ProgNodeP GDLInterpreter::NULLProgNodeP = &GDLInterpreter::NULLProgNode;
 
+void LibInit(); // defined in libinit.cpp
+
 DInterpreter::DInterpreter(): GDLInterpreter()
 {
 //  DataStackT::Init();
@@ -113,14 +115,14 @@ RetCode GDLInterpreter::NewInterpreterInstance( SizeT lineOffset)
   return static_cast<DInterpreter*>( this)->InnerInterpreterLoop(lineOffset);
 }
 
-DStructGDL* GDLInterpreter::ObjectStruct( BaseGDL* self, ProgNodeP mp)
+DStructGDL* GDLInterpreter::ObjectStruct( DObjGDL* self, ProgNodeP mp)
 {
-  DType selfType = self->Type();
-  if( selfType != GDL_OBJ) 
-    throw GDLException( mp, "Object reference type"
-			" required in this context: "+Name(self));
+//   DType selfType = self->Type();
+//   if( selfType != GDL_OBJ) 
+//     throw GDLException( mp, "Object reference type"
+// 			" required in this context: "+Name(self));
 
-  DObjGDL* obj=static_cast<DObjGDL*>(self);
+  DObjGDL* obj=self;//static_cast<DObjGDL*>(self);
 
   SizeT o;
   if( !obj->Scalar( o))
@@ -143,22 +145,200 @@ DStructGDL* GDLInterpreter::ObjectStruct( BaseGDL* self, ProgNodeP mp)
   return oStructGDL;
 }
 
-DStructGDL* GDLInterpreter::ObjectStructCheckAccess( BaseGDL* self, ProgNodeP mp)
-{
-  DStructGDL* oStruct = ObjectStruct( self, mp);
-  
-  // check accessibility
-  DStructDesc* desc = oStruct->Desc();
-  if( !desc->IsParent( callStack.back()->GetPro()->Object()))
-    {
-      throw GDLException( mp, "Object of type "+desc->Name()+
-			  " is not accessible within "+
-			  callStack.back()->GetProName() + ": "+Name(self));
-    }
-  
-  return oStruct;
+void GDLInterpreter::SetRootL( ProgNodeP tt, DotAccessDescT* aD, BaseGDL* r, ArrayIndexListT* aL) 
+{ 
+  if( r->Type() == GDL_STRUCT)
+  {
+      if( r->IsAssoc())
+	  {
+	      ArrayIndexListGuard guard( aL);
+	      throw GDLException( tt, "File expression not allowed "
+				  "in this context: "+Name(r),true,false);
+	  }
+      DStructGDL* structR=static_cast<DStructGDL*>(r);
+      aD->ADRoot(structR, aL); 
+  }
+  else
+  {
+      if( r->Type() != GDL_OBJ)
+	  {
+	      throw GDLException( tt, "Expression must be a"
+				  " STRUCT in this context: "+Name(r),
+				  true,false);
+	  }
+
+      ArrayIndexListGuard guard( aL);
+
+      DStructGDL* oStruct = ObjectStruct( static_cast<DObjGDL*>(r), tt);
+      DStructDesc* desc = oStruct->Desc();
+
+      bool isObj = callStack.back()->IsObject(); // called from member subroutine?
+
+      if( desc->IsParent( GDL_OBJECT_NAME))
+	  {
+	    SizeT sss = 0;
+	    SizeT ooo = 0;
+	    if( isObj)
+	    {
+	      static_cast<DObjGDL*>(r)->Scalar( ooo); // checked in ObjectStruct
+
+	      BaseGDL* self = callStack.back()->GetKW(callStack.back()->GetPro()->NKey()); // SELF
+
+	      assert( dynamic_cast<DObjGDL*>(self) != NULL);
+
+	      if( !static_cast<DObjGDL*>(self)->Scalar( sss))
+		  throw GDLException( tt, "Internal error: SELF Object reference"
+				    " must be scalar in this context: "+Name(self));
+
+	      assert( sss != 0);
+	    }
+
+	    if( !isObj || (sss != ooo))
+	    {
+	      // call SetProperty
+	      throw GDLException( tt, "Calling SetProperty not yet implemented: "+Name(r));
+	      //return;
+	    }
+	  }
+
+      if( isObj) // member access to object?
+	  {
+	      if( !desc->IsParent( callStack.back()->GetPro()->Object()))
+		  {
+		      throw GDLException( tt, "Object of type "+desc->Name()+
+					  " is not accessible within "+
+					  callStack.back()->GetProName() + 
+					  ": "+Name(r));
+		  }
+	      // DStructGDL* oStruct = 
+	      //        ObjectStructCheckAccess( static_cast<DObjGDL*>(r), tt);
+
+	      // oStruct cannot be "Assoc_"
+	      aD->ADRoot( oStruct, guard.release()); 
+	  }
+      else
+	  {
+	      throw GDLException( tt, "Expression must be a"
+				  " STRUCT in this context: "+Name(r),
+				  true,false);
+	  }
+  }
 }
 
+void GDLInterpreter::SetRootR( ProgNodeP tt, DotAccessDescT* aD, BaseGDL* r, ArrayIndexListT* aL) 
+{ 
+// check here for object and get struct
+if( r->Type() == GDL_STRUCT)
+  {
+      if( r->IsAssoc())
+	  {
+	      ArrayIndexListGuard guard( aL);
+	      throw GDLException( tt, "File expression not allowed "
+				  "in this context: "+Name(r),true,false);
+	  }
+      DStructGDL* structR=static_cast<DStructGDL*>(r);
+      aD->ADRoot( structR, aL); 
+  }
+else
+  {
+      ArrayIndexListGuard guard( aL);
+
+      if( r->Type() != GDL_OBJ)
+	  {
+	      throw GDLException( tt, "Expression must be a"
+				  " STRUCT in this context: "+Name(r),
+				  true,false);
+	  }
+
+      DStructGDL* oStruct = ObjectStruct( static_cast<DObjGDL*>(r), tt);
+      DStructDesc* desc = oStruct->Desc();
+
+      bool isObj = callStack.back()->IsObject();
+
+      if( desc->IsParent( GDL_OBJECT_NAME))
+	  {
+	    SizeT sss = 0;
+	    SizeT ooo = 0;
+	    if( isObj)
+	    {
+	      static_cast<DObjGDL*>(r)->Scalar( ooo); // checked in ObjectStruct
+
+	      BaseGDL* self = callStack.back()->GetKW(callStack.back()->GetPro()->NKey()); // SELF
+
+	      assert( dynamic_cast<DObjGDL*>(self) != NULL);
+
+	      if( !static_cast<DObjGDL*>(self)->Scalar( sss))
+		  throw GDLException( tt, "Internal error: SELF Object reference"
+				    " must be scalar in this context: "+Name(self));
+
+	      assert( sss != 0);
+	    }
+
+	    if( !isObj || (sss != ooo))
+	    {
+	      // call GetProperty
+	      throw GDLException( tt, "Calling GetProperty not yet implemented: "+Name(r));
+
+	      //aD->ADRootGetProperty( oStruct, guard.release()); 
+	      return;
+	    }
+	  }
+
+      if( isObj)
+	  {
+	      if( !desc->IsParent( callStack.back()->GetPro()->Object()))
+		  {
+		      throw GDLException( tt, "Object of type "+desc->Name()+
+					  " is not accessible within "+
+					  callStack.back()->GetProName() + 
+					  ": "+Name(r));
+		  }
+	      // DStructGDL* oStruct = 
+	      //     ObjectStructCheckAccess( static_cast<DObjGDL*>(r), tt);
+
+	      if( aD->IsOwner()) delete r; 
+	      aD->SetOwner( false); // object struct, not owned
+	      
+	      aD->ADRoot( oStruct, guard.release()); 
+	  }
+      else
+	  {
+	      throw GDLException( tt, "Expression must be a"
+				  " STRUCT in this context: "+Name(r),true,false);
+	  }
+  }
+}
+
+// DStructDesc* GDLInterpreter::GDLObjectDesc( DStructGDL* oStruct, ProgNodeP mp)
+// {
+//   //DStructGDL* oStruct = ObjectStruct( self, mp);
+//   
+//   // check accessibility
+//   DStructDesc* desc = oStruct->Desc();
+//   if( !desc->IsParent( GDL_OBJECT_NAME))
+//     {
+//       return NULL;
+//     }
+//   
+//   return desc;
+// }
+// 
+// void GDLInterpreter::ObjectStructCheckAccess( DStructGDL* oStruct, ProgNodeP mp)
+// {
+//   //DStructGDL* oStruct = ObjectStruct( self, mp);
+//   
+//   // check accessibility
+//   DStructDesc* desc = oStruct->Desc();
+//   if( !desc->IsParent( callStack.back()->GetPro()->Object()))
+//     {
+//       throw GDLException( mp, "Object of type "+desc->Name()+
+// 			  " is not accessible within "+
+// 			  callStack.back()->GetProName() + ": "+Name(self));
+//     }
+//   
+//   //return oStruct;
+// }
+
 // searches and compiles procedure (searchForPro == true) or function (searchForPro == false)  'pro'
 bool GDLInterpreter::SearchCompilePro(const string& pro, bool searchForPro) 
 {
@@ -333,7 +513,7 @@ void GDLInterpreter::AdjustTypes(BaseGDL* &a, BaseGDL* &b)
   DType aTy=a->Type();
   DType bTy=b->Type();
   if( aTy == bTy) return;
-  if( aTy > 100 || bTy > 100)
+  if( DTypeOrder[aTy] > 100 || DTypeOrder[bTy] > 100)
     {
       //exception
       throw GDLException( "Expressions of this type cannot be converted.");
@@ -467,6 +647,15 @@ void AppendExtension( string& argstr)
     }
 }
 
+DInterpreter::CommandCode DInterpreter::CmdReset()
+{
+  RetAll( RetAllException::RESET);
+}
+DInterpreter::CommandCode DInterpreter::CmdFullReset()
+{
+  RetAll( RetAllException::FULL_RESET);
+}
+
 DInterpreter::CommandCode DInterpreter::CmdCompile( const string& command)
 {
   string cmdstr = command;
@@ -623,8 +812,7 @@ DInterpreter::CommandCode DInterpreter::ExecuteCommand(const string& command)
     }
   if( cmd( "FULL_RESET_SESSION"))
     {
-      cout << "FULL_RESET_SESSION not implemented yet." << endl;
-      return CC_OK;
+      return CmdFullReset();
     }
   if( cmd( "GO"))
     {
@@ -647,8 +835,7 @@ DInterpreter::CommandCode DInterpreter::ExecuteCommand(const string& command)
     }
   if( cmd( "RESET_SESSION"))
     {
-      cout << "RESET_SESSION not implemented yet." << endl;
-      return CC_OK;
+      return CmdReset();
     }
   if( cmd( "RNEW"))
     {
@@ -771,6 +958,20 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
       return ExecuteCommand( line.substr(1));
     }
 
+  // command
+  if( firstChar == "?") 
+    {
+      // later, we will have to check whether we have X11/Display or not
+      // on some computing nodes on supercomputers, this is de-activated.
+      if (line.substr(1).length() > 0) {
+	line=line.substr(1);
+	StrTrim(line);
+	line="online_help, '"+line+"'"; //'
+      } else {
+	line="online_help";
+      }
+    }
+  
   // shell command
   if( firstChar == "$") 
     {
@@ -805,7 +1006,7 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
 
   RefDNode theAST;
   try { 
-    auto_ptr<GDLLexer> lexer;
+    Guard<GDLLexer> lexer;
 
     // LineContinuation LC
     // conactenate the strings and insert \n
@@ -815,17 +1016,17 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
     int lCNum = 0;
     for(;;) 
       {
-	lexer.reset( new GDLLexer(executeLine, "", callStack.back()->CompileOpt()));
+	lexer.Reset( new GDLLexer(executeLine, "", callStack.back()->CompileOpt()));
 	try {
 	  // works, but ugly -> depends from parser detecting an error
 	  // (which it always will due to missing END_U token in case of LC)
  	  //lexer->Parser().SetCompileOpt(callStack.back()->CompileOpt());
- 	  lexer->Parser().interactive();
+ 	  lexer.Get()->Parser().interactive();
 	  break; // no error -> everything ok
 	}
 	catch( GDLException& e)
 	  {
-	    int lCNew = lexer->LineContinuation();
+	    int lCNew = lexer.Get()->LineContinuation();
 	    if( lCNew == lCNum)
 // 	      throw; // no LC -> real error
 	{
@@ -868,7 +1069,7 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
       } 
     
     //    lexer->Parser().interactive();
-    theAST = lexer->Parser().getAST();
+    theAST = lexer.Get()->Parser().getAST();
 
   }
   catch( GDLException& e)
@@ -946,7 +1147,7 @@ DInterpreter::CommandCode DInterpreter::ExecuteLine( istream* in, SizeT lineOffs
       cerr << "Compiler exception: " <<  e.getMessage() << endl;
       return CC_OK;
     }
-  auto_ptr< ProgNode> progAST_guard( progAST);
+  Guard< ProgNode> progAST_guard( progAST);
 
   try
     {
@@ -1372,7 +1573,7 @@ RetCode DInterpreter::InterpreterLoop( const string& startup,
   rl_event_hook = GDLEventHandler;
   {
     int edit_input = SysVar::Edit_Input();
-    stifle_history(edit_input == 1 || edit_input < 0 ? 20 : edit_input);
+    stifle_history(edit_input == 1 || edit_input < 0 ? 200 : edit_input);
   }
   
   // Eventually read back the ".gdl" path in user $HOME
@@ -1454,6 +1655,36 @@ historyIntialized = true;
     catch( RetAllException& retAllEx)
       {
 	runCmd = (retAllEx.Code() == RetAllException::RUN);
+	bool resetCmd = (retAllEx.Code() == RetAllException::RESET);
+	bool fullResetCmd = (retAllEx.Code() == RetAllException::FULL_RESET);
+	if( resetCmd || fullResetCmd)
+	{
+	  // remove $MAIN$
+	  delete callStack.back();
+	  callStack.pop_back();
+	  assert( callStack.empty());
+	  
+	  ResetObjects();
+	  ResetHeap();
+	  if( fullResetCmd)
+	  {
+	    PurgeContainer(libFunList);
+	    PurgeContainer(libProList);
+	  }
+	  // initially done in InitGDL()
+	  // initializations
+	  InitObjects();
+	  // init library functions
+	  if( fullResetCmd)
+	  {
+	      LibInit(); 
+	  }
+	  
+	  // initially done in constructor: setup main level environment
+	  DPro* mainPro=new DPro();        // $MAIN$  NOT inserted into proList
+	  EnvUDT* mainEnv=new EnvUDT(NULL, mainPro);
+	  callStack.push_back(mainEnv);   // push main environment (necessary)
+	}
       }
     catch( exception& e)
       {
