@@ -61,7 +61,7 @@ DLib::DLib( const string& n, const string& o, const int nPar_,
   key.resize(nKey_);
   for( SizeT k=0; k<nKey_; ++k) key[k]=keyNames[k];
 
-  if( nKey_ >= 1)
+  if( nKey_ >= 1) {
     if( keyNames[0] == "_EXTRA")
       {
 	extra = EXTRA;
@@ -72,6 +72,7 @@ DLib::DLib( const string& n, const string& o, const int nPar_,
 	extra = REFEXTRA;
 	extraIx = 0;
       }
+  }
 
   SizeT nWarnKey_=0;
   if( warnKeyNames != NULL)
@@ -247,7 +248,7 @@ unsigned DSubUD::AddVar(const string& v)
   return var.size()-1;
 }
 
-void DSubUD::AddKey(const string& k, const string& v)
+DSubUD* DSubUD::AddKey(const string& k, const string& v)
 {
   if( k == "_REF_EXTRA")
     {
@@ -269,8 +270,19 @@ void DSubUD::AddKey(const string& k, const string& v)
     {
       if(extraIx != -1) extraIx++; // update extra ix index
     }
-  key.push_front(k);
-  var.push_front(v);
+  // as we only push_front during compilation, we better use a vector here
+//   key.push_front(k);
+//   var.push_back(v);
+  // we want push_front
+  key.resize( key.size() + 1);
+  for( int i= key.size()-1; i>0; --i)
+    key[ i] = key[ i-1];
+  key[ 0] = k;
+  var.resize( var.size() + 1);
+  for( int i= var.size()-1; i>0; --i)
+    var[ i] = var[ i-1];
+  var[ 0] = v;
+  return this;
 }
 
 // must be done after the tree is converted
