@@ -62,14 +62,15 @@ public:
   // upon library routines exit (normal or on error)
   // elimates the need of auto_ptr and in some places later destruction is needed
   
-  void Guard( BaseGDL* toGuard)
+  void DeleteAtExit( BaseGDL* toGuard)
     {
-// 		if( toDestroy == NULL)
-// 			toDestroy = new ContainerT();
-		toDestroy.push_back( toGuard);
-	}
+//	if( toDestroy == NULL)
+//		toDestroy = new ContainerT();
+      toDestroy.push_back( toGuard);
+    }
 
 protected:
+
   // for obj cleanup
   static std::set< DObj> inProgress;
   
@@ -85,10 +86,66 @@ protected:
   EnvBaseT*      newEnv;
 
   // finds the local variable pp points to
-  int FindLocalKW( BaseGDL** pp) { return env.FindLocal( pp);}
+//   int FindLocalKW( BaseGDL** pp) { return env.FindLocal( pp);}
+private:
+    BaseGDL** ptrToReturnValue;
+public:
+
+  void SetPtrToReturnValue(BaseGDL** p) { ptrToReturnValue = p;}
+  BaseGDL** GetPtrToReturnValueNull() { BaseGDL** p = ptrToReturnValue;ptrToReturnValue=NULL;return p;}
+  BaseGDL** GetPtrToReturnValue() const { return ptrToReturnValue;}
+  BaseGDL** GetPtrToGlobalReturnValue() 
+  { 
+    if( ptrToReturnValue == NULL)
+      return NULL;
+    if( env.InLoc(ptrToReturnValue))
+    {
+      *ptrToReturnValue = NULL; // steal local value
+      return NULL; // return as not global
+    }
+    return ptrToReturnValue;
+    
+  }
+
   // used by the interperter returns the keyword index, used for UD functions
+  // and used by WRAPPED subroutines
   int GetKeywordIx( const std::string& k);
 
+  
+  bool StealLocalKW( SizeT ix) 
+  { 
+    if( LocalKW( ix))
+      {
+	env.Clear( ix);
+	return true;
+      }
+    return false;
+  }
+
+//   bool StealLocalKW( BaseGDL** ref) 
+//   { 
+//    if( !env.InLoc( pp))
+//      return false;
+// 
+//    *ref = NULL;
+//     return true;
+//   }
+
+  bool LocalKW( SizeT ix) const
+  {
+    if( ix >= env.size()) return false;
+    return ( env.Loc( ix) != NULL);
+  }
+
+  bool GlobalKW( SizeT ix) const
+  {
+    if( ix >= env.size()) return false;
+    return ( env.Env( ix) != NULL);
+  }
+
+  bool KeywordSet( SizeT ix);
+
+protected:
   // for HEAP_GC
   static void AddStruct( DPtrListT& ptrAccessible,  DPtrListT& objAccessible, 
 		  DStructGDL* stru);
@@ -98,6 +155,11 @@ protected:
 	       DObjGDL* obj);
   static void Add( DPtrListT& ptrAccessible, DPtrListT& objAccessible, 
 	    BaseGDL* p);
+  
+  // definition in list.cpp
+  static void AddLIST( DPtrListT& ptrAccessible,
+		        DPtrListT& objAccessible, DStructGDL* listStruct);
+
 
 public:
 // 	void DebugInfo()
@@ -111,8 +173,8 @@ public:
   virtual void ObjCleanup( DObj actID);
 
   // for CLEANUP calls due to reference counting
-  void PushNewEmptyEnvUD(  DSub* newPro, BaseGDL** newObj = NULL);
-  void PushNewEmptyEnvUDWithExtra(  DSub* newPro, BaseGDL** newObj = NULL);
+  void PushNewEmptyEnvUD(  DSubUD* newPro, DObjGDL** newObj = NULL);
+//   void PushNewEmptyEnvUDWithExtra(  DSubUD* newPro, BaseGDL** newObj = NULL);
   
   void AddEnv( DPtrListT& ptrAccessible, DPtrListT& objAccessible);
   void AddToDestroy( DPtrListT& ptrAccessible, DPtrListT& objAccessible);
@@ -177,7 +239,7 @@ public:
   // checks if pp points to a local variable
   bool IsLocalKW( BaseGDL** pp) const { return env.InLoc( pp);}
 
-  void RemoveLoc( BaseGDL* p) { env.RemoveLoc( p);}
+//   void RemoveLoc( BaseGDL* p) { env.RemoveLoc( p);}
 
   // called after parameter definition
   void ResolveExtra();
@@ -221,9 +283,9 @@ public:
   void SetKeyword( const std::string& k, BaseGDL** const val); // reference
 
   // to check if a lib function returned a variable of this env
-  bool Contains( BaseGDL* p) const;
+//   bool Contains( BaseGDL* p) const;
 
-  BaseGDL** GetPtrTo( BaseGDL* p);
+//   BaseGDL** GetPtrTo( BaseGDL* p);
 
   DInterpreter* Interpreter() const { return interpreter;}
 
@@ -266,14 +328,10 @@ public:
   BaseGDL*& GetParDefined(SizeT i); //, const std::string& subName = "");
   bool KeywordPresent( SizeT ix)
   { return (env.Loc(ix)!=NULL)||(env.Env(ix)!=NULL);}
-  bool GlobalKW( SizeT ix) 
-  {
-    if( ix >= env.size()) return false;
-    return ( env.Env( ix) != NULL);
-  }
-    void SetNextParUnckeckedVarNum(BaseGDL** arg1);
+  void SetNextParUnckeckedVarNum(BaseGDL** arg1);
 
   friend class DInterpreter; // gcc 4.4 compatibility
+  friend class InProgressGuard;
 };
 
 
@@ -343,7 +401,7 @@ void resize( SizeT s)
 		return;
     }
     // this should never happen (or only in extreme rarely cases)
-    // hence the performance will go down
+    // the performance will go down
     // s > defaultLength
     T* newArr = new T[ s]; // ctor called
 	if( eArr != reinterpret_cast<T*>(buf))
@@ -368,8 +426,8 @@ void resize( SizeT s)
 // T operator[]( SizeT i) const { assert( i<sz);  return eArr[i];}
 T& operator[]( SizeT i) { assert( i<sz);  return eArr[i];}
 SizeT size() const { return sz;}
-iterator begin()  { return &eArr[0];}
-iterator end()  { return &eArr[sz];}
+iterator begin() const { return &eArr[0];}
+iterator end() const { return &eArr[sz];}
 bool empty() const { return sz == 0;}
 T& front() { return eArr[0];}
 const T& front() const { return eArr[0];}
@@ -382,12 +440,19 @@ const T& back() const { return eArr[sz-1];}
 // for UD subroutines (written in GDL) ********************************
 class EnvUDT: public EnvBaseT
 {
-static std::deque< void*> freeList;
+// static std::deque< void*> freeList;
+static FreeListT freeList;
 
 public:
 static 	void* operator new( size_t bytes);
 static	void operator delete( void *ptr);
 
+enum CallContext {
+   RFUNCTION = 0
+  ,LFUNCTION
+  ,LRFUNCTION
+};
+
 private:
 ForInfoListT<ForLoopInfoT, 32> forLoopInfo;
 // std::vector<ForLoopInfoT> forLoopInfo;
@@ -396,7 +461,7 @@ ForInfoListT<ForLoopInfoT, 32> forLoopInfo;
   DLong             onError; // on_error setting
   BaseGDL**         catchVar;
   ProgNodeP         catchNode; 
-  bool              lFun; // assignment paramter for functions as l_value
+  CallContext       callContext; // assignment paramter for functions as l_value
   SizeT             nJump; // number of jumps in current environment
   int               lastJump; // to which label last jump went
   
@@ -407,22 +472,25 @@ public:
   void ResizeForLoops( int newSize) { forLoopInfo.resize(newSize);}
 
   // UD pro/fun
-  EnvUDT( ProgNodeP idN, DSub* pro_, bool lF = false);
+  EnvUDT( ProgNodeP idN, DSubUD* pro_, CallContext lF = RFUNCTION);
 
   // member procedure
   EnvUDT( ProgNodeP idN, BaseGDL* self, 
 	const std::string& parent="");
   // member function
   EnvUDT( BaseGDL* self, ProgNodeP idN, 
-	const std::string& parent="", bool lF = false);
+	const std::string& parent="", CallContext lF = RFUNCTION);
 
 
   // for obj_new and obj_destroy
   //EnvUDT( EnvBaseT* pEnv, DSub* newPro, BaseGDL** self); 
-  EnvUDT( ProgNodeP callindNode_, DSub* newPro, BaseGDL** self); 
+  EnvUDT( ProgNodeP callindNode_, DSubUD* newPro, DObjGDL** self); 
 
   DLong GetOnError() const { return onError;}
 
+  ProgNodeP GetCatchNode() const { return catchNode;} 
+  BaseGDL** GetCatchVar() const { return catchVar;} 
+
   SizeT NJump() const { return nJump;}
   int   LastJump() const { return lastJump;}
   ProgNodeP GotoTarget( int ix)
@@ -431,7 +499,7 @@ public:
     ++nJump;
     return static_cast<DSubUD*>( pro)->GotoTarget( ix);
   }
-  bool LFun() const { return lFun;} // left-function
+  CallContext GetCallContext() const { return callContext;} // left-function
 
   void SetIOError( int targetIx) 
   { // this isn't a jump
@@ -461,15 +529,14 @@ public:
 // this contains the library function API ***********************
 class EnvT: public EnvBaseT
 {
-static std::deque< void*> freeList;
-
+static std::vector< void*> freeList;
+  
 public:
 static 	void* operator new( size_t bytes);
 static	void operator delete( void *ptr);
 
   // Please use non library API (see below) function with caution
   // (most of them can be ignored by library function authors)
-
 public:
   ~EnvT()
   {
@@ -478,14 +545,14 @@ public:
   EnvT( ProgNodeP cN, DSub* pro_);
 
   // for obj_new and obj_destroy
-  EnvT( EnvT* pEnv, DSub* newPro, BaseGDL** self); 
+  EnvT( EnvT* pEnv, DSub* newPro, DObjGDL** self); 
 
   void HeapGC( bool doPtr, bool doObj, bool verbose);
   void ObjCleanup( DObj actID);
 
   // used by obj_new (basic_fun.cpp)
-  EnvT* NewEnv(  DSub* newPro, SizeT skipP, BaseGDL** newObj=NULL);
-  void PushNewEnvUD(  DSub* newPro, SizeT skipP, BaseGDL** newObj=NULL);
+  EnvT* NewEnv(  DSub* newPro, SizeT skipP, DObjGDL** newObj=NULL);
+  void PushNewEnvUD(  DSubUD* newPro, SizeT skipP, DObjGDL** newObj=NULL);
   // for exclusive use by lib::on_error
   void OnError();
   // for exclusive use by lib::catch_pro
@@ -493,7 +560,7 @@ public:
 
   const std::string GetFilename() const
   {
-    static const std::string internal("<INTERNAL_LIBRARY>");
+    static const std::string internal(INTERNAL_LIBRARY_STR);
     return internal;
   }
 
@@ -508,29 +575,21 @@ public:
 //   void Throw( const std::string& s)
 //   { throw GDLException( CallingNode(), pro->ObjectName()+": "+s);}
 
+  // From now on all library functions which return a l-value must
+  // call SetPtrToReturnValue with the ptr to the returned value
+  void SetPtrToReturnValue(BaseGDL** p) { EnvBaseT::SetPtrToReturnValue(p);}
+
   // will print the message (can be multiline) and exit
   // first usage in "math_fun_ac.cpp"
-  void Help(const std::string s_help[], int size_of_s)
-  { 
-    if (size_of_s == 0) 
-      throw GDLException( CallingNode(), pro->ObjectName()+	\
-			  ": no inline doc ready");
-    else {
-      int i;
-      for (i = 0; i < size_of_s-1; i++)
-	Message(pro->ObjectName()+": "+s_help[i]);
-      throw GDLException( CallingNode(), pro->ObjectName()+": "+s_help[i]);
-    }
-  }
-
-
+  void Help(const std::string s_help[], int size_of_s);
+ 
   // returns environment data, by value (but that by C++ reference)
   // in EnvBaseT
-//   BaseGDL*& GetKW(SizeT ix) { return env[ix];}
+  //   BaseGDL*& GetKW(SizeT ix) { return env[ix];}
 
   // it is now possible to define a niminum number of parameters for library subroutines
   // if this is done the next function can be used
-//   BaseGDL*& GetParUnchecked(SizeT i);
+  //   BaseGDL*& GetParUnchecked(SizeT i);
 
   // returns the ix'th parameter (NULL if not defined)
   BaseGDL*& GetPar( SizeT i);  
@@ -588,7 +647,7 @@ public:
 //     T* res = dynamic_cast<T*>( p);
 //     if( res != NULL) return res;
     T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
-    Guard( res);
+    this->DeleteAtExit( res);
     return res;
   }
   // same as before for keywords
@@ -603,7 +662,7 @@ public:
 //     T* res = dynamic_cast<T*>( p);
 //     if( res != NULL) return res;
     T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
-    Guard( res);
+    this->DeleteAtExit( res);
     return res;
   }
 
@@ -618,7 +677,7 @@ public:
 //     T* res = dynamic_cast<T*>( p);
 //     if( res != NULL) return res;
     T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
-    Guard( res);
+    this->DeleteAtExit( res);
     return res;
   }
   // same as before for keywords
@@ -632,7 +691,7 @@ public:
 //     T* res = dynamic_cast<T*>( p);
 //     if( res != NULL) return res;
     T* res = static_cast<T*>( p->Convert2( T::t, BaseGDL::COPY));
-    Guard( res);
+    this->DeleteAtExit( res);
     return res;
   }
 
@@ -661,17 +720,19 @@ public:
   // this one together with a static int holding the index is faster
   // (after the first call)
   bool KeywordSet( SizeT ix);
-
+  // GD added -- possibly very wrong?
+  bool KeywordPresent( const std::string& kw);
   bool KeywordPresent( SizeT ix)
   { return EnvBaseT::KeywordPresent( ix);}
 
   // local/global keyword/paramter
-  bool LocalKW( SizeT ix) 
+  bool LocalKW( SizeT ix) const
   {
-    if( ix >= env.size()) return false;
-    return ( env.Loc( ix) != NULL);
+    return EnvBaseT::LocalKW( ix);
+//     if( ix >= env.size()) return false;
+//     return ( env.Loc( ix) != NULL);
   }
-  bool GlobalKW( SizeT ix) 
+  bool GlobalKW( SizeT ix) const
   {
     return EnvBaseT::GlobalKW( ix);
   }
@@ -724,10 +785,10 @@ public:
   void AssureScalarPar( SizeT pIx, typename T::Ty& scalar)
   {
     BaseGDL* p = GetParDefined( pIx);
-    T* tp= dynamic_cast<T*>(p);
-    if( tp == NULL)
+    if( p->Type() != T::t)
       Throw( "Variable must be a "+T::str+" in this context: "+
 	     GetParString(pIx));
+    T* tp= static_cast<T*>(p);
     if( !tp->Scalar( scalar))
       Throw("Variable must be a scalar in this context: "+
 	    GetParString(pIx));
@@ -739,10 +800,10 @@ public:
     BaseGDL* p = GetKW( ix);
     if( p == NULL)
       Throw("Keyword undefined: "+GetString(ix));
-    T* tp= dynamic_cast<T*>(p);
-    if( tp == NULL)
+    if( p->Type() != T::t)
       Throw("Keyword must be a "+T::str+" in this context: "+
 	    GetString(ix));
+    T* tp= static_cast<T*>(p);
     if( !tp->Scalar( scalar))
       Throw("Keyword must be a scalar in this context: "+
 	    GetString(ix));
@@ -789,9 +850,59 @@ public:
   void ShiftParNumbering(int n);
 };
 
+const int defaultStackDepth = 64;
+class EnvStackT
+{
+  EnvUDT** envStackFrame;
+  EnvUDT** envStack;
+
+  SizeT top;
+  SizeT sz;
+
+public:
+  typedef SizeT size_type;
+  typedef EnvUDT* pointer_type;
+  
+  EnvStackT(): sz(defaultStackDepth), top(0) 
+  {
+    envStackFrame = new EnvUDT* [ sz+1];
+    envStack = envStackFrame + 1;
+  }
+  ~EnvStackT() { delete[] envStackFrame;}
+  
+  bool empty() const { return top == 0;}
+  
+  void push_back( EnvUDT* b) 
+  {
+    if( top >= sz)
+    {
+      if( sz >= 32768)
+	    throw GDLException("Recursion limit reached ("+i2s(sz)+").");
+
+      EnvUDT** newEnvStackFrame = new EnvUDT* [ sz + sz + 1];
+      EnvUDT** newEnvStack = newEnvStackFrame + 1;
+
+      for( SizeT i=0; i<sz; ++i)
+	newEnvStack[ i] = envStack[ i];
+
+      delete[] envStackFrame;
+      envStackFrame = newEnvStackFrame;
+      envStack = newEnvStack;
+      sz += sz;
+    }
+    envStackFrame[ ++top] = b; 
+  }
+  void pop_back() { assert(top>0); --top;}
+  EnvUDT* back() const { assert(top>0); return envStackFrame[ top];}
+  SizeT size() const { return top;}
+  EnvUDT* operator[]( SizeT ix) const { return envStack[ ix];}
+  EnvUDT*& operator[]( SizeT ix) { return envStack[ ix];}
+//   EnvUDT** begin() const { return &envStack[0];}
+//   EnvUDT** end() const { return &envStack[sz];}
+};
 
 // typedef std::deque<EnvBaseT*> EnvStackT;
-typedef std::deque<EnvUDT*> EnvStackT;
+// typedef std::deque<EnvUDT*> EnvStackT;
 
 #endif
 
