@@ -37,12 +37,41 @@ if(interpreter!=NULL && interpreter->CallStack().size()>0)
 return "";
 }
 
+GDLException::GDLException(DLong eC, const string& s, bool pre, bool decorate): 
+  ANTLRException(s),
+  errorNode(static_cast<RefDNode>(antlr::nullAST)),
+  errorNodeP( NULL),
+  errorCode(eC),
+  line( 0), col( 0), prefix( pre),
+  ioException( false),
+  targetEnv( NULL),
+  arrayexprIndexeeFailed(false)
+{
+if(decorate && interpreter!=NULL && interpreter->CallStack().size()>0) 
+{
+  EnvBaseT* e = interpreter->CallStack().back();
+  errorNodeP = e->CallingNode();
+  msg = e->GetProName();
+  if( msg != "$MAIN$") msg +=  ": "+ s; else msg = s;
+}
+else
+{
+  msg = s;
+}
+  // note: This is for cases, when form a destructor is thrown
+  // in these cases, program aborts
+#ifdef GDL_DEBUG
+   cerr << s << endl;
+#endif
+}
 GDLException::GDLException(const string& s, bool pre, bool decorate): 
   ANTLRException(s),
   errorNode(static_cast<RefDNode>(antlr::nullAST)),
   errorNodeP( NULL),
+  errorCode(-1),
   line( 0), col( 0), prefix( pre),
-		  targetEnv( NULL)
+  ioException( false),
+  targetEnv( NULL)
 {
 if(decorate && interpreter!=NULL && interpreter->CallStack().size()>0) 
 {
@@ -66,8 +95,36 @@ GDLException::GDLException(const RefDNode eN, const string& s):
   ANTLRException(s), 
   errorNode(eN),
   errorNodeP( NULL),
+  errorCode(-1),
   line( 0), col( 0), prefix( true),
-		  targetEnv( NULL)
+  ioException( false),
+  targetEnv( NULL),
+  arrayexprIndexeeFailed(false)
+{
+if(interpreter!=NULL && interpreter->CallStack().size()>0) 
+{
+  EnvBaseT* e = interpreter->CallStack().back();
+  errorNodeP = e->CallingNode();
+  msg = e->GetProName();
+  if( msg != "$MAIN$") msg +=  ": "+ s; else msg = s;
+}
+else
+{
+  msg = s;
+}
+#ifdef GDL_DEBUG
+   cerr << s << endl;
+#endif
+}
+GDLException::GDLException(DLong eC, const RefDNode eN, const string& s): 
+  ANTLRException(s), 
+  errorNode(eN),
+  errorNodeP( NULL),
+  errorCode(eC),
+  line( 0), col( 0), prefix( true),
+  ioException( false),
+  targetEnv( NULL),
+  arrayexprIndexeeFailed(false)
 {
 if(interpreter!=NULL && interpreter->CallStack().size()>0) 
 {
@@ -89,8 +146,11 @@ GDLException::GDLException(const ProgNodeP eN, const string& s, bool decorate, b
   ANTLRException(s), 
   errorNode(static_cast<RefDNode>(antlr::nullAST)),
   errorNodeP( eN),
+  errorCode(-1),
   line( 0), col( 0), prefix( true),
-		  targetEnv( NULL)
+  ioException( false),
+  targetEnv( NULL),
+  arrayexprIndexeeFailed(false)
 {
 if( overWriteNode && interpreter!=NULL && interpreter->CallStack().size()>0) 
 {
@@ -111,25 +171,81 @@ else
    cerr << s << endl;
 #endif
 }
+GDLException::GDLException(DLong eC, const ProgNodeP eN, const string& s, bool decorate, bool overWriteNode): 
+  ANTLRException(s), 
+  errorNode(static_cast<RefDNode>(antlr::nullAST)),
+  errorNodeP( eN),
+  errorCode(eC),
+  line( 0), col( 0), prefix( true),
+  ioException( false),
+  targetEnv( NULL),
+  arrayexprIndexeeFailed(false)
+{
+  if( overWriteNode && interpreter!=NULL && interpreter->CallStack().size()>0) 
+  {
+    EnvBaseT* e = interpreter->CallStack().back();
+    errorNodeP = e->CallingNode();
+  }
+  if( decorate && interpreter!=NULL && interpreter->CallStack().size()>0)
+  {
+    EnvBaseT* e = interpreter->CallStack().back();
+    msg = e->GetProName();
+    if( msg != "$MAIN$") msg +=  ": "+ s; else msg = s;
+  }
+  else
+  {
+    msg = s;
+  }
+#ifdef GDL_DEBUG
+   cerr << s << endl;
+#endif
+}
 
 GDLException::GDLException(SizeT l, SizeT c, const string& s): 
   ANTLRException(s),
   errorNode(static_cast<RefDNode>(antlr::nullAST)),
   errorNodeP( NULL),
+  errorCode(-1),
   line( l), col( c), prefix( true),
-		  targetEnv( NULL)
+  ioException( false),
+  targetEnv( NULL),
+  arrayexprIndexeeFailed(false)
 {
-if(interpreter!=NULL && interpreter->CallStack().size()>0) 
-{
-  EnvBaseT* e = interpreter->CallStack().back();
-  errorNodeP = e->CallingNode();
-  msg = e->GetProName();
-  if( msg != "$MAIN$") msg +=  ": "+ s; else msg = s;
+  if(interpreter!=NULL && interpreter->CallStack().size()>0) 
+  {
+    EnvBaseT* e = interpreter->CallStack().back();
+    errorNodeP = e->CallingNode();
+    msg = e->GetProName();
+    if( msg != "$MAIN$") msg +=  ": "+ s; else msg = s;
+  }
+  else
+  {
+    msg = s;
+  }
+#ifdef GDL_DEBUG
+   cerr << s << endl;
+#endif
 }
-else
+GDLException::GDLException(DLong eC, SizeT l, SizeT c, const string& s): 
+  ANTLRException(s),
+  errorNode(static_cast<RefDNode>(antlr::nullAST)),
+  errorNodeP( NULL),
+  errorCode(eC),
+  line( l), col( c), prefix( true),
+  targetEnv( NULL),
+  arrayexprIndexeeFailed(false)
 {
-  msg = s;
-}
+  if(interpreter!=NULL && interpreter->CallStack().size()>0) 
+  {
+    EnvBaseT* e = interpreter->CallStack().back();
+    errorNodeP = e->CallingNode();
+    msg = e->GetProName();
+    if( msg != "$MAIN$") msg +=  ": "+ s; else msg = s;
+  }
+  else
+  {
+    msg = s;
+  }
 #ifdef GDL_DEBUG
    cerr << s << endl;
 #endif
@@ -157,7 +273,9 @@ throw GDLException( str);
 
 void WarnAboutObsoleteRoutine(const string& name)
 {
-  static DStructGDL* warnStruct = SysVar::Warn();
+  // no static here due to .RESET_SESSION
+  DStructGDL* warnStruct = SysVar::Warn();
+  // this static is ok as it will evaluate always to the same value
   static unsigned obs_routinesTag = warnStruct->Desc()->TagIndex( "OBS_ROUTINES");
   if (((static_cast<DByteGDL*>( warnStruct->GetTag(obs_routinesTag, 0)))[0]).LogTrue())
     Message("Routine compiled from an obsolete library: " + name);
@@ -167,15 +285,17 @@ void WarnAboutObsoleteRoutine(const string& name)
 void WarnAboutObsoleteRoutine(const RefDNode eN, const string& name)
 {
 // TODO: journal?
-  static DStructGDL* warnStruct = SysVar::Warn();
+  // no static here due to .RESET_SESSION
+  DStructGDL* warnStruct = SysVar::Warn();
+  // this static is ok as it will evaluate always to the same value
   static unsigned obs_routinesTag = warnStruct->Desc()->TagIndex( "OBS_ROUTINES");
   if (((static_cast<DByteGDL*>( warnStruct->GetTag(obs_routinesTag, 0)))[0]).LogTrue())
   {
     GDLException* e = new GDLException(eN, 
       "Routine compiled from an obsolete library: " + name
-    );  
+    );
+    Guard<GDLException> eGuard(e);
     GDLInterpreter::ReportCompileError(*e, "");
-//                              TODO: file /\
-    delete e;
+// TODO: file 
   }
 }
