@@ -38,9 +38,9 @@ namespace lib {
 
   using namespace std;
 
-  const int szdbl=sizeof(double);
-  const int szflt=sizeof(float);
-  const int szlng=sizeof(long);
+  const int szdbl=sizeof(DDouble);
+  const int szflt=sizeof(DFloat);
+  const int szlng=sizeof(DLong);
 
   void ludc_pro( EnvT* e)
   {
@@ -97,7 +97,7 @@ namespace lib {
     }
     
     // copying over p0 the updated matrix	
-    DLong dims[2] = {p0->Dim(0), p0->Dim(0)};
+    SizeT dims[2] = {p0->Dim(0), p0->Dim(0)};
     dimension dim0(dims, (SizeT) 2);
 
     BaseGDL** p0Do = &e->GetPar( 0);
@@ -116,7 +116,7 @@ namespace lib {
     // { p0->Convert2(GDL_FLOAT, BaseGDL::CONVERT); }
 
     // copying over p1 the permutation vector
-    DLong n = p0->Dim(0);
+    SizeT n = p0->Dim(0);
     dimension dim1(&n, (SizeT) 1);
     BaseGDL** p1D = &e->GetPar( 1);
     GDLDelete((*p1D));
@@ -132,7 +132,7 @@ namespace lib {
   BaseGDL* lusol_fun( EnvT* e)
   {
     SizeT nParam=e->NParam(1);
-    int s;
+//    int s;
     
     //     if( nParam == 0)
     //       e->Throw( "Incorrect number of arguments.");
@@ -207,7 +207,7 @@ namespace lib {
     
     int debug=0;
     if (debug) {
-      cout << "permutation order: " << s << endl;
+//      cout << "permutation order: " << s << endl;
       cout << "permutation vector:";
       gsl_permutation_fprintf (stdout, p, " %u");
       cout << endl;
