@@ -26,11 +26,15 @@
 #include <cassert>
 
 #include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <netdb.h>
-#include <unistd.h>
-#include <arpa/inet.h>
+#ifdef _MSC_VER
+	#include <winsock2.h>
+#else
+	#include <sys/socket.h>
+	#include <netinet/in.h>
+	#include <netdb.h>
+	#include <arpa/inet.h>
+	#include <unistd.h>
+#endif
 
 #include <string.h>  // for memcpy
 #include "gzstream.hpp"
@@ -60,6 +64,7 @@ public:
   std::fstream* fStream;
   igzstream* igzStream; // for gzip compressed input
   ogzstream* ogzStream; // for gzip compressed output
+
 //public:
   AnyStream()
     : fStream(NULL) 
@@ -281,6 +286,8 @@ public:
 
 class GDLStream
 {
+  bool getLunLock;
+   
   std::string name;
   std::ios_base::openmode mode;
 
@@ -324,6 +331,7 @@ class GDLStream
 
 public:
   GDLStream(): 
+    getLunLock(false),
     name(), 
     mode(), 
     anyStream(NULL), 
@@ -430,6 +438,9 @@ public:
   
   void Free(); 
 
+  void SetGetLunLock( bool b) { getLunLock = b;}
+  bool GetGetLunLock() { return getLunLock;}
+
   const std::string& Name() { return name;}
   
   bool SwapEndian() { return swapEndian;}
