@@ -27,6 +27,9 @@
 #include "basic_fun.hpp"
 #include "basic_pro.hpp"
 
+#include "list.hpp"
+#include "hash.hpp"
+
 #include "math_fun.hpp"
 #include "math_fun_ac.hpp"
 #include "math_fun_gm.hpp"
@@ -37,6 +40,8 @@
 
 #include "gsl_fun.hpp"
 
+#include "convol.hpp"
+
 #ifdef USE_PYTHON
 #  include "gdlpython.hpp"
 #endif
@@ -81,6 +86,12 @@ void LibInit()
   LibInit_ng(); 
   const char KLISTEND[] = "";
 
+  const string listKey[]={"EXTRACT", "LENGTH", "NO_COPY", KLISTEND};
+  new DLibFunRetNew(lib::list_fun,string("LIST"),-1,listKey);
+
+  const string hashKey[]={"NO_COPY", KLISTEND};
+  new DLibFunRetNew(lib::hash_fun,string("HASH"),-1,hashKey);
+
   const string scope_varfetchKey[]={"LEVEL", KLISTEND};
   new DLibFun(lib::scope_varfetch_value,string("SCOPE_VARFETCH"),-1,scope_varfetchKey);
 
@@ -97,8 +108,8 @@ void LibInit()
   new DLibFunRetNew(lib::temporary,string("TEMPORARY"),1);
 
   const string routine_infoKey[]={"FUNCTIONS","SYSTEM","DISABLED","ENABLED",
-				  "PARAMETERS", KLISTEND};
-  new DLibFun(lib::routine_info,string("ROUTINE_INFO"),1,routine_infoKey);
+				  "PARAMETERS","SOURCE", KLISTEND};
+  new DLibFunRetNew(lib::routine_info,string("ROUTINE_INFO"),1,routine_infoKey);
 
 #ifndef _MSC_VER
   const string spawnKey[]={ "COUNT","EXIT_STATUS","PID",
@@ -107,7 +118,7 @@ void LibInit()
 #endif
 
   const string bytsclKey[]={"MIN","MAX","TOP","NAN",KLISTEND};
-  new DLibFun(lib::bytscl,string("BYTSCL"),3,bytsclKey);
+  new DLibFunRetNew(lib::bytscl,string("BYTSCL"),3,bytsclKey);
 
   const string n_tagsKey[]={"DATA_LENGTH","LENGTH",KLISTEND};
   new DLibFunRetNew(lib::n_tags,string("N_TAGS"),1,n_tagsKey);
@@ -128,8 +139,9 @@ void LibInit()
   const string rebinKey[]={"SAMPLE",KLISTEND};
   new DLibFunRetNew(lib::rebin_fun,string("REBIN"),9,rebinKey);
 
-  const string convolKey[]={"CENTER","EDGE_TRUNCATE","EDGE_WRAP",KLISTEND};
-  new DLibFunRetNew(lib::convol,string("CONVOL"),3,convolKey);
+  const string convolKey[]={"CENTER","EDGE_TRUNCATE","EDGE_WRAP",
+			    "BIAS","NORMALIZE",KLISTEND};
+  new DLibFunRetNew(lib::convol_fun,string("CONVOL"),3,convolKey);
 
 #ifndef _MSC_VER
   const string file_searchKey[]={"COUNT","EXPAND_ENVIRONMENT","EXPAND_TILDE",
@@ -155,8 +167,10 @@ void LibInit()
 
   const string messageKey[]={"CONTINUE","INFORMATIONAL","IOERROR",
 			     "NONAME","NOPREFIX","NOPRINT",
-			     "RESET","TRACEBACK",KLISTEND};
-  new DLibPro(lib::message,string("MESSAGE"),1,messageKey);
+			     "RESET","REISSUE_LAST","TRACEBACK", KLISTEND}; 
+  //TRACEBACK is in MESSAGE but obsolete since 5.0. it is used widely in CMSVlib !
+  const string messageWarnKey[]={"LEVEL", "NAME", "BLOCK",KLISTEND};
+  new DLibPro(lib::message,string("MESSAGE"),1,messageKey,messageWarnKey);
   
   const string cdKey[]={"CURRENT",KLISTEND};
   new DLibPro(lib::cd_pro,string("CD"),1,cdKey);
@@ -168,6 +182,13 @@ void LibInit()
 			       "NAMED_PIPE","SOCKET","SYMLINK","NOEXPAND_PATH",KLISTEND};
   new DLibFunRetNew(lib::file_test,string("FILE_TEST"),1,file_testKey);
 
+  const string file_basenameKey[]={"FOLD_CASE",KLISTEND};
+  new DLibFunRetNew(lib::file_basename,string("FILE_BASENAME"),2,file_basenameKey);
+
+  const string file_dirnameKey[]={"MARK_DIRECTORY",KLISTEND};
+  new DLibFunRetNew(lib::file_dirname,string("FILE_DIRNAME"),1,file_dirnameKey);
+
+
   const string file_sameKey[]={"NOEXPAND_PATH",KLISTEND};
   new DLibFunRetNew(lib::file_same,string("FILE_SAME"),2,file_sameKey);
 
@@ -198,11 +219,12 @@ void LibInit()
   const string exitKey[]={"NO_CONFIRM","STATUS",KLISTEND};
   new DLibPro(lib::exitgdl,string("EXIT"),0,exitKey);
   
-  const string helpKey[]={"STRUCTURES","ROUTINES","BRIEF",
-			  "OUTPUT","PROCEDURES","FUNCTIONS",
-			  "INFO","LIB","LIB_GDL_INTERNAL","CALLS","RECALL_COMMANDS",
-                          "MEMORY","PREFERENCES", KLISTEND};
-  new DLibPro(lib::help,string("HELP"),-1,helpKey);
+  const string helpKey[]={"ALL_KEYS","BRIEF","CALLS","FUNCTIONS","HELP","INFO",
+			  "INTERNAL_LIB_GDL","LAST_MESSAGE","LIB","MEMORY",
+			  "OUTPUT","PATH_CACHE","PREFERENCES","PROCEDURES",
+			  "RECALL_COMMANDS","ROUTINES","SOURCE_FILES","STRUCTURES", KLISTEND};
+  const string helpWarnKey[]={"FULL","TRACEBACK", KLISTEND};
+  new DLibPro(lib::help,string("HELP"),-1,helpKey,helpWarnKey);
 
   const string memoryKey[]={"CURRENT","HIGHWATER","NUM_ALLOC",
     "NUM_FREE","STRUCTURE","L64",KLISTEND};
@@ -234,7 +256,7 @@ void LibInit()
 
   const string heap_gcKey[]={"PTR","OBJ","VERBOSE",KLISTEND};
   new DLibPro(lib::heap_gc,string("HEAP_GC"),0,heap_gcKey); 
-  new DLibPro(lib::heap_free,string("HEAP_FREE"),1,heap_gcKey); 
+  new DLibPro(lib::heap_free,string("HEAP_FREE"),1,heap_gcKey);
 
 
   new DLibPro(lib::ptr_free,string("PTR_FREE"),-1);
@@ -457,7 +479,7 @@ void LibInit()
   new DLibFunRetNew(lib::rotate,string("ROTATE"),2,NULL,NULL,true);
 
   const string reverseKey[] = {"OVERWRITE", KLISTEND};
-  new DLibFunRetNew(lib::reverse, string("REVERSE"), 2, reverseKey, NULL, true);
+  new DLibFun(lib::reverse, string("REVERSE"), 2, reverseKey, NULL, true);
 
 //   const string minKey[]={"MAX",KLISTEND};
 //   new DLibFun(lib::min_fun,string("MIN"),2,minKey);
@@ -490,23 +512,31 @@ void LibInit()
   new DLibPro(lib::set_plot,string("SET_PLOT"),1,set_plotKey);
   
   const string get_screen_sizeKey[]={"RESOLUTION",KLISTEND};
-  new DLibFun(lib::get_screen_size,string("GET_SCREEN_SIZE"),1,get_screen_sizeKey);
+  new DLibFunRetNew(lib::get_screen_size,string("GET_SCREEN_SIZE"),1,get_screen_sizeKey);
 
   const string tvlctKey[]={"GET","HLS","HSV",KLISTEND};
   new DLibPro(lib::tvlct,string("TVLCT"),4,tvlctKey);
 
+  const string tvcrsKey[]={"DATA","DEVICE","NORMAL",KLISTEND};
+  const string tvcrsWarnKey[]={"CENTIMETERS","INCHES","HIDE_CURSOR","T3D","Z",KLISTEND};
+  new DLibPro(lib::tvcrs,string("TVCRS"),2,tvcrsKey,tvcrsWarnKey);
+
+  new DLibPro(lib::empty,string("EMPTY"),-1);
+  
   const string deviceKey[]=
     {
       "CLOSE_FILE","FILENAME","LANDSCAPE","PORTRAIT",
       "DECOMPOSED","GET_DECOMPOSED","Z_BUFFERING","SET_RESOLUTION",
       "SET_CHARACTER_SIZE","GET_VISUAL_DEPTH","XSIZE","YSIZE",
       "COLOR","GET_SCREEN_SIZE","INCHES","WINDOW_STATE","SCALE_FACTOR", 
-      "XOFFSET", "YOFFSET", "ENCAPSULATED", KLISTEND
+      "XOFFSET", "YOFFSET", "ENCAPSULATED", "GET_GRAPHICS_FUNCTION", 
+      "SET_GRAPHICS_FUNCTION", "CURSOR_STANDARD", "CURSOR_ORIGINAL",
+      "CURSOR_CROSSHAIR","RETAIN", KLISTEND
     };
-  const string deviceWarnKey[] = {"RETAIN", "SET_FONT", "HELVETICA", 
+  const string deviceWarnKey[] = {"SET_FONT", "HELVETICA", 
     "AVANTGARDE", "BKMAN", "COURIER", "PALATINO", 
     "SCHOOLBOOK", "TIMES", "ZAPFCHANCERY", "ZAPFDINGBATS", "BITS_PER_PIXEL", 
-    "ITALIC", "BOLD", "TRUE_COLOR", KLISTEND};
+    "ITALIC", "BOLD", "TRUE_COLOR", "CURSOR_IMAGE","CURSOR_MASK","CURSOR_XY", KLISTEND};
   new DLibPro(lib::device,string("DEVICE"),0, deviceKey, deviceWarnKey);
 
   const string usersymKey[]= 
@@ -523,64 +553,67 @@ void LibInit()
      "LINESTYLE", "NOCLIP",  "NODATA",   "NOERASE", 
      "NORMAL",    "POSITION","PSYM",     "SUBTITLE",
      "SYMSIZE",   "THICK",    "TICKLEN", "TITLE",
-     "XCHARSIZE", "XMARGIN", "XMINOR",
-     "XRANGE",    "XSTYLE",     "XTHICK",  "XTICKFORMAT",
-     "XTICKLEN",  "XTICKS",     "XTITLE",
-     "YCHARSIZE", "YMARGIN", "YMINOR",
-     "YRANGE",    "YSTYLE",     "YTHICK",  "YTICKFORMAT", "YTICKLEN",
-     "YTICKS",    "YTITLE", "ZCHARSIZE",    "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
-     "ZRANGE",    "ZSTYLE",     "ZTHICK",  "ZTICKFORMAT",
-     "ZTICKLEN",  "ZTICKS",    "ZTITLE", "ZVALUE", "MAX_VALUE", "MIN_VALUE",
-     "XLOG",      "YLOG",     "YNOZERO", "XTYPE", "YTYPE", "POLAR", "NSUM",
-     KLISTEND
+     "MAX_VALUE", "MIN_VALUE",
+     "XLOG",      "YLOG",
+     "YNOZERO",   "XTYPE",    "YTYPE",   "POLAR", "NSUM",
+      "XCHARSIZE", "YCHARSIZE",
+      "XGRIDSTYLE", "YGRIDSTYLE",
+      "XMARGIN", "YMARGIN",
+      "XMINOR", "YMINOR",
+      "XRANGE", "YRANGE",
+      "XSTYLE", "YSTYLE",
+      "XTHICK", "YTHICK",
+      "XTICK_GET", "YTICK_GET",
+      "XTICKFORMAT", "YTICKFORMAT",
+      "XTICKINTERVAL", "YTICKINTERVAL",
+      "XTICKLAYOUT", "YTICKLAYOUT",
+      "XTICKLEN", "YTICKLEN",
+      "XTICKNAME", "YTICKNAME",
+      "XTICKS", "YTICKS",
+      "XTICKUNITS", "YTICKUNITS",
+      "XTICKV", "YTICKV",
+      "XTITLE", "YTITLE",
+      "ZCHARSIZE", "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
+      "ZRANGE", "ZSTYLE", "ZTHICK", "ZTICK_GET", "ZTICKFORMAT", "ZTICKINTERVAL",
+      "ZTICKLAYOUT", "ZTICKLEN", "ZTICKNAME", "ZTICKS", "ZTICKUNITS", "ZTICKV",
+      "ZTITLE", "T3D", "ZVALUE", KLISTEND
     };
   //
-  const string plotWarnKey[]= {
-    "FONT", "T3D", "YTICKUNITS", "XTICKLAYOUT", "YTICKLAYOUT", 
-    "ZTICKLAYOUT", "XGRIDSTYLE", "YGRIDSTYLE", "XTICKUNITS", "XTICKV", "XTICK_GET", "YTICKV",
-    "YTICK_GET", "XTICKNAME", "ZTICKNAME", "ZTICKUNITS", "ZTICKV",  "ZTICK_GET",
-    "XTICKINTERVAL", "YTICKINTERVAL", "YTICKNAME", "ZTICKINTERVAL", 
-    KLISTEND
-  };
+  const string plotWarnKey[]= { "FONT","CHANNEL", KLISTEND };
   new DLibPro(lib::plot,string("PLOT"),2,plotKey,plotWarnKey);
 
   const string axisKey[]=
-    {
-     "CHARSIZE","CHARTHICK",
-     "COLOR",     "DATA",    "DEVICE",   "FONT",
-     "NODATA",    "NOERASE", 
-     "NORMAL",    "SUBTITLE",
-     "T3D",       "TICKLEN", 
-     "XCHARSIZE", "XGRIDSTYLE", "XMARGIN", "XMINOR",
-     "XRANGE",    "XSTYLE",     "XTHICK",  "XTICKFORMAT",
-     "XTICKLEN",  "XTICKNAME",
-     "XTICKS",    
-     "XTITLE",
-     "YCHARSIZE",  "YGRIDSTYLE", "YMARGIN", "YMINOR",
-     "YRANGE",     "YSTYLE",     "YTHICK",  "YTICKFORMAT",
-     "YTICKLEN",   "YTICKNAME",
-     "YTICKS",
-     "YTITLE",
-     "ZCHARSIZE",  "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
-     "ZRANGE",     "ZSTYLE",     "ZTHICK",  "ZTICKFORMAT",
-     "ZTICKLEN",   "ZTICKNAME",
-     "ZTICKS",
-     "ZTITLE",
-     "ZVALUE", 
-
-     "SAVE", "XAXIS", "YAXIS", "XLOG", "YLOG", "XTYPE", "YTYPE", "YNOZERO", "THICK",
-      KLISTEND
-     };
-   const string axisWarnKey[]=
-   {
-   "ZAXIS","ZLOG", 
-   "XTICKINTERVAL", "YTICKINTERVAL", "ZTICKINTERVAL",
-   "XTICKLAYOUT", "YTICKLAYOUT", "ZTICKLAYOUT",
-   "XTICKUNITS", "YTICKUNITS", "ZTICKUNITS",
-   "XTICKV", "YTICKV", "ZTICKV",
-   "XTICK_GET", "YTICK_GET", "ZTICK_GET",
-   KLISTEND
+  {
+    "CHARSIZE",  "CHARTHICK",
+    "COLOR",     "DATA",    "DEVICE",   "FONT",
+    "NODATA",    "NOERASE", 
+    "NORMAL",    "SUBTITLE",
+    "T3D",       "TICKLEN", 
+    "SAVE", "XAXIS", "YAXIS", "XLOG", "YLOG", "XTYPE", "YTYPE", "YNOZERO", "THICK",
+    "XCHARSIZE", "YCHARSIZE",
+    "XGRIDSTYLE", "YGRIDSTYLE",
+    "XMARGIN", "YMARGIN",
+    "XMINOR", "YMINOR",
+    "XRANGE", "YRANGE",
+    "XSTYLE", "YSTYLE",
+    "XTHICK", "YTHICK",
+    "XTICK_GET", "YTICK_GET",
+    "XTICKFORMAT", "YTICKFORMAT",
+    "XTICKINTERVAL", "YTICKINTERVAL",
+    "XTICKLAYOUT", "YTICKLAYOUT",
+    "XTICKLEN", "YTICKLEN",
+    "XTICKNAME", "YTICKNAME",
+    "XTICKS", "YTICKS",
+    "XTICKUNITS", "YTICKUNITS",
+    "XTICKV", "YTICKV",
+    "XTITLE", "YTITLE",
+    "ZAXIS", "ZLOG", "ZVALUE", "ZCHARSIZE", "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
+    "ZRANGE", "ZSTYLE", "ZTHICK", "ZTICK_GET", "ZTICKFORMAT", "ZTICKINTERVAL",
+    "ZTICKLAYOUT", "ZTICKLEN", "ZTICKNAME", "ZTICKS", "ZTICKUNITS", "ZTICKV",
+    "ZTITLE",
+    KLISTEND
   };
+  const string axisWarnKey[]={"CHANNEL",KLISTEND};
   new DLibPro(lib::axis,string("AXIS"),3,axisKey,axisWarnKey);
 
   const string oplotKey[]=
@@ -589,25 +622,68 @@ void LibInit()
       // 0
       "CLIP","COLOR", "LINESTYLE", "NOCLIP",
       // 4
-      "PSYM", "SYMSIZE",  "T3D",  "THICK",
+      "PSYM", "SYMSIZE",  "T3D",  "ZVALUE", "THICK",
       // 8
       "MAX_VALUE", "MIN_VALUE", "NSUM", "POLAR",
       KLISTEND
     };
-  new DLibPro(lib::oplot, string("OPLOT"),2,oplotKey);
+  const string oplotWarnKey[]={"CHANNEL",KLISTEND};
+  new DLibPro(lib::oplot, string("OPLOT"),2,oplotKey,oplotWarnKey);
 
   const string plotsKey[]=
     {
       "CLIP","COLOR", "LINESTYLE", "NOCLIP",
-      "PSYM", "SYMSIZE",  "T3D",  "THICK",
+      "PSYM", "SYMSIZE",  "T3D",  "Z", "THICK",
       "DATA","DEVICE","NORMAL","CONTINUE",
       KLISTEND
     };
-  const string plotsWarnKey[] = 
+
+  new DLibPro(lib::plots, string("PLOTS"), 3, plotsKey);
+
+  const string set_shadingKey[]=
   {
-    "Z", KLISTEND
+    "LIGHT", KLISTEND
   };
-  new DLibPro(lib::plots, string("PLOTS"), 3, plotsKey, plotsWarnKey);
+  
+  const string set_shadingWarnKey[]=
+  {
+     "GOURAUD","REJECT","VALUES",KLISTEND
+  };
+  new DLibPro(lib::set_shading, string("SET_SHADING"), 0, set_shadingKey, set_shadingWarnKey);
+
+  const string shade_surfKey[]=
+    {
+      "AX", "AZ",  "MAX_VALUE", "MIN_VALUE", "SHADES", "XLOG", "YLOG","ZLOG", 
+      //General Graphics KW
+      "BACKGROUND", "NOERASE",
+      "CHARSIZE", "CHARTHICK", "COLOR", "DATA", "DEVICE", "NORMAL", "FONT",
+      "NODATA", "POSITION", "SUBTITLE", "THICK", "TICKLEN", "TITLE",
+      //Axis KW
+      "ZCHARSIZE", "YCHARSIZE", "XCHARSIZE",
+      "ZGRIDSTYLE", "YGRIDSTYLE", "XGRIDSTYLE",
+      "ZMARGIN", "YMARGIN", "XMARGIN",
+      "ZMINOR", "YMINOR", "XMINOR",
+      "ZRANGE", "YRANGE", "XRANGE",
+      "ZSTYLE", "YSTYLE", "XSTYLE",
+      "ZTHICK", "YTHICK", "XTHICK",
+      "ZTICKFORMAT", "YTICKFORMAT", "XTICKFORMAT",
+      "ZTICKINTERVAL", "YTICKINTERVAL", "XTICKINTERVAL",
+      "ZTICKLAYOUT", "YTICKLAYOUT", "XTICKLAYOUT",
+      "ZTICKLEN", "YTICKLEN", "XTICKLEN",
+      "ZTICKNAME", "YTICKNAME", "XTICKNAME",
+      "ZTICKS", "YTICKS", "XTICKS",
+      "ZTICKUNITS", "YTICKUNITS", "XTICKUNITS",
+      "ZTICKV", "YTICKV", "XTICKV",
+      "ZTICK_GET", "YTICK_GET", "XTICK_GET",
+      "ZTITLE", "YTITLE", "XTITLE",
+      //3D KW
+      "ZVALUE","T3D", "SAVE", "SKIRT", "ZAXIS", KLISTEND
+    };
+  const string shade_surfWarnKey[]=
+  {
+      "IMAGE", "PIXELS", "CHANNEL", KLISTEND
+  };
+  new DLibPro(lib::shade_surf,string("SHADE_SURF"),3,shade_surfKey, shade_surfWarnKey);
 
   const string surfaceKey[]=
     {
@@ -621,7 +697,7 @@ void LibInit()
      // 12
      "NORMAL",    "POSITION",     "SUBTITLE",
      // 15
-     "T3D",     "THICK",    "TICKLEN", 
+     "THICK",    "TICKLEN", 
      // 18
      "TITLE",
      // 19
@@ -655,52 +731,65 @@ void LibInit()
      // 69
      "ZTITLE",
      // 70
-     "ZVALUE", 
+
 
      // SURFACE keywords
      // 73
-     "MAX_VALUE",  "MIN_VALUE", "AX", "AZ", "XLOG", "YLOG", "ZLOG", KLISTEND
+     "MAX_VALUE",  "MIN_VALUE", "AX", "AZ", "XLOG", "YLOG", "ZLOG",
+     "HORIZONTAL", "LOWER_ONLY", "UPPER_ONLY", "SHADES", "ZAXIS",  "BOTTOM", 
+     "SKIRT", "SAVE", "T3D",  "ZVALUE", KLISTEND
     };
   const string surfaceWarnKey[]=
   {
-    "SAVE", "BOTTOM", "HORIZONTAL", "LOWER_ONLY", "UPPER_ONLY",
-    "SHADES", "SKIRT", "ZAXIS", "LEGO", KLISTEND
+      "LEGO", "CHANNEL",KLISTEND
   };
   new DLibPro(lib::surface,string("SURFACE"),3,surfaceKey, surfaceWarnKey);
 
   const string contourKey[]=
     {
-     "BACKGROUND","CHARSIZE","CHARTHICK","CLIP",
-     "COLOR",     "DATA",    "DEVICE",   "FONT",
-     "NOCLIP",  "NODATA",   "NOERASE", 
-     "NORMAL",    "POSITION",     "SUBTITLE",
-     "T3D",     "THICK",    "TICKLEN", 
-     "TITLE",
-     "XCHARSIZE",    "XGRIDSTYLE", "XMARGIN", "XMINOR",
-     "XRANGE",       "XSTYLE",     "XTHICK",  "XTICKFORMAT",
-     "XTICKLEN","XTICKNAME",
-     "XTICKS",       "XTICKV",  "XTICK_GET",
-     "XTITLE",
-     "YCHARSIZE",    "YGRIDSTYLE", "YMARGIN", "YMINOR",
-     "YRANGE",       "YSTYLE",     "YTHICK",  "YTICKFORMAT",
-     "YTICKLEN","YTICKNAME",
-     "YTICKS",       "YTICKV",  "YTICK_GET",
-     "YTITLE",
-     "ZCHARSIZE",    "ZGRIDSTYLE", "ZMARGIN", "ZMINOR",
-     "ZRANGE",       "ZSTYLE",     "ZTHICK",  "ZTICKFORMAT",
-     "ZTICKLEN","ZTICKNAME",
-     "ZTICKS",       "ZTICKV",  "ZTICK_GET",
-     "ZTITLE",
-     "ZVALUE", 
-     "LEVELS", "NLEVELS",
-     "MAX_VALUE", "MIN_VALUE",
-     "XLOG", "XTYPE", "YLOG", "YTYPE", "ZLOG", "ZTYPE", "FILL", "ISOTROPIC",
-     "FOLLOW", "C_CHARSIZE","OVERPLOT","C_COLORS","C_LINESTYLE",
-     KLISTEND
+      "BACKGROUND","CHARSIZE","CHARTHICK","CLIP",
+      "COLOR",     "DATA",    "DEVICE",   "FONT",
+      "NOCLIP",  "NODATA",   "NOERASE",
+      "NORMAL",    "POSITION", "SUBTITLE",
+      "T3D",     "THICK",    "TICKLEN",
+      "TITLE" ,    "LEVELS", "NLEVELS",
+      "MAX_VALUE", "MIN_VALUE",
+      "XLOG", "YLOG", "FILL", "ISOTROPIC",
+      "FOLLOW",
+
+      "XCHARSIZE", "YCHARSIZE", "ZCHARSIZE",
+      "XGRIDSTYLE", "YGRIDSTYLE", "ZGRIDSTYLE",
+      "XMARGIN", "YMARGIN", "ZMARGIN",
+      "XMINOR", "YMINOR", "ZMINOR",
+      "XRANGE", "YRANGE", "ZRANGE",
+      "XSTYLE", "YSTYLE", "ZSTYLE",
+      "XTHICK", "YTHICK", "ZTHICK",
+      "XTICK_GET", "YTICK_GET", "ZTICK_GET",
+      "XTICKFORMAT", "YTICKFORMAT", "ZTICKFORMAT",
+      "XTICKINTERVAL", "YTICKINTERVAL", "ZTICKINTERVAL",
+      "XTICKLAYOUT", "YTICKLAYOUT", "ZTICKLAYOUT",
+      "XTICKLEN", "YTICKLEN", "ZTICKLEN",
+      "XTICKNAME", "YTICKNAME", "ZTICKNAME",
+      "XTICKS", "YTICKS",  "ZTICKS",
+      "XTICKUNITS", "YTICKUNITS",  "ZTICKUNITS",
+      "XTICKV", "YTICKV","ZTICKV",
+      "XTITLE", "YTITLE", "ZTITLE",
+      "ZVALUE", "ZAXIS",
+      "C_CHARSIZE","OVERPLOT","C_COLORS","C_LINESTYLE",
+      "C_LABELS", "C_CHARTHICK", "C_ORIENTATION", "C_SPACING", "C_THICK",
+      "PATH_INFO","PATH_XY",
+      "ZLOG","IRREGULAR", //ZLOG is an addition for GDL only
+      KLISTEND
     };
+   // NO SUPPORT AT ALL for:,"CLOSED","DOWNHILL","IRREGULAR","PATH_DATA_COORDS","PATH_FILENAME",
+   // "PATH_INFO","PATH_XY","TRIANGULATION","PATH_DOUBLE","ZAXIS"
+  // "CHANNEL" is supposed to be passed from CONTOUR, PLOT, OPLOT, SHADE_SURF etc to ERASER
    const string contourWarnKey[]=
     {
-        "CELL_FILL","C_LABELS","C_ANNOTATIONS", "C_CHARTHICK","C_ORIENTATION","C_SPACING","C_THICK",KLISTEND 
+      
+      "CELL_FILL","C_ANNOTATIONS","CLOSED","DOWNHILL",
+      "PATH_DATA_COORDS","PATH_FILENAME",
+      "PATH_INFO","PATH_XY","TRIANGULATION","PATH_DOUBLE","CHANNEL",KLISTEND
     };
    new DLibPro(lib::contour,string("CONTOUR"),3,contourKey,contourWarnKey);
 
@@ -709,104 +798,120 @@ void LibInit()
   const string xyoutsKey[]=
     {
       "COLOR","DATA","NORMAL","DEVICE","CLIP",
-      "ORIENTATION","ALIGNMENT","CHARSIZE","SIZE","CHARTHICK",
-      "NOCLIP","Z","WIDTH","FONT",KLISTEND
+      "ORIENTATION","ALIGNMENT","CHARSIZE","SIZE","CHARTHICK", //note SIZE is not in doc but in XYOUTS demo!
+      "NOCLIP","T3D","Z","TEXT_AXES","WIDTH","FONT",KLISTEND
     };
-   const string xyoutsWarnKey[]=
-    {
-      "TEXT_AXES","T3D", KLISTEND 
-    };  
-   new DLibPro(lib::xyouts, string("XYOUTS"), 3, xyoutsKey, xyoutsWarnKey);
+   const string xyoutsWarnKey[]={KLISTEND};  
+   new DLibPro(lib::xyouts, string("XYOUTS"), 3, xyoutsKey);//, xyoutsWarnKey);
    
    const string polyfillKey[]=
     {
-      "COLOR","DATA","NORMAL","DEVICE","CLIP","NOCLIP",
-      "LINE_FILL","SPACING","LINESTYLE","ORIENTATION","THICK",
+      "CLIP", "COLOR","DATA","DEVICE","NORMAL","LINESTYLE","NOCLIP", //supported graphic KW
+      "ORIENTATION","THICK", //supported graphic KW
+      "LINE_FILL", "SPACING", "T3D","Z", //polyfill specifics
       KLISTEND
     };
-  new DLibPro(lib::polyfill, string("POLYFILL"), 3, polyfillKey);
+   const string polyfillWarnKey[]=
+    {
+     "IMAGE_COORD","IMAGE_INTERP", "PATTERN", "TRANSPARENT",KLISTEND
+    };
+  new DLibPro(lib::polyfill, string("POLYFILL"), 3, polyfillKey,polyfillWarnKey);
+
+  const string scale3Key[]=
+  {
+    "AX", "AZ", "XRANGE", "YRANGE", "ZRANGE", KLISTEND
+  };
+  new DLibPro(lib::scale3_pro, string("SCALE3"), 0, scale3Key); //in plotting_convert_coord.cpp
+
+  const string t3dKey[]=
+  {
+     "RESET", "MATRIX", "OBLIQUE", "PERSPECTIVE", "ROTATE", "SCALE", "TRANSLATE",
+     "XYEXCH", "XZEXCH", "YZEXCH", KLISTEND
+  };
+  new DLibPro(lib::t3d_pro, string("T3D"), 1, t3dKey); //in plotting_convert_coord.cpp
 
   const string eraseKey[]= {"CHANNEL","COLOR",KLISTEND};
   new DLibPro(lib::erase,string("ERASE"),1, eraseKey);
   
   const string laguerreKey[]={"DOUBLE","COEFFICIENTS",KLISTEND};
-  new DLibFun(lib::laguerre,string("LAGUERRE"),3,laguerreKey);
+  new DLibFunRetNew(lib::laguerre,string("LAGUERRE"),3,laguerreKey);
 
-  new DLibFun(lib::gauss_pdf,string("GAUSS_PDF"),1);
-  new DLibFun(lib::gauss_cvf,string("GAUSS_CVF"),1);
+  new DLibFunRetNew(lib::gauss_pdf,string("GAUSS_PDF"),1);
+  new DLibFunRetNew(lib::gauss_cvf,string("GAUSS_CVF"),1);
 
-  new DLibFun(lib::t_pdf,string("T_PDF"),2);
+  new DLibFunRetNew(lib::t_pdf,string("T_PDF"),2);
 
   new DLibPro(lib::caldat, string("CALDAT"), 7);
+  new DLibFunRetNew(lib::julday, string("JULDAY"), 6);
 
   // SA: the HYBRID key is used in imsl_zerosys.pro to switch to the modif. Powell algo. 
   const string newtonKey[] = {"DOUBLE", "ITMAX", "TOLF", "TOLX", "HYBRID", KLISTEND };
   const string newtonWarnKey[] = {"CHECK", "STEPMAX", "TOLMIN", KLISTEND};
-  new DLibFun(lib::newton_broyden, string("NEWTON"), 2, newtonKey, newtonWarnKey);
+  new DLibFunRetNew(lib::newton_broyden, string("NEWTON"), 2, newtonKey, newtonWarnKey);
 
   const string broydenKey[] = {"DOUBLE", "ITMAX", "TOLF", "TOLX", KLISTEND };
   const string broydenWarnKey[] = {"CHECK", "EPS", "STEPMAX", "TOLMIN", KLISTEND};
-  new DLibFun(lib::newton_broyden, string("BROYDEN"), 2, broydenKey, broydenWarnKey);
+  new DLibFunRetNew(lib::newton_broyden, string("BROYDEN"), 2, broydenKey, broydenWarnKey);
 
-  new DLibFun(lib::parse_url, string("PARSE_URL"), 1);
-  new DLibFun(lib::locale_get, string("LOCALE_GET"), 0);
-  new DLibFun(lib::get_login_info, string("GET_LOGIN_INFO"), 0);
-  new DLibFun(lib::idl_base64, string("IDL_BASE64"), 1);
+  new DLibFunRetNew(lib::parse_url, string("PARSE_URL"), 1);
+  new DLibFunRetNew(lib::locale_get, string("LOCALE_GET"), 0);
+  new DLibFunRetNew(lib::get_login_info, string("GET_LOGIN_INFO"), 0);
+  new DLibFunRetNew(lib::idl_base64, string("IDL_BASE64"), 1);
 
   const string ll_arc_distanceKey[] = {"DEGREES", KLISTEND };
-  new DLibFun(lib::ll_arc_distance, string("LL_ARC_DISTANCE"), 3, ll_arc_distanceKey);
+  new DLibFunRetNew(lib::ll_arc_distance, string("LL_ARC_DISTANCE"), 3, ll_arc_distanceKey);
 
   const string command_line_argsKey[] = {"COUNT", KLISTEND };
-  new DLibFun(lib::command_line_args_fun, string("COMMAND_LINE_ARGS"), 0, command_line_argsKey);
+  new DLibFunRetNew(lib::command_line_args_fun, string("COMMAND_LINE_ARGS"), 0, command_line_argsKey);
 
   const string pmKey[] = {"FORMAT", "TITLE", KLISTEND };
   new DLibPro(lib::pm, string("PM"), -1, pmKey);
 
   const string constantKey[] = {"DOUBLE", KLISTEND };
-  new DLibFun(lib::constant, string("IMSL_CONSTANT"), 2, constantKey);
+  new DLibFunRetNew(lib::constant, string("IMSL_CONSTANT"), 2, constantKey);
 
   const string get_drive_listKey[] = {"COUNT", KLISTEND };
-  new DLibFun(lib::get_drive_list, string("GET_DRIVE_LIST"), 0, get_drive_listKey);
+  new DLibFunRetNew(lib::get_drive_list, string("GET_DRIVE_LIST"), 0, get_drive_listKey);
 
   const string binomialcoefKey[] = {"DOUBLE", KLISTEND };
-  new DLibFun(lib::binomialcoef, string("IMSL_BINOMIALCOEF"), 2, binomialcoefKey);
+  new DLibFunRetNew(lib::binomialcoef, string("IMSL_BINOMIALCOEF"), 2, binomialcoefKey);
 
   // SA: GRIB format support based on the ECMWF GRIB_API package (IDL does not support it yet)
   // GRIBAPI_ prefix is used in order to (hopefully) prevent future incompatibilities with IDL
   // -----------------------------------------------------------------------------------------
   // GRIB: file related
-  new DLibFun(lib::grib_open_file_fun, string("GRIBAPI_OPEN_FILE"), 1); 
-  new DLibFun(lib::grib_count_in_file_fun, string("GRIBAPI_COUNT_IN_FILE"), 1); 
+  new DLibFunRetNew(lib::grib_open_file_fun, string("GRIBAPI_OPEN_FILE"), 1); 
+  new DLibFunRetNew(lib::grib_count_in_file_fun, string("GRIBAPI_COUNT_IN_FILE"), 1); 
   new DLibPro(lib::grib_close_file_pro, string("GRIBAPI_CLOSE_FILE"), 1); 
   // GRIB: message related
-  new DLibFun(lib::grib_new_from_file_fun, string("GRIBAPI_NEW_FROM_FILE"), 1); 
+  new DLibFunRetNew(lib::grib_new_from_file_fun, string("GRIBAPI_NEW_FROM_FILE"), 1); 
   new DLibPro(lib::grib_release_pro, string("GRIBAPI_RELEASE"), 1); 
   // see comment in grib.cpp
-  //new DLibFun(lib::grib_get_message_size_fun, string("GRIBAPI_GET_MESSAGE_SIZE"), 1);
-  new DLibFun(lib::grib_clone_fun, string("GRIBAPI_CLONE"), 1); 
+  //new DLibFunRetNew(lib::grib_get_message_size_fun, string("GRIBAPI_GET_MESSAGE_SIZE"), 1);
+  new DLibFunRetNew(lib::grib_clone_fun, string("GRIBAPI_CLONE"), 1); 
   // GRIB: data related
-  new DLibFun(lib::grib_get_size_fun, string("GRIBAPI_GET_SIZE"), 2); 
+  new DLibFunRetNew(lib::grib_get_size_fun, string("GRIBAPI_GET_SIZE"), 2); 
   new DLibPro(lib::grib_get_pro, string("GRIBAPI_GET"), 3); 
   new DLibPro(lib::grib_get_data_pro, string("GRIBAPI_GET_DATA"), 4); 
   // -----------------------------------------------------------------------------------------
 
-  new DLibFun(lib::crossp, string("CROSSP"), 2);
+  new DLibFunRetNew(lib::crossp, string("CROSSP"), 2);
   //const string hanningKey[] = {"DOUBLE", "ALPHA", KLISTEND };
-  //new DLibFun(lib::hanning, string("HANNING"), 2, hanningKey);
+  //new DLibFunRetNew(lib::hanning, string("HANNING"), 2, hanningKey);
   const string wtnKey[] = {"COLUMN", "DOUBLE", "INVERSE", "OVERWRITE", KLISTEND };
   new DLibFun(lib::wtn, string("WTN"), 2, wtnKey);
   const string zeropolyKey[] = {"DOUBLE", "COMPANION", "JENKINS_TRAUB", KLISTEND };
-  new DLibFun(lib::zeropoly, string("IMSL_ZEROPOLY"), 1, zeropolyKey);
+  new DLibFunRetNew(lib::zeropoly, string("IMSL_ZEROPOLY"), 1, zeropolyKey);
   const string spher_harmKey[] = {"DOUBLE", KLISTEND };
-  new DLibFun(lib::spher_harm, string("SPHER_HARM"), 4, spher_harmKey);
+  new DLibFunRetNew(lib::spher_harm, string("SPHER_HARM"), 4, spher_harmKey);
   const string gdl_erfinvKey[] = {"DOUBLE", KLISTEND };
-  new DLibFun(lib::gdl_erfinv_fun, string("GDL_ERFINV"), 1, gdl_erfinvKey);
+  new DLibFunRetNew(lib::gdl_erfinv_fun, string("GDL_ERFINV"), 1, gdl_erfinvKey);
 
   // by Mateusz Turcza
   const string sem_createKey[] = {"DESTROY_SEMAPHORE", KLISTEND };
-  new DLibFun(lib::sem_create, string("SEM_CREATE"), 1, sem_createKey);
+  new DLibFunRetNew(lib::sem_create, string("SEM_CREATE"), 1, sem_createKey);
   new DLibPro(lib::sem_delete, string("SEM_DELETE"), 1);
-  new DLibFun(lib::sem_lock, string("SEM_LOCK"), 1);
+  new DLibFunRetNew(lib::sem_lock, string("SEM_LOCK"), 1);
   new DLibPro(lib::sem_release, string("SEM_RELEASE"), 1);
 
   // sort lists
