@@ -135,11 +135,11 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
       BinaryExpr* newNode = NULL;
       switch( refNode->getType())
 	{
-	case GDLTokenTypes::QUESTION:
-	  {
-	    return new QUESTIONNode( refNode);
-	  }
-
+// 	case GDLTokenTypes::QUESTION:
+// 	  {
+// 	    return new QUESTIONNode( refNode);
+// 	  }
+// 
 	  // unary
 	case GDLTokenTypes::UMINUS:
 	  {
@@ -359,7 +359,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	{
 	  if( !newNode->ConstantExpr()) return newNode;
 
-	  auto_ptr<ProgNode> guard( newNode);
+	  Guard<ProgNode> guard( newNode);
 
 	  BaseGDL* cData = newNode->Eval();
 
@@ -373,7 +373,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	{
 	  if( !newUnary->ConstantExpr()) return newUnary;
 
-	  auto_ptr<ProgNode> guard( newUnary);
+	  Guard<ProgNode> guard( newUnary);
 
 	  BaseGDL* cData = newUnary->Eval();
 
@@ -390,11 +390,11 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
       BinaryExpr* newNode = NULL;
       switch( refNode->getType())
 	{
-	case GDLTokenTypes::QUESTION:
-	  {
-	    return new QUESTIONNode( refNode);
-	  }
-
+// 	case GDLTokenTypes::QUESTION:
+// 	  {
+// 	    return new QUESTIONNode( refNode);
+// 	  }
+// 
 	  // unary
 	case GDLTokenTypes::UMINUS:
 	  {
@@ -540,7 +540,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	{
 	  if( !newNode->ConstantExpr()) return newNode;
 
-	  auto_ptr<ProgNode> guard( newNode);
+	  Guard<ProgNode> guard( newNode);
 
 	  BaseGDL* cData = newNode->Eval();
 
@@ -554,7 +554,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	{
 	  if( !newUnary->ConstantExpr()) return newUnary;
 
-	  auto_ptr<ProgNode> guard( newUnary);
+	  Guard<ProgNode> guard( newUnary);
 
 	  BaseGDL* cData = newUnary->Eval();
 
@@ -569,6 +569,11 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
   // independed of nonCopy:
   switch( refNode->getType())
     {
+    case GDLTokenTypes::QUESTION:
+      {
+	return new QUESTIONNode( refNode);
+      }
+
     case GDLTokenTypes::FCALL_LIB_RETNEW:
       {
 	ProgNodeP c = new FCALL_LIB_RETNEWNode( refNode);
@@ -576,7 +581,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	if( !static_cast<DLibFunRetNew*>(c->libFun)->RetConstant()
 	    || !ConstantPar( c->getFirstChild())) return c;
 
-	auto_ptr< ProgNode> guard( c);
+	Guard< ProgNode> guard( c);
 
 	BaseGDL* cData = c->Eval();
 
@@ -593,7 +598,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	if( !static_cast<DLibFunDirect*>(c->libFun)->RetConstant()
 	    || !ConstantPar( c->getFirstChild())) return c;
 
-	auto_ptr< ProgNode> guard( c);
+	Guard< ProgNode> guard( c);
 
 	BaseGDL* cData = c->Eval();
 
@@ -610,7 +615,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	if( !static_cast<DLibFunRetNew*>(c->libFun)->RetConstant()
 	    || !ConstantPar( c->getFirstChild())) return c;
 
-	auto_ptr< ProgNode> guard( c);
+	Guard< ProgNode> guard( c);
 
 	BaseGDL* cData = c->Eval();
 
@@ -698,7 +703,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	ARRAYDEFNode* c = new ARRAYDEFNode( refNode);
 	if( !c->ConstantArray()) return c;
 
-	auto_ptr< ARRAYDEFNode> guard( c);
+	Guard< ARRAYDEFNode> guard( c);
 
 	BaseGDL* cData = c->Eval();
 
@@ -723,7 +728,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 //   cout << endl;
 	assert( c->ConstantArray());
 
-	auto_ptr< ProgNode> guard( c);
+	Guard< ProgNode> guard( c);
 
 	BaseGDL* cData = c->Eval();
 
@@ -741,7 +746,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 // 	refNode->setNextSibling(antlr::nullAST);
 
 // 	ProgNodeP c = new ARRAYDEFNode( refNode);
-// 	auto_ptr< ProgNode> guard( c);
+// 	Guard< ProgNode> guard( c);
 // 	//c->setType(  GDLTokenTypes::ARRAYDEF);
 
 // 	// evaluate constant
@@ -963,7 +968,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	new ParameterNode( refNode);//->GetFirstChild());
       return firstChild;
 
-// 		  auto_ptr<ProgNode> guard(nn);
+// 		  Guard<ProgNode> guard(nn);
 // 		  return nn->StealFirstChild();
     }
     case GDLTokenTypes::REF_EXPR_VN:
@@ -978,7 +983,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	new ParameterNode( refNode);//->GetFirstChild());
       return firstChild;
 
-// 		  auto_ptr<ProgNode> guard(nn);
+// 		  Guard<ProgNode> guard(nn);
 // 		  return nn->StealFirstChild();
     }
     case GDLTokenTypes::KEYDEF_REF_CHECK:
@@ -1004,7 +1009,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	new ParameterNode( refNode);
       return firstChild;
 
-// 		  auto_ptr<ProgNode> guard(nn);
+// 		  Guard<ProgNode> guard(nn);
 // 		  ProgNodeP firstChild = nn->StealFirstChild();
 // 		  firstChild->SetNextSibling( nn->StealNextSibling());
 // 		  return firstChild;
@@ -1021,7 +1026,7 @@ ProgNodeP ProgNode::NewProgNode( const RefDNode& refNode)
 	new ParameterNode( refNode);
       return firstChild;
 
-// 		  auto_ptr<ProgNode> guard(nn);
+// 		  Guard<ProgNode> guard(nn);
 // 		  ProgNodeP firstChild = nn->StealFirstChild();
 // 		  firstChild->SetNextSibling( nn->StealNextSibling());
 // 		  return firstChild;
