@@ -24,18 +24,101 @@ namespace lib{
 
 using namespace std;
 
-// get cursor from plPlot     AC February 2008
-// known limitations : WAIT parameter and similar keywords not fully managed (wait, nowait ...)
+void empty(EnvT* e)
+{
+  Graphics* actDevice = Graphics::GetDevice();
+  if (actDevice->Name() == "X")
+  {
+    GDLGStream *plg = actDevice->GetStream();
+    if (plg != NULL) plg->Flush();
+  }
+}
 
+void tvcrs( EnvT* e)
+{
+  Graphics* actDevice = Graphics::GetDevice();
+
+  if (actDevice->Name() != "X")
+  {
+    e->Throw("Routine is not defined for current graphics device.");
+  }
+  SizeT nParam = e->NParam(1);
+
+  if (nParam < 2 )
+  {
+    e->Throw("TVCRS with 1 argument not implemented (fixme)");
+  }
+  DDoubleGDL *x,*y;
+
+  x = e->GetParAs< DDoubleGDL > (0);
+  y = e->GetParAs< DDoubleGDL > (1);
+
+  GDLGStream *plg = actDevice->GetStream();
+  if (plg == NULL) e->Throw("Unable to create window.");
+  PLINT plplot_level;
+  plg->glevel(plplot_level);
+  // when level < 2, we have to read if ![x|y].crange exist
+  // if not, we have to build a [0,1]/[0,1] window
+  if (plplot_level < 2)
+  {
+    plg->NextPlot();
+
+    plg->vpor(0, 1, 0, 1);
+    plg->wind(0, 1, 0, 1);
+
+  }
+
+  PLFLT ix,iy;
+
+  if (e->KeywordSet("DATA")) // /DATA
+  {
+    DDouble tempx,tempy;
+    tempx=(*x)[0];
+    tempy=(*y)[0];
+#ifdef USE_LIBPROJ4
+    bool mapSet = false;
+    get_mapset(mapSet);
+    if (mapSet)
+    {
+      PROJTYPE* ref = map_init();
+      if (ref == NULL) e->Throw("Projection initialization failed.");
+      LPTYPE idataN;
+      idataN.lam = tempx* RAD_TO_DEG;
+      idataN.phi = tempy* RAD_TO_DEG;
+      XYTYPE odata = PJ_FWD(idataN, ref);
+      tempx = odata.x;
+      tempy = odata.y;
+    }
+#endif
+    bool xLog, yLog;
+    gdlGetAxisType("X", xLog);
+    gdlGetAxisType("Y", yLog);
+    if(xLog) tempx=pow(10,tempx);
+    if(yLog) tempy=pow(10,tempy);
+    plg->WorldToDevice(tempx,tempy,ix,iy);
+  }
+  else if (e->KeywordSet("NORMAL"))
+  {
+    plg->NormedDeviceToDevice((*x)[0],(*y)[0],ix,iy);
+  }
+  else // (e->KeywordSet("DEVICE"))
+  {
+    ix=(*x)[0];
+    iy=(*y)[0];
+  }
+  plg->WarpPointer(ix,iy);
+}
+
+// get cursor from plPlot     AC February 2008
+// fully compatible with IDL using our own cursor routines GD Jan 2013
 void cursor(EnvT* e){
   Graphics* actDevice = Graphics::GetDevice();
-  //cout << actDevice->Name() << endl;
+
   if (actDevice->Name() != "X")
   {
     e->Throw("Routine is not defined for current graphics device.");
   }
 
-  int debug = 0;
   SizeT nParam = e->NParam(1);
 
   if (nParam < 2 || nParam > 3)
@@ -51,12 +134,8 @@ void cursor(EnvT* e){
 
   static PLGraphicsIn gin;
 
-  // content of : plGetCursor();
-  // [retval, state, keysym, button, string, pX, pY, dX, dY, wX, wY, subwin]
-
   PLINT plplot_level;
   plg->glevel(plplot_level);
-  if (debug) cout << "Plplot_level : " << plplot_level << endl;
   // when level < 2, we have to read if ![x|y].crange exist
   // if not, we have to build a [0,1]/[0,1] window
   if (plplot_level < 2)
@@ -66,124 +145,73 @@ void cursor(EnvT* e){
     plg->vpor(0, 1, 0, 1);
     plg->wind(0, 1, 0, 1);
 
-    /* we are not ready for the correct way (rebuilding a window following
-   stored info)
-
-    // we have to read them back from !x.crange and !y.crange
-    PLFLT xStart, xEnd, yStart, yEnd;
-    get_axis_crange("X", xStart, xEnd);
-    get_axis_crange("Y", yStart, yEnd);
-    if ((xStart == xEnd ) || ( xEnd ==0.0)) {
-  xStart=0.0;
-  xEnd=1.0;
-    }
-    if ((yStart == yEnd ) || ( yEnd ==0.0)) {
-  yStart=0.0;
-  yEnd=1.0;
-    }
-    AC_debug("crange", xStart, xEnd, yStart, yEnd);
-
-    bool okVPWC = SetVP_WC( e, plg, NULL, NULL, 0, 0,
-                xStart, xEnd, yStart, yEnd, false, false);
-    if( !okVPWC) return;
-     */
   }
-
+  // mimic idl logic:
   DLong wait = 1;
-
   if (nParam == 3)
   {
     e->AssureLongScalarPar(2, wait);
   }
+  if (e->KeywordSet("NOWAIT")) wait=0;
+  if (e->KeywordSet("CHANGE")) wait=2;
+  if (e->KeywordSet("WAIT")) wait=1;
+  if (e->KeywordSet("DOWN")) wait=3;
+  if (e->KeywordSet("UP")) wait=4;
+  PLFLT xp, yp;
+  PLINT xleng, yleng, xoff, yoff;
+  plg->gpage(xp, yp, xleng, yleng, xoff, yoff);
 
-  if ((wait == 1) || (wait == 3) || (wait == 4) ||
-  e->KeywordSet("WAIT") ||
-  e->KeywordSet("DOWN") ||
-  e->KeywordSet("UP"))
+  if (wait == 0)
   {
-    //cout << "Sorry, this option is currently not *really* managed. Help welcome" << endl;
-    // we toggle to "wait == 1" (the mouse can move but we return if mouse is pressed)
-    wait = 1;
+    if(plg->GetGin(&gin, 0)==false) return;
   }
-
-  int mode = 0; // just a flag to manage the general case (cursor,x,y)
-
-  if ((wait == 0) || e->KeywordSet("NOWAIT"))
+  else if (wait == 2)
   {
-    gin.button = 1;
-    plg->GetCursor(&gin);
-    gin.button = 0;
-    mode = 1;
-    wait = 0;
-  }
-  if (wait == 1)
-  {
-    while (1)
-    {
-      plg->GetCursor(&gin);
-      //	cout << gin.button << endl;
-      if (gin.button > 0) break;
-      if (sigControlC)
-        return;
-    }
-    mode = 1;
-  }
-  if ((wait == 2) || e->KeywordSet("CHANGE"))
-  {
-    plg->GetCursor(&gin);
-    long RefX, RefY;
+    if(plg->GetGin(&gin, 0)==false) return;
+    PLFLT RefX, RefY;
     RefX = gin.pX;
     RefY = gin.pY;
-    if (gin.button == 0)
-    {
-      while (1)
-      {
-        plg->GetCursor(&gin);
-        if (abs(RefX - gin.dX) > 0 || abs(RefY - gin.dY) > 0) break;
-        if (gin.button > 0) break;
-        if (sigControlC)
-          return;
-      }
-    }
-    mode = 1;
-  }
-
-  if (mode == 0)
-  {
+    unsigned int refstate=gin.state;
     while (1)
     {
-      plg->GetCursor(&gin);
-      // TODO: When no Mouse, should be extended later to any key of the keyboard
-      if (gin.keysym == PLK_Escape) break;
-      if (gin.button > 0) break;
-      if (sigControlC)
-        return;
+      if(plg->GetGin(&gin, 2)==false) return;
+      if (abs(RefX - gin.pX) > 0 || abs(RefY - gin.pY) > 0)
+      {
+        RefX = gin.pX;
+        RefY = gin.pY;
+        break;
+      }
+      if (gin.state != refstate)
+      {
+        refstate=gin.state;
+        break;
+      }
     }
   }
-
-  if (debug)
+  else if (wait == 3)
   {
-    // plg->text();
-    cout << "mouse button : " << gin.button << endl;
-    cout << "keysym : " << gin.keysym << endl;
-    //plg->gra();
-    /* if (gin.keysym < 0xFF && isprint(gin.keysym))
-   cout << "wx = " << gin.wX << ", wy = " << gin.wY <<
-   ", dx = " << gin.dX << ",  dy = " << gin.dY <<
-   ",  c = '" << gin.keysym << "'" << endl;
-   plg->gra(); */
+    if(plg->GetGin(&gin, 3)==false) return;
+  }
+  else if (wait == 4)
+  {
+    if(plg->GetGin(&gin, 4)==false) return;
+  }
+  else
+  {
+    if(plg->GetGin(&gin, 1)==false) return;
+  }
+  // outside window report -1 -1 at least for DEVICE values
+  if (gin.pX < 0 || gin.pX > plg->xPageSize() || gin.pY < 0 || gin.pY > plg->yPageSize())
+  {
+    gin.pX = -1;
+    gin.pY = -1;
   }
-
   if (e->KeywordSet("DEVICE"))
   {
-    PLFLT xp, yp;
-    PLINT xleng, yleng, xoff, yoff;
-    plg->gpage(xp, yp, xleng, yleng, xoff, yoff);
-
     DLongGDL* xLong;
     DLongGDL* yLong;
     xLong = new DLongGDL(gin.pX);
-    yLong = new DLongGDL(yleng - gin.pY);
+    yLong = new DLongGDL(gin.pY);
 
     e->SetPar(0, xLong);
     e->SetPar(1, yLong);
@@ -206,8 +234,7 @@ void cursor(EnvT* e){
       if (!mapSet)
       {
 #endif
-
-        getWorldCoordinatesFromPLPLOT(plg, (DDouble)gin.dX, (DDouble)gin.dY, &tempx, &tempy);
+        plg->NormToWorld((DDouble)gin.dX, (DDouble)gin.dY, tempx, tempy);
 #ifdef USE_LIBPROJ4
       }
       else
@@ -217,15 +244,15 @@ void cursor(EnvT* e){
         XYTYPE idata, idataN;
         idataN.x = gin.dX;
         idataN.y = gin.dY;
-        getWorldCoordinatesFromPLPLOT(plg, idataN.x, idataN.y, &idata.x, &idata.y);
+        plg->NormToWorld(idataN.x, idataN.y, idata.x, idata.y);
         LPTYPE odata = PJ_INV(idata, ref);
         tempx = odata.lam * RAD_TO_DEG;
         tempy = odata.phi * RAD_TO_DEG;
       }
 #endif
       bool xLog, yLog;
-      get_axis_type("X", xLog);
-      get_axis_type("Y", yLog);
+      gdlGetAxisType("X", xLog);
+      gdlGetAxisType("Y", yLog);
       if(xLog) tempx=pow(10,tempx);
       if(yLog) tempy=pow(10,tempy);
       x = new DDoubleGDL(tempx);
@@ -236,6 +263,9 @@ void cursor(EnvT* e){
   }
 
   // we update the !Mouse structure (4 fields, only 3 managed up to now)
+  // found on the web:
+  //"Information about which mouse button has been used (if) any is stored in the !err variable. A value of 1 corresponds to the left, 2 to middle and 4 to the right button."
+  //!err is obsolete but still working:
   DStructGDL* Struct = SysVar::Mouse();
   if (Struct != NULL)
   {
@@ -247,6 +277,8 @@ void cursor(EnvT* e){
     if (gin.button == 3) gin.button = 4; // 4 values only (0,1,2,4)
     (*static_cast<DLongGDL*>(Struct->GetTag(ButtonMouseTag)))[0] = gin.button;
   }
+  DVar *err=FindInVarList(sysVarList, "ERR");
+  (static_cast<DLongGDL*>(err->Data()))[0]=  gin.button;
 }
 
 } // namespace
