@@ -24,23 +24,31 @@ namespace lib {
 
   class oplot_call : public plotting_routine_call 
   {
-    DDoubleGDL *yVal, *xVal, *xTemp, *yTemp;
-    SizeT xEl, yEl;
-    auto_ptr<BaseGDL> xval_guard,yval_guard,xtempval_guard;
+    DDoubleGDL *yVal, *xVal, *zVal, *xTemp, *yTemp;
+    SizeT xEl, yEl, zEl;
+    Guard<BaseGDL> xval_guard,yval_guard, zval_guard, xtempval_guard;
+    bool doT3d;
+    DDouble zValue;
 
     private: bool handle_args( EnvT* e) // {{{
     {
+      //T3D?
+      static int t3dIx = e->KeywordIx( "T3D");
+      doT3d=(e->KeywordSet(t3dIx)|| T3Denabled(e));
+
+      //note: Z (VALUE) will be used uniquely if Z is not effectively defined.
+      zValue=0.0;
+      static int zvIx = e->KeywordIx( "ZVALUE");
+      e->AssureDoubleScalarKWIfPresent ( zvIx, zValue );
+
       bool polar=FALSE;
       DLong nsum=1;
       e->AssureLongScalarKWIfPresent( "NSUM", nsum);
       if ( e->KeywordSet( "POLAR"))
       {
         polar=TRUE;
-       // e->Throw( "Sorry, POLAR keyword not ready");
       }
 
-      DDoubleGDL *yValBis, *xValBis;
-      auto_ptr<BaseGDL> xvalBis_guard, yvalBis_guard;
       //test and transform eventually if POLAR and/or NSUM!
       if( nParam() == 1)
       {
@@ -50,7 +58,7 @@ namespace lib {
         yEl=yTemp->N_Elements();
         xEl=yEl;
         xTemp = new DDoubleGDL( dimension( xEl), BaseGDL::INDGEN);
-        xtempval_guard.reset( xTemp); // delete upon exit
+        xtempval_guard.Reset( xTemp); // delete upon exit
       }
       else
       {
@@ -80,15 +88,15 @@ namespace lib {
         if (polar)
         {
           xVal = new DDoubleGDL(dimension(xEl), BaseGDL::NOZERO);
-          xval_guard.reset(xVal); // delete upon exit
+          xval_guard.Reset(xVal); // delete upon exit
           yVal = new DDoubleGDL(dimension(yEl), BaseGDL::NOZERO);
-          yval_guard.reset(yVal); // delete upon exit
+          yval_guard.Reset(yVal); // delete upon exit
           for (int i = 0; i < xEl; i++) (*xVal)[i] = (*xTemp)[i] * cos((*yTemp)[i]);
           for (int i = 0; i < yEl; i++) (*yVal)[i] = (*xTemp)[i] * sin((*yTemp)[i]);
         }
         else
         { //careful about previously set autopointers!
-          if (nParam() == 1) xval_guard = xtempval_guard;
+          if (nParam() == 1) xval_guard.Init( xtempval_guard.release());
           xVal = xTemp;
           yVal = yTemp;
         }
@@ -98,9 +106,9 @@ namespace lib {
         int i, j, k;
         DLong size = xEl / nsum;
         xVal = new DDoubleGDL(size, BaseGDL::ZERO); //SHOULD BE ZERO, IS NOT!
-        xval_guard.reset(xVal); // delete upon exit
+        xval_guard.Reset(xVal); // delete upon exit
         yVal = new DDoubleGDL(size, BaseGDL::ZERO); //IDEM
-        yval_guard.reset(yVal); // delete upon exit
+        yval_guard.Reset(yVal); // delete upon exit
         for (i = 0, k = 0; i < size; i++)
         {
           (*xVal)[i] = 0.0;
@@ -126,25 +134,32 @@ namespace lib {
           }
         }
       }
+      if (doT3d)
+      {
+        //make zVal
+        zEl=xVal->N_Elements();
+        zVal=new DDoubleGDL(dimension(zEl), BaseGDL::NOZERO);
+        zval_guard.Reset(zVal); // delete upon exit
+        for (SizeT i=0; i< zEl ; ++i) (*zVal)[i]=zValue;
+      }
+	  return 0;
     }
 
-  private: void old_body( EnvT* e, GDLGStream* actStream) // {{{
+  private: void old_body( EnvT* e, GDLGStream* actStream) 
   {
-    bool valid;
-    valid=true;
     DLong psym;
 
     // get ![XY].CRANGE
     DDouble xStart, xEnd, yStart, yEnd;
-    get_axis_crange("X", xStart, xEnd);
-    get_axis_crange("Y", yStart, yEnd);
+    gdlGetCurrentAxisRange("X", xStart, xEnd);
+    gdlGetCurrentAxisRange("Y", yStart, yEnd);
     DDouble minVal, maxVal;
     bool doMinMax;
 
     bool xLog;
     bool yLog;
-    get_axis_type("X", xLog);
-    get_axis_type("Y", yLog);
+    gdlGetAxisType("X", xLog);
+    gdlGetAxisType("Y", yLog);
 
     if ((yStart == yEnd) || (xStart == xEnd))
     {
@@ -157,9 +172,6 @@ namespace lib {
         Message("OPLOT: !X.CRANGE ERROR, setting to [0,1]");
       xStart = 0; //xVal->min();
       xEnd = 1; //xVal->max();
-
-      set_axis_crange("X", xStart, xEnd, xLog);
-      set_axis_crange("Y", yStart, yEnd, yLog);
     }
     
     //now we can setup minVal and maxVal to defaults: Start-End and overload if KW present
@@ -172,45 +184,104 @@ namespace lib {
     e->AssureDoubleScalarKWIfPresent( "MIN_VALUE", minVal);
     e->AssureDoubleScalarKWIfPresent( "MAX_VALUE", maxVal);
 
-    // CLIPPING
-    DDoubleGDL* clippingD=NULL;
-    DLong noclip=0;
-    e->AssureLongScalarKWIfPresent( "NOCLIP", noclip);
-    if(noclip == 0)
+    int noclipvalue=0;
+    e->AssureLongScalarKWIfPresent( "NOCLIP", noclipvalue);
+    // Clipping is enabled by default for OPLOT.
+    // make all clipping computations BEFORE setting graphic properties (color, size)
+    bool doClip=(e->KeywordSet("CLIP")||noclipvalue==1);
+    bool stopClip=false;
+    if ( doClip )  if ( startClipping(e, actStream, false)==TRUE ) stopClip=true;
+
+    // start drawing. Graphic Keywords accepted:CLIP(YES), COLOR(YES), LINESTYLE(YES), NOCLIP(YES),
+    //                                          PSYM(YES), SYMSIZE(YES), T3D(YES), ZVALUE(YES)
+    gdlSetGraphicsForegroundColorFromKw(e, actStream);
+    gdlGetPsym(e, psym);
+    gdlSetPenThickness(e, actStream);
+    gdlSetSymsize(e, actStream);
+    gdlSetLineStyle(e, actStream);
+
+    static DDouble x0,y0,xs,ys; //conversion to normalized coords
+    x0=(xLog)?-log10(xStart):-xStart;
+    y0=(yLog)?-log10(yStart):-yStart;
+    xs=(xLog)?(log10(xEnd)-log10(xStart)):xEnd-xStart;xs=1.0/xs;
+    ys=(yLog)?(log10(yEnd)-log10(yStart)):yEnd-yStart;ys=1.0/ys;
+
+    if ( doT3d ) //convert X,Y,Z in X',Y' as per T3D perspective.
+    {
+      DDoubleGDL* plplot3d;
+      DDouble az, alt, ay, scale;
+      ORIENTATION3D axisExchangeCode;
+
+      plplot3d = gdlConvertT3DMatrixToPlplotRotationMatrix( zValue, az, alt, ay, scale, axisExchangeCode);
+      if (plplot3d == NULL)
       {
-	static int clippingix = e->KeywordIx( "CLIP"); 
-	clippingD = e->IfDefGetKWAs<DDoubleGDL>( clippingix);
+        e->Throw("Illegal 3D transformation. (FIXME)");
       }
-    
-    // start drawing. Graphic Keywords accepted:CLIP(NO), COLOR(YES), LINESTYLE(YES), NOCLIP(YES),
-    //                                          PSYM(YES), SYMSIZE(YES), T3D(NO), ZVALUE(NO)
-    gkw_background(e, actStream, false);
-    gkw_color(e, actStream);
-    //    gkw_noerase(e, actStream, true);
-    gkw_psym(e, psym);
-    DFloat charsize;
-    gkw_charsize(e,actStream, charsize, false); //set !P.CHARSIZE
-    gkw_thick(e, actStream);
-    gkw_symsize(e, actStream);
-    gkw_linestyle(e, actStream);
+      Data3d.zValue = zValue;
+      Data3d.Matrix = plplot3d; //try to change for !P.T in future?
+        switch (axisExchangeCode) {
+          case NORMAL: //X->X Y->Y plane XY
+            Data3d.x0=x0;
+            Data3d.y0=y0;
+            Data3d.xs=xs;
+            Data3d.ys=ys;
+            Data3d.code = code012;
+            break;
+          case XY: // X->Y Y->X plane XY
+            Data3d.x0=0;
+            Data3d.y0=x0;
+            Data3d.xs=ys;
+            Data3d.ys=xs;
+            Data3d.code = code102;
+            break;
+          case XZ: // Y->Y X->Z plane YZ
+            Data3d.x0=x0;
+            Data3d.y0=y0;
+            Data3d.xs=xs;
+            Data3d.ys=ys;
+            Data3d.code = code210;
+            break;
+          case YZ: // X->X Y->Z plane XZ
+            Data3d.x0=x0;
+            Data3d.y0=y0;
+            Data3d.xs=xs;
+            Data3d.ys=ys;
+            Data3d.code = code021;
+            break;
+          case XZXY: //X->Y Y->Z plane YZ
+            Data3d.x0=x0;
+            Data3d.y0=y0;
+            Data3d.xs=xs;
+            Data3d.ys=ys;
+            Data3d.code = code120;
+            break;
+          case XZYZ: //X->Z Y->X plane XZ
+            Data3d.x0=x0;
+            Data3d.y0=y0;
+            Data3d.xs=xs;
+            Data3d.ys=ys;
+            Data3d.code = code201;
+            break;
+        }
 
-    // plot the data
-    if(valid) //invalid is not yet possible. Could be done by a severe clipping for example.
-      valid=draw_polyline(e, actStream, 
-			  xVal, yVal, minVal, maxVal, doMinMax, xLog, yLog,
-			  psym, FALSE);
+        actStream->stransform(gdl3dTo2dTransform, &Data3d);
+    }
 
+      // TODO: handle "valid"!
+    bool valid=draw_polyline(e, actStream, xVal, yVal, minVal, maxVal, doMinMax, xLog, yLog, psym, FALSE);
+    if (stopClip) stopClipping(actStream);
+  } 
 
-    actStream->lsty(1);//reset linestyle
-  } // }}}
-
-    private: void call_plplot(EnvT* e, GDLGStream* actStream) // {{{
+    private: void call_plplot(EnvT* e, GDLGStream* actStream) 
     {
-    } // }}}
+    } 
 
-    private: void post_call(EnvT* e, GDLGStream* actStream) // {{{
+    private: void post_call(EnvT* e, GDLGStream* actStream)
     {
-    } // }}}
+     if (doT3d) actStream->stransform(NULL,NULL);
+      actStream->lsty(1);//reset linestyle
+      actStream->sizeChar(1.0);
+    } 
 
   }; // oplot_call class 
 
