@@ -61,7 +61,7 @@ namespace lib {
 	title = "GDL "+i2s( wIx);
       }
 
-    DLong xPos=0, yPos=0;
+    DLong xPos=-1, yPos=-1;
     e->AssureLongScalarKWIfPresent( "XPOS", xPos);
     e->AssureLongScalarKWIfPresent( "YPOS", yPos);
 
@@ -71,6 +71,16 @@ namespace lib {
 #else
     xSize = 640;
     ySize = 512;
+#ifdef _MSC_VER
+    // the following not tested (no access to windows) (GD)
+    bool noQscreen=true;
+    string gdlQscreen=GetEnvString("GDL_GR_WIN_QSCREEN");
+    if( gdlQscreen == "1") noQscreen=false;
+    string gdlXsize=GetEnvString("GDL_GR_WIN_WIDTH");
+    if( gdlXsize != "" && noQscreen ) *xSize=atoi(gdlXsize.c_str()); 
+    string gdlYsize=GetEnvString("GDL_GR_WIN_HEIGHT");
+    if( gdlYsize != "" && noQscreen) *ySize=atoi(gdlYsize.c_str()); 
+#endif
 #endif
     e->AssureLongScalarKWIfPresent( "XSIZE", xSize);
     e->AssureLongScalarKWIfPresent( "YSIZE", ySize);
@@ -80,8 +90,7 @@ namespace lib {
       cout << "xPos/yPos   :"<<  xPos << " " << yPos << endl;
       cout << "xSize/ySize :"<<  xSize << " " << ySize << endl;
     }
-
-    if( xSize <= 0 || ySize <= 0 || xPos < 0 || yPos < 0)
+    if( xSize <= 0 || ySize <= 0 || xPos < -1 || yPos < -1)
       e->Throw(  "Unable to create window "
 		 "(BadValue (integer parameter out of range for "
 		 "operation)).");
@@ -89,7 +98,18 @@ namespace lib {
     bool success = actDevice->WOpen( wIx, title, xSize, ySize, xPos, yPos);
     if( !success)
       e->Throw(  "Unable to create window.");
-  }
+    success = actDevice->CursorCrosshair();
+    success = actDevice->UnsetFocus();
+    bool doretain=true;
+    DLong retainType ; //=Graphics::getRetain();
+//    if (retainType=0) doretain=false;
+    if( e->KeywordPresent( 3)) // RETAIN
+    {
+      e->AssureLongScalarKWIfPresent( "RETAIN", retainType);
+      if (retainType == 0) doretain=false;
+    }
+    success = actDevice->EnableBackingStore(doretain);
+ }
 
   void wset( EnvT* e)
   {
@@ -116,9 +136,12 @@ namespace lib {
                 xSize = 640;
                 ySize = 512;
             #endif
-	    bool success = actDevice->WOpen( 0, "GDL 0", xSize, ySize, 0, 0);
+	    bool success = actDevice->WOpen( 0, "GDL 0", xSize, ySize, -1, -1);
 	    if( !success)
 	      e->Throw( "Unable to create window.");
+        success = actDevice->CursorCrosshair();
+        success = actDevice->UnsetFocus();
+        //FIXME: ADD support for RETAIN (BackingSTORE))
 	    return;
 	  }
       }
@@ -141,6 +164,7 @@ namespace lib {
     // On the system I tested (Ubuntu 10.4), I was not able to have
     // the expected SHOW behavior, with IDL 7.0 and GDL :(
     // Help/suggestions welcome
+    // works for me (GD) on mandriva 2010.2
     bool show = true;
     if (nParam == 2) { 
       DIntGDL *showval = e->GetParAs<DIntGDL>(1);
@@ -151,12 +175,13 @@ namespace lib {
     // I don't know how to find the sub-window number (third parametre
     // in call XIconifyWindow())
     // Help/suggestions welcome
+    //GD: it is not a sub-window, but a screen number: xwd->screen, but that does not make window iconic any better!
 
     bool iconic = false;
     if( e->KeywordSet("ICONIC")) iconic=true;
 
     if (!actDevice->WShow( wIx, show, iconic)) 
-      e->Throw( "Window is closed and unavailable.");
+      e->Throw( "Window number "+i2s(wIx)+" out of range or no more windows.");
   }
 
   void wdelete( EnvT* e)
