@@ -30,6 +30,11 @@ enum RetCode {
   RC_ABORT, // checked as retCode >= RC_RETURN
 };
 
+class EnvT;
+typedef void     (*LibPro)(EnvT*);
+typedef BaseGDL* (*LibFun)(EnvT*);
+typedef BaseGDL* (*LibFunDirect)(BaseGDL* param,bool canGrab);
+
 class ProgNode;
 typedef ProgNode* ProgNodeP;
 
@@ -86,17 +91,19 @@ protected:
   ProgNodeP down;
   ProgNodeP right;
 
-  static void AdjustTypes(std::auto_ptr<BaseGDL>& a, 
-			  std::auto_ptr<BaseGDL>& b);
+  static void AdjustTypes(Guard<BaseGDL>& a, 
+			  Guard<BaseGDL>& b);
   // for overloaded operators
-  static void AdjustTypesObj(std::auto_ptr<BaseGDL>& a, 
-			  std::auto_ptr<BaseGDL>& b);
+  static void AdjustTypesObj(Guard<BaseGDL>& a, 
+			  Guard<BaseGDL>& b);
 
   BaseGDL*   cData;           // constant data
   DVar*      var;             // ptr to variable 
 
-  DLibFun*   libFun;
-  DLibPro*   libPro;
+  DLibFun*     libFun;
+  DLibPro*     libPro;
+  LibFun       libFunFun;
+  LibPro       libProPro;
 
   union {
     int        initInt;    // for c-i not actually used
@@ -114,9 +121,12 @@ protected:
   };
 
   void SetType( int tt, const std::string& txt) { ttype = tt; text = txt;} 
-
+  
   static ProgNodeP GetNULLProgNodeP(); 
 
+public:
+  int GetVarIx() const { return varIx;}
+
 private:
   // from DNode (see there)
   int lineNumber;
@@ -277,9 +287,10 @@ public:
       }
   }
 
-  int getType() { return ttype;}
+  int getType() const { return ttype;}
   void setType( int t) { ttype=t;}
-  std::string getText() { return text;}
+//   std::string getText() { return text;}
+  const std::string& getText() const { return text;}
   void setText(const std::string t) { text = t;}
   int getLine() const { return lineNumber;}
   void setLine( int l) { lineNumber = l;}
@@ -289,11 +300,11 @@ public:
 
   ProgNodeP BreakTarget() const { return breakTarget;}
   
-  bool LabelInRange( const int lIx)
-  {
-    // 	std::cout << "LabelInRange: " << ((lIx >= labelStart) && (lIx < labelEnd)) << "     " << lIx << "   [" << labelStart << "," << labelEnd << ")" << std::endl;
-    return (lIx >= labelStart) && (lIx < labelEnd);
-  }
+//   bool LabelInRange( int lIx) const
+//   {
+//     // 	std::cout << "LabelInRange: " << ((lIx >= labelStart) && (lIx < labelEnd)) << "     " << lIx << "   [" << labelStart << "," << labelEnd << ")" << std::endl;
+//     return (lIx >= labelStart) && (lIx < labelEnd);
+//   }
   
   friend class GDLInterpreter;
   friend class DInterpreter;
@@ -318,6 +329,7 @@ public:
   friend class ARRAYEXPR_FCALLNode;
   friend class EXPRNode;
   friend class SYSVARNode;
+  friend class DOTNode;
 };
 
 
@@ -1543,6 +1555,7 @@ public:
   BaseGDL** LExpr( BaseGDL* right);
   //   BaseGDL** LExprGrab( BaseGDL* right);
   BaseGDL* Eval();
+  BaseGDL** LEval();
 };
 class PCALL_LIBNode: public CommandNode
 {
@@ -1571,24 +1584,30 @@ public:
 class DECNode: public CommandNode
 { public:
   DECNode( const RefDNode& refNode): CommandNode( refNode){}
+  BaseGDL** EvalRefCheck( BaseGDL*& res);
+  BaseGDL** LEval();
   BaseGDL* Eval();
   RetCode Run();
 };
 class INCNode: public CommandNode
 { public:
   INCNode( const RefDNode& refNode): CommandNode( refNode){}
+  BaseGDL** EvalRefCheck( BaseGDL*& res);
+  BaseGDL** LEval();
   BaseGDL* Eval();
   RetCode Run();
 };
-class POSTDECNode: public DECNode
+class POSTDECNode: public CommandNode
 { public:
-  POSTDECNode( const RefDNode& refNode): DECNode( refNode){}
+  POSTDECNode( const RefDNode& refNode): CommandNode( refNode){}
   BaseGDL* Eval();
+//   BaseGDL** LEval();
 };
-class POSTINCNode: public INCNode
+class POSTINCNode: public CommandNode
 { public:
-  POSTINCNode( const RefDNode& refNode): INCNode( refNode){}
+  POSTINCNode( const RefDNode& refNode): CommandNode( refNode){}
   BaseGDL* Eval();
+//   BaseGDL** LEval();
 };
 
 class ARRAYEXPRNode: public DefaultNode
