@@ -19,6 +19,7 @@
 #define prognodeexpr_hpp__
 
 #include "prognode.hpp"
+#include "dpro.hpp"
 
 
 class UnaryExpr: public DefaultNode
@@ -58,8 +59,8 @@ protected:
 public:
   BinaryExprNC( const RefDNode& refNode);
 
-  void AdjustTypesNC( std::auto_ptr<BaseGDL>& g1, BaseGDL*& e1, 
-		      std::auto_ptr<BaseGDL>& g2, BaseGDL*& e2);
+  void AdjustTypesNC( Guard<BaseGDL>& g1, BaseGDL*& e1, 
+		      Guard<BaseGDL>& g2, BaseGDL*& e2);
   // for overloaded operators 
   void SetupGuards( Guard<BaseGDL>& g1, BaseGDL*& e1,
 		    Guard<BaseGDL>& g2, BaseGDL*& e2);
@@ -110,9 +111,13 @@ public:
 
 class FCALL_LIB_DIRECTNode: public LeafNode
 {
+  LibFunDirect libFunDirectFun;
 public:
   FCALL_LIB_DIRECTNode( const RefDNode& refNode): LeafNode( refNode)
-  {}
+  {
+    assert( this->libFun != NULL);
+    libFunDirectFun = static_cast<DLibFunDirect*>(this->libFun)->FunDirect();
+  }
   BaseGDL** LEval();
   BaseGDL* Eval();
 };
@@ -136,7 +141,7 @@ public:
   BaseGDL** EvalRefCheck( BaseGDL*& rEval);
   BaseGDL** LEval();
   BaseGDL* Eval();
-  BaseGDL* EvalFCALL_LIB(); // might return a non new value
+  BaseGDL* EvalFCALL_LIB( BaseGDL**& retValPtr); // might return a non new value
   BaseGDL** LExpr( BaseGDL* right);
 //   BaseGDL** LExprGrab( BaseGDL* right);  
 };
@@ -335,10 +340,23 @@ class NSTRUC_REFNode: public DefaultNode
 class QUESTIONNode: public TrinaryExpr
 { public:
   QUESTIONNode( const RefDNode& refNode): TrinaryExpr( refNode){}
+  ProgNodeP GetThisBranch(); // as parameter or keyword
   BaseGDL* Eval();
-  ProgNodeP AsParameter(); // as parameter or keyword
   BaseGDL** LExpr(BaseGDL* right);
+  BaseGDL** EvalRefCheck( BaseGDL*& rEval);
+
 //   BaseGDL** LExprGrab(BaseGDL* right);
+  ProgNodeP GetBranch() 
+  {
+    ProgNodeP branch = this->GetThisBranch();
+    while( branch->getType() == GDLTokenTypes::QUESTION)
+    {
+      QUESTIONNode* qRecursive = static_cast<QUESTIONNode*>( branch);
+      branch = qRecursive->GetThisBranch();
+    }
+    return branch;
+  }
+
 };
 class UMINUSNode: public UnaryExpr
 { public:
