@@ -45,7 +45,7 @@ namespace lib {
     treeParser.interactive(theAST);
     trAST = treeParser.getAST();
     ProgNodeP progAST = ProgNode::NewProgNode( trAST);
-    auto_ptr< ProgNode> progAST_guard( progAST);
+    Guard< ProgNode> progAST_guard( progAST);
 
     // necessary for correct FOR loop handling
     assert( dynamic_cast<EnvUDT*>(caller) != NULL);
@@ -63,6 +63,7 @@ namespace lib {
   {
 #ifndef HAVE_LIBWXWIDGETS
     e->Throw("GDL was compiled without support for wxWidgets");
+    return NULL; // avoid warning
 #else
     SizeT nParam = e->NParam();
 
@@ -216,12 +217,12 @@ namespace lib {
     e->AssureLongScalarKWIfPresent( y_scroll_sizeIx, y_scroll_size);
     
     BaseGDL* uvalue = e->GetKW( uvalueIx);
-    if( uvalue != NULL)
+    if( uvalue != NULL) {
       if( no_copy)
 	e->GetKW( uvalueIx) = NULL;
       else
 	uvalue = uvalue->Dup();
-    
+    }
     DString event_func = "";
     e->AssureStringScalarKWIfPresent( event_funcIx, event_func);
 
@@ -331,6 +332,7 @@ namespace lib {
   {
 #ifndef HAVE_LIBWXWIDGETS
     e->Throw("GDL was compiled without support for wxWidgets");
+    return NULL; // avoid warning
 #else
     DLongGDL* p0L = e->GetParAs<DLongGDL>( 0);
     WidgetIDT parentID = (*p0L)[0];
@@ -360,6 +362,7 @@ namespace lib {
 	{
 #ifndef HAVE_LIBWXWIDGETS
     e->Throw("GDL was compiled without support for wxWidgets");
+    return NULL; // avoid warning
 #else
 		//SizeT nParam = e->NParam();
 		DLongGDL* p0L = e->GetParAs<DLongGDL>( 0);
@@ -439,6 +442,7 @@ namespace lib {
 	{
 #ifndef HAVE_LIBWXWIDGETS
     e->Throw("GDL was compiled without support for wxWidgets");
+    return NULL; // avoid warning
 #else
     DLongGDL* p0L = e->GetParAs<DLongGDL>( 0);
     WidgetIDT parentID = (*p0L)[0];
@@ -480,6 +484,7 @@ namespace lib {
   {
 #ifndef HAVE_LIBWXWIDGETS
     e->Throw("GDL was compiled without support for wxWidgets");
+    return NULL; // avoid warning
 #else
     DLongGDL* p0L = e->GetParAs<DLongGDL>( 0);
     WidgetIDT parentID = (*p0L)[0];
@@ -522,6 +527,7 @@ namespace lib {
   {
 #ifndef HAVE_LIBWXWIDGETS
     e->Throw("GDL was compiled without support for wxWidgets");
+    return NULL; // avoid warning
 #else
     DLongGDL* p0L = e->GetParAs<DLongGDL>( 0);
     WidgetIDT parentID = (*p0L)[0];
@@ -560,6 +566,7 @@ namespace lib {
   {
 #ifndef HAVE_LIBWXWIDGETS
     e->Throw("GDL was compiled without support for wxWidgets");
+    return NULL; // avoid warning
 #else
     DLongGDL* p0L = e->GetParAs<DLongGDL>( 0);
     WidgetIDT parentID = (*p0L)[0];
@@ -593,6 +600,7 @@ namespace lib {
   {
 #ifndef HAVE_LIBWXWIDGETS
     e->Throw("GDL was compiled without support for wxWidgets");
+    return NULL;
 #else
     SizeT nParam=e->NParam();
 
@@ -747,6 +755,7 @@ namespace lib {
   {
 #ifndef HAVE_LIBWXWIDGETS
     e->Throw("GDL was compiled without support for wxWidgets");
+    return NULL;
 #else
     SizeT nParam=e->NParam();
 
@@ -822,7 +831,7 @@ namespace lib {
       trAST = treeParser.getAST();
 
       ProgNodeP progAST = ProgNode::NewProgNode( trAST);
-      auto_ptr< ProgNode> progAST_guard( progAST);
+      Guard< ProgNode> progAST_guard( progAST);
 
       // necessary for correct FOR loop handling
       assert( dynamic_cast<EnvUDT*>(caller) != NULL);
@@ -1069,7 +1078,7 @@ namespace lib {
 	callF = StrUpCase( callF);
 
 	SizeT funIx = GDLInterpreter::GetFunIx( callF);
-	EnvUDT* newEnv= new EnvUDT( e->CallingNode(), funList[ funIx], (BaseGDL**)NULL);
+	EnvUDT* newEnv= new EnvUDT( e->CallingNode(), funList[ funIx], (DObjGDL**)NULL);
 
 	// add parameter
 	newEnv->SetNextPar( new DLongGDL(widgetID)); // pass as local
